// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "indexingdialog.h"
#include "ui_indexingdialog.h"
#include "Dialogs/indexwindow.h"
#include "Dialogs/tablepreview.h"
#include "Dialogs/messagelogwindow.h"
#include "Dialogs/mainwindow.h"
#include "OtherWssi/initialpos.h"
#include "OtherWssi/wssiproto.h"
#include "3rdparty/qsvgaspect.h"

#include <QString>
#include <QSettings>
#include <QtSvg/QSvgWidget>

// This routine shows the animation during indexing

IndexingDialog::IndexingDialog(QWidget *parent, MessageLogWindow *ml, struct diskrec *currentVrec, enum disktypes dtype)
	: QMainWindow(parent), id(new Ui::IndexingDialog)
	{
	animationType = dtype;
	id->setupUi(this);
	id->label_current_vol->setText(tr("Label = ") + currentVrec->VolName);
	this->setWindowFlags(MyWindowFlags);
	restoreSettings();
	iw = static_cast<IndexWindow *>(parent);
	connect(id->pushButton_stop, SIGNAL(clicked()),
			  this, SLOT(slot_pushbuttonStopClicked()));
	connect(iw, SIGNAL(signal_indexIsDone()),
			  this, SLOT(slot_indexIsDone()));
	if(MessageLogWindowOpen)
		{
		connect(this, SIGNAL(signal_keyToMl(int,int)),
			ml, SLOT(slot_keyToMl(int,int)));
		connect(this, SIGNAL(signal_wheelToMl(QPoint,int,Qt::MouseButtons,Qt::KeyboardModifiers,Qt::Orientation)),
			ml, SLOT(slot_wheelToMl(QPoint,int,Qt::MouseButtons,Qt::KeyboardModifiers,Qt::Orientation)));
		}
	armAngle = HD_ARM_ANGLE_MIN;
	diskAngle = cdAngle = sliderPos = netColor = 0;
	netSkip = true;
	usbColor = 127;
	run_images_backwards = false;
	svgWidget = new QSvgAspect();
	svgWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	id->gridLayout->addWidget(svgWidget, 2, 0, 1, -1);

	QFile svgfile;
	switch(dtype)
		{
		case WSS_FLOPPY:
			svgWidget->setSvgAspectRatio(570./590.);
			svgfile.setFileName(":/Animation/3_5_floppy_diskette.svg");
			break;
		case WSS_CDROM:
			svgWidget->setSvgAspectRatio(1.0);
			svgfile.setFileName(":/Animation/CDrom.svg");
			break;
		case WSS_RAMDISK:
		case WSS_HARD:
		case WSS_MOUNTED:
			svgWidget->setSvgAspectRatio(62./43.);
			svgfile.setFileName(":/Animation/DiskAnimation.svg");
			break;
		case WSS_NETWORK:
			svgWidget->setSvgAspectRatio(1.0);
			svgfile.setFileName(":/Animation/network.svg");
			break;
		case WSS_USB:
			svgWidget->setSvgAspectRatio(103./40.);
			svgfile.setFileName(":/Animation/USBkey.svg");
			break;
		}
	svgfile.open(QIODevice::ReadOnly);
	QTextStream in(&svgfile);
	svgAnimation = in.readAll();
	svgfile.close();
	timer.start(ANIMATION_MILLISEC, this);
	}

IndexingDialog::~IndexingDialog()
	{
	iw->Stop_indexing = true;	// In case closed by X instead of stop button
	delete id;
	}

void IndexingDialog::saveSettings()
	{
	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	settings.setValue("IndexMsgGeometryPos", pos());
	settings.setValue("IndexMsgGeometrySize", size());
	}

void IndexingDialog::restoreSettings()
	{
	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	if(settings.contains("IndexMsgGeometrySize"))
		{
		QSize size = settings.value("IndexMsgGeometrySize").toSize();
		resize(size);
		}
	if(settings.contains("IndexMsgGeometryPos"))
		{
		QPoint pos = settings.value("IndexMsgGeometryPos").toPoint();
		move(pos);
		forceOnscreen(this);
		}
	else
		{
		this->show();
		move(mainWindowPtr->offCenter(this->frameGeometry(), INDEXSTATUS_OFFSET));
		}
	}

void IndexingDialog::slot_pushbuttonStopClicked()
	{
	iw->Stop_indexing = true;
	saveSettings();
	this->close();
	}

void IndexingDialog::slot_indexIsDone()
	{
	saveSettings();
	this->close();
	}

void IndexingDialog::timerEvent(QTimerEvent *event)
	{
	QStringList colors;
	colors << QString("red") << QString("green") << QString("blue") << QString("red") << QString("green");
	if(event->timerId() != timer.timerId())
		{
		QWidget::timerEvent(event);
		return;
		}
	switch(animationType)
		{
		case WSS_CDROM:
			cdAngle = (cdAngle + CD_ANGLE_STEP) % 360;
			svgWidget->load((svgAnimation).arg(cdAngle).toLocal8Bit());
			break;

		case WSS_FLOPPY:
			withingRange(&sliderPos, &run_images_backwards, FLOPPY_SLIDER_MIN, FLOPPY_SLIDER_MAX, FLOPPY_SLIDER_STEP);
			svgWidget->load((svgAnimation.arg(sliderPos)).toLocal8Bit());
			break;

		case WSS_NETWORK:
			netSkip = !netSkip;		// Run this animation at half the rate of the others
			if(netSkip)
				break;
			netColor = (netColor + 1) % 3;
			svgWidget->load((svgAnimation.arg(colors[netColor+2], colors[netColor+1], colors[netColor])).toLocal8Bit());
			break;

		case WSS_RAMDISK:
		case WSS_MOUNTED:
		case WSS_HARD:
			diskAngle = (diskAngle + HD_ANGLE_STEP) % 60;
			withingRange(&armAngle, &run_images_backwards, HD_ARM_ANGLE_MIN, HD_ARM_ANGLE_MAX, HD_ARM_ANGLE_STEP);
			svgWidget->load((svgAnimation.arg(armAngle).arg(diskAngle)).toLocal8Bit());
			break;

		case WSS_USB:
			withingRange(&usbColor, &run_images_backwards, USB_COLOR_MIN, USB_COLOR_MAX, USB_COLOR_STEP);
			svgWidget->load((svgAnimation.arg(255).arg(128+usbColor).arg(2*usbColor)).toLocal8Bit());
			break;
		}
	this->repaint();
	}

void IndexingDialog::withingRange(int *value, bool *backwards, int smallest, int biggest, int step)
	{
	if(*backwards)
		{
		*value -= step;
		if(*value <= smallest)
			{
			*value = smallest;
			*backwards = false;
			}
		}
	else
		{
		*value += step;
		if(*value >= biggest)
			{
			*value = biggest;
			*backwards = true;
			}
		}
	}

void IndexingDialog::keyPressEvent(QKeyEvent *event)
	{
	switch(event->key())
		{
		case Qt::Key_Home:
		case Qt::Key_End:
			if(!(Qt::ControlModifier & event->modifiers()))
				break;
			Q_FALLTHROUGH();
		case Qt::Key_PageDown:
		case Qt::Key_PageUp:
			Q_EMIT signal_keyToMl(event->key(), static_cast<int>(event->modifiers()));
			break;
		case Qt::Key_Escape:
		case Qt::Key_Enter:
		case Qt::Key_Return:
			slot_pushbuttonStopClicked();
			break;
		case Qt::Key_Tab:
			iw->raise();
			break;
		default:
			return;
		}
	event->ignore();
	return;
	}

void IndexingDialog::wheelEvent(QWheelEvent *event)
	{
	Q_EMIT signal_wheelToMl(event->pos(), event->delta(), event->buttons(),
										event->modifiers(), event->orientation());
	event->ignore();
	return;
	}
