// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "Dialogs/indextimefilter.h"
#include "ui_indextimefilter.h"
#include "OtherWssi/wssindex.h"
#include "OtherWssi/wssiproto.h"
#include "OtherWssi/globalsettings.h"
#include "Dialogs/indexwindow.h"
#include "Dialogs/mainwindow.h"

#include <QSettings>
#include <QFileDialog>

IndexTimeFilter::IndexTimeFilter(QWidget *parent)
	: QMainWindow(parent), ui(new Ui::IndexTimeFilter)
	{
	ui->setupUi(this);
	this->setWindowFlags(MyWindowFlags);
	restoreSettings();
	connect(ui->pushButton_accept, SIGNAL(clicked()), this, SLOT(slot_acceptClicked()));
	connect(ui->pushButton_saveAndAccept, SIGNAL(clicked()), this, SLOT(slot_saveAndAcceptClicked()));
	connect(ui->pushButton_cancel, SIGNAL(clicked()), this, SLOT(slot_cancelClicked()));
	connect(ui->pushButton_clear_dir_exclude, SIGNAL(clicked()), this, SLOT(slot_clearDirExcludeClicked()));
	connect(ui->pushButton_clear_dir_include, SIGNAL(clicked()), this, SLOT(slot_clearDirIncludeClicked()));
	connect(ui->pushButton_clear_file_exclude, SIGNAL(clicked()), this, SLOT(slot_clearFileExcludeClicked()));
	connect(ui->pushButton_clear_file_include, SIGNAL(clicked()), this, SLOT(slot_clearFileIncludeClicked()));
	connect(ui->pushButton_sortall, SIGNAL(clicked()), this, SLOT(slot_sortallClicked()));
	connect(ui->pushButton_browse_dir_exclude, SIGNAL(clicked()), this, SLOT(slot_buttonBrowseDirExcludeClicked()));
	connect(ui->pushButton_browse_dir_include, SIGNAL(clicked()), this, SLOT(slot_buttonBrowseDirIncludeClicked()));

	ui->checkBox_exclude_dir->setChecked(IndexTimeDirExclude);
	ui->checkBox_include_dir->setChecked(IndexTimeDirInclude);
	ui->checkBox_exclude_file->setChecked(IndexTimeFileExclude);
	ui->checkBox_include_file->setChecked(IndexTimeFileInclude);

	ui->plainTextEdit_dir_exclude->setPlainText(QDir::toNativeSeparators(IndexDirExcludeList.join(QFA("\n"))));
	ui->plainTextEdit_dir_include->setPlainText(QDir::toNativeSeparators(IndexDirIncludeList.join(QFA("\n"))));
	ui->plainTextEdit_file_exclude->setPlainText(QDir::toNativeSeparators(IndexFileExcludeList.join(QFA("\n"))));
	ui->plainTextEdit_file_include->setPlainText(QDir::toNativeSeparators(IndexFileIncludeList.join(QFA("\n"))));
	ui->plainTextEdit_file_exclude->setFocus();
	}

IndexTimeFilter::~IndexTimeFilter()
	{
	delete ui;
	}

void IndexTimeFilter::saveSettings()
	{
	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	settings.setValue("IndexFilterGeometryPos", pos());
	settings.setValue("IndexFilterGeometrySize", size());
	setFilterLists();
	(static_cast<IndexWindow *>(parentWidget()))->saveIndexTimeFilter();
	}

void IndexTimeFilter::setFilterLists()
	{
	IndexFileExcludeList = QDir::fromNativeSeparators(ui->plainTextEdit_file_exclude->toPlainText()).split(QFA("\n"));
	IndexFileIncludeList = QDir::fromNativeSeparators(ui->plainTextEdit_file_include->toPlainText()).split(QFA("\n"));
	IndexDirExcludeList = QDir::fromNativeSeparators(ui->plainTextEdit_dir_exclude->toPlainText()).split(QFA("\n"));
	IndexDirIncludeList = QDir::fromNativeSeparators(ui->plainTextEdit_dir_include->toPlainText()).split(QFA("\n"));
	}

void IndexTimeFilter::restoreSettings()
	{
	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	if(settings.contains("IndexFilterGeometryPos"))
		{
		QPoint pos = settings.value("IndexFilterGeometryPos").toPoint();
		move(pos);
		forceOnscreen(this);
		}
	if(settings.contains("IndexFilterGeometrySize"))
		{
		QSize size = settings.value("IndexFilterGeometrySize").toSize();
		resize(size);
		}
	}

void IndexTimeFilter::sortlist(QPlainTextEdit *list)
	{
	QString content = list->toPlainText();
#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
	QStringList contentList = content.split(QFA("\n"), QString::SkipEmptyParts);
#else
	QStringList contentList = content.split(QFA("\n"), Qt::SkipEmptyParts);
#endif
	contentList.sort();
	contentList.removeDuplicates();
	list->setPlainText(QDir::toNativeSeparators(contentList.join(QFA("\n"))));
	}

void IndexTimeFilter::slot_sortallClicked()
	{
	sortlist(ui->plainTextEdit_file_exclude);
	sortlist(ui->plainTextEdit_file_include);
	cleanupDirList(ui->plainTextEdit_dir_exclude);
	sortlist(ui->plainTextEdit_dir_exclude);
	cleanupDirList(ui->plainTextEdit_dir_include);
	sortlist(ui->plainTextEdit_dir_include);
	}

void IndexTimeFilter::slot_clearDirExcludeClicked()
	{
	ui->plainTextEdit_dir_exclude->clear();
	}

void IndexTimeFilter::slot_clearDirIncludeClicked()
	{
	ui->plainTextEdit_dir_include->clear();
	}

void IndexTimeFilter::slot_clearFileExcludeClicked()
	{
	ui->plainTextEdit_file_exclude->clear();
	}

void IndexTimeFilter::slot_clearFileIncludeClicked()
	{
	ui->plainTextEdit_file_include->clear();
	}

void IndexTimeFilter::slot_cancelClicked()
	{
	close();
	}

void IndexTimeFilter::slot_saveAndAcceptClicked()
	{
	updateCheckboxStatus();
// saveSettings() calls setFilterLists(), no need to do it here
	saveSettings();
	close();
	}

void IndexTimeFilter::slot_acceptClicked()
	{
	updateCheckboxStatus();
	setFilterLists();
	close();
	}

void IndexTimeFilter::updateCheckboxStatus()
	{
	IndexTimeDirExclude = ui->checkBox_exclude_dir->isChecked();
	IndexTimeDirInclude = ui->checkBox_include_dir->isChecked();
	IndexTimeFileExclude = ui->checkBox_exclude_file->isChecked();
	IndexTimeFileInclude = ui->checkBox_include_file->isChecked();
	slot_sortallClicked();
	}

void IndexTimeFilter::slot_buttonBrowseDirExcludeClicked()
	{
	QString path = QFileDialog::getExistingDirectory(this,
										 tr("Directory to exclude"), tr("/"));
	if (!path.isEmpty())
		{
		ui->plainTextEdit_dir_exclude->appendPlainText(QFA("\n")+path.mid(2));
		cleanupDirList(ui->plainTextEdit_dir_exclude);
		sortlist(ui->plainTextEdit_dir_exclude);
		}
	}

void IndexTimeFilter::slot_buttonBrowseDirIncludeClicked()
	{
	QString path = QFileDialog::getExistingDirectory(this,
										 tr("Directory to include"), tr("/"));
	if (!path.isEmpty())
		{
		ui->plainTextEdit_dir_include->appendPlainText(QFA("\n")+path.mid(2));
		cleanupDirList(ui->plainTextEdit_dir_include);
		sortlist(ui->plainTextEdit_dir_include);
		}
	}

// Cleaup directory in/exclude entries
// Change from native separators
// // goes to /
// Force leading /
// Force trailing / if * wildcard not found
void IndexTimeFilter::cleanupDirList(QPlainTextEdit *list)
	{
	QStringList tlist = list->toPlainText().split("\n");
	list->clear();
	for(int i=0; i<tlist.size(); i++)
		{if(0 == tlist[i].size())
			continue;
		tlist[i] = QDir::fromNativeSeparators(tlist[i]);
		tlist[i].replace("//", "/");
		if(!tlist[i].startsWith("/"))
			tlist[i] = "/"+ tlist[i];
		if(!tlist[i].endsWith("/") && !tlist[i].endsWith("*"))
			tlist[i] = tlist[i] + "/";
		list->appendPlainText(tlist[i] + "\n");
		}
	}
