// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "Dialogs/mainwindow.h"
#include "OtherWssi/tablelayout.h"

#include <QPushButton>

// ---- Csv export
void MainWindow::slot_csvExportClicked()
	{
	if(0 == Noselect)
		return;
	QFileDialog *fdialog = new QFileDialog(this, tr("Csv export file"));
	fdialog->setViewMode(QFileDialog::Detail);
	fdialog->setFileMode(QFileDialog::AnyFile);
	fdialog->setNameFilter(tr("Comma-separated values (*.csv)"));
	fdialog->setMinimumWidth(750);
	fdialog->setLabelText(QFileDialog::Accept, tr("Export"));

	if(!fdialog->exec())
		{
		delete fdialog;
		return;
		}
	QFile f(fdialog->selectedFiles().at(0));
	delete fdialog;
	QIODevice::OpenMode openmode = QIODevice::WriteOnly | QIODevice::Text;
	bool export_fieldnames = true;
	if(f.exists())
		{
		QMessageBox msgBox;
		msgBox.setText(tr("<html><head/><body><p>File exists.  Append, overwrite or cancel?<p>"
		"Appending only makes sense if the current field list matches the existing csv file.</p></body></html>"));
		QPushButton *appendButton = msgBox.addButton(tr("Append"), QMessageBox::ActionRole);
		QPushButton *overwriteButton = msgBox.addButton(tr("Overwrite"), QMessageBox::ActionRole);
		msgBox.addButton(QMessageBox::Cancel);
		msgBox.exec();

		if(msgBox.clickedButton() == appendButton)
			{
			openmode |= QIODevice::Append;
			export_fieldnames = false;
			}
		else if (msgBox.clickedButton() == overwriteButton)
			openmode |= QIODevice::Truncate;
		else
			return;
		}
	else
		{
		if(!f.fileName().endsWith(".csv",Qt::CaseInsensitive))
			f.setFileName(f.fileName() + ".csv");
		}

	if(!f.open(openmode))
		{
		QMessageBox::warning(this, tr("Error"), tr("Open failed"));
		return;
		}
	QTextStream csvout(&f);

// First line has field names unless appending
	if(export_fieldnames)
		{
		bool need_comma=false;
		for(int i=0; i<NFIELDS; i++)
			{
			if(fieldlayout[i].enabled)
				{
				if(need_comma)
					csvout << ",";
				csvout << "\"" << fieldlayout[i].fieldname << "\"";
				need_comma = true;
				}
			}
		csvout << endl;
		}

	for(unsigned long i=0; i<Noselect; i++)
		{
		unsigned long arcbase;
		bool inarc;
		unsigned long k = Sortindx[i];
		struct filerec *dr_k = Dr + k;
		unsigned long volno = dr_k->Volno;
		struct diskrec *vrec_volno = Vrec + volno;
		if(inArc(dr_k))
			{
			inarc = true;
			arcbase = dr_k->Subdirno;
			}
		else
			{
			inarc = false;
			arcbase = k;
			}
		struct filerec *dr_arcbase = Dr + arcbase;
		bool need_comma = false;

// Long file name
		if(fieldlayout[0].enabled)
			{
			if(isDotID(dr_k))
				if(0 == dr_k->Subdirno)
					quoteout("<Volume>", &csvout, &need_comma);
				else
					quoteout("<Subdir>", &csvout, &need_comma);
			else
				quoteout(dr_k->Lfn, &csvout, &need_comma);
			}

// Extension
		if(fieldlayout[1].enabled)
			{
			if(!isDotID(dr_k))
				quoteout(dr_k->Lfn.mid(dr_k->ExtOffset), &csvout, &need_comma);
			}

// Volume
		if(fieldlayout[2].enabled)
			quoteout(vrec_volno->VolName, &csvout, &need_comma);

// Subdirectory
		if(fieldlayout[3].enabled)
			{
			if(!isDotID(dr_k) || dr_k->Subdirno > 0)
				quoteout(SubDir[dr_arcbase->Subdirno], &csvout, &need_comma);
			else
				quoteout("", &csvout, &need_comma);
			}

// Archive
		if(fieldlayout[4].enabled)
			{
			QString arcptr;
			if(inarc)
				arcptr = dr_arcbase->Lfn;
			else
				arcptr = QString("");
			quoteout(arcptr, &csvout, &need_comma);
			}

// Size
		if(fieldlayout[5].enabled)
			{
			if(need_comma)
				csvout << ",";
			csvout << "\"";
			if(!isDotID(dr_k))
				csvout << dr_k->Dsize;
			csvout << "\"";
			need_comma = true;
			}

// Timestamp
		if(fieldlayout[6].enabled)
			{
			if(need_comma)
				csvout << ",";
			csvout << "\"";
			if(dr_k->Ddate.isValid())
				{
				csvout << dr_k->Ddate.toString(ShortDateFormat)
					<< " " << dr_k->Dtime.toString();
				}
			csvout << "\"";
			need_comma = true;
			}

// Category
		if(fieldlayout[7].enabled)
			quoteout(dr_k->Category, &csvout, &need_comma);

// Comment
		if(fieldlayout[8].enabled)
			quoteout(dr_k->Comment, &csvout, &need_comma);

// Extra info
		if(fieldlayout[9].enabled)
			{
			quoteout(dr_k->ExtraInfo, &csvout, &need_comma);
			}
		csvout << endl;
		}
	f.close();
	}
