// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "Dialogs/mainwindow.h"
#include "ui_mainwindow.h"
#include "OtherWssi/globalsettings.h"
#include "3rdparty/helpmenustyle.h"
#include "3rdparty/windowsmodernstyle.h"

#include <QMenu>
#include <QMenuBar>
#include <QSpinBox>

QAction *MainWindow::addIconAction(QToolBar *whichBar, QIcon icon, QString tooltip, const char *method)
	{
	QAction *toolAction = new QAction(icon, tooltip, this);
	whichBar->addAction(toolAction);
	connect(toolAction, SIGNAL(triggered()), this, method);
	return toolAction;
	}

QAction *MainWindow::addMenuItem(QMenu *whichMenu, QString menuText, const char *method, bool queue)
	{
	QAction *openAction = new QAction(menuText, this);
	if(queue)
		connect(openAction, SIGNAL(triggered()), this, method, Qt::QueuedConnection);
	else
		connect(openAction, SIGNAL(triggered()), this, method);
	whichMenu->addAction(openAction);
	return openAction;
	}

// This adds a menu item with no subitems to a menu bar.  Clicking on the item
// performs an action, it doesn't open up a menu.
QAction *MainWindow::addTopLevelMenuButton(QMenuBar *whichBar, QString menuText, const char *method)
	{
	QAction *buttonAction = new QAction(menuText, this);
	whichBar->addAction(buttonAction);
	connect(buttonAction, SIGNAL(triggered()), this, method);
	return buttonAction;
	}

// Paths are saved with '/' as separators.
// Display routine converts to native separators
void MainWindow::addtoRecent(QString *recent, bool *pinstate, QString newitem, int maxcount)
	{
	QString unixitem = QDir::fromNativeSeparators(newitem);
// Is this already in the list?  If so, move to top but pinned items must
// remain above unpinned.
	for(int i=0; i<maxcount; i++)
		{
		if(recent[i].isEmpty())
			break;
		if(0 == unixitem.compare(recent[i], Qt::CaseInsensitive))
			{
			int insertionsub = 0;
			for(int j=i; j>0; j--)
				{
				if(pinstate[j] == pinstate[j-1])
					recent[j] = recent[j-1];
				else
					{
					insertionsub = j;
					break;
					}
				}
			recent[insertionsub] = unixitem;
			return;
			}
		}
// It's new (therefore unpinned, possibly no room if everything is pinned)
	if(!recent[maxcount-1].isEmpty() && pinstate[maxcount-1])
		return;
	int insertionsub = maxcount - 1;
	for(int i=maxcount-1; i>0; i--)
		{
		if(recent[i].isEmpty())
			{
			insertionsub = i;
			continue;
			}
		if(pinstate[i])
			break;
		insertionsub = i;
		if(i < maxcount-2)
			recent[i+1] = recent[i];
		}
	recent[insertionsub] = unixitem;
	pinstate[insertionsub] = false;
	}

void MainWindow::createMenus()
	{
	if(!UseModernStyle)
		menuBar()->setStyle(menuBarStyle);

// File menu
	fileMenu = menuBar()->addMenu(tr("&File"));
	fileMenu->setStyleSheet(MENU_NO_CHECKBOX);
	fileMenu->setToolTipsVisible(true);
// Open, close, save, csv export, [separator], exit on file menu
	addMenuItem(fileMenu, tr("&Open"), SLOT(slot_openClicked()), false);
	openRecentFileMenu = fileMenu->addMenu(tr("    open a &recently used database"));
// This doesn't do anything, it was supposed to reduce space to left of menu text
//	openRecentFileMenu->setStyleSheet(MENU_NO_CHECKBOX);
	recentFileOpenActs.clear();
	for(int i=0; i<NrecentFiles; i++)
		{
		recentFileOpenActs.append(new PinMenuItem(fileMenu, i));
		openRecentFileMenu->addAction(recentFileOpenActs[i]);
		connect(recentFileOpenActs[i], SIGNAL(signal_textClicked(int)), this, SLOT(slot_openRecentFileClicked(int)));
		connect(recentFileOpenActs[i], SIGNAL(signal_pinClicked(int)), this, SLOT(slot_pinRecentFileClicked(int)));
		}
	connect(openRecentFileMenu, SIGNAL(aboutToShow()), this, SLOT(slot_recentFileMenuOpened()));
	openRecentDirMenu = fileMenu->addMenu(tr("    open &from recently used directory"));
	recentDirOpenActs.clear();
	for(int i=0; i<NrecentDirs; i++)
		{
		recentDirOpenActs.append(new PinMenuItem(fileMenu, i));
		openRecentDirMenu->addAction(recentDirOpenActs[i]);
		connect(recentDirOpenActs[i], SIGNAL(signal_textClicked(int)), this, SLOT(slot_openRecentDirClicked(int)));
		connect(recentDirOpenActs[i], SIGNAL(signal_pinClicked(int)), this, SLOT(slot_pinRecentDirClicked(int)));
		}
	connect(openRecentDirMenu, SIGNAL(aboutToShow()), this, SLOT(slot_recentDirMenuOpened()));
	addMenuItem(fileMenu, tr("&Close"), SLOT(slot_closeDbClicked()), false);
	addMenuItem(fileMenu, tr("&Save"), SLOT(slot_saveClicked()), false);
	saveRecentDirMenu = fileMenu->addMenu(tr("    save &to recently used directory"));
	recentDirSaveActs.clear();
	for(int i=0; i<NrecentDirs; i++)
		{
		recentDirSaveActs.append(new PinMenuItem(fileMenu, i));
		saveRecentDirMenu->addAction(recentDirSaveActs[i]);
		connect(recentDirSaveActs[i], SIGNAL(signal_textClicked(int)), this, SLOT(slot_saveRecentDirClicked(int)));
		connect(recentDirSaveActs[i], SIGNAL(signal_pinClicked(int)), this, SLOT(slot_pinRecentDirClicked(int)));
		}
	connect(saveRecentDirMenu, SIGNAL(aboutToShow()), this, SLOT(slot_saveRecentDirMenuOpened()));
	addMenuItem(fileMenu, tr("Cs&v export"), SLOT(slot_csvExportClicked()), false);
	addMenuItem(fileMenu, tr("File names (possibly unicode) to clip&board"), SLOT(slot_filenameClipboardClicked()), true);
	fileMenu->addSeparator();
	QAction *exitMenuItem = addMenuItem(fileMenu, tr("E&xit"), SLOT(slot_closeClicked()), false);
	exitMenuItem->setToolTip(tr("<html><head/><body><p>Any settings changes made from top-level menus will be saved.</p></body></html>"));

// Settings menu
	addTopLevelMenuButton(ui->menuBar, tr("S&ettings"), SLOT(slot_settingsClicked()));

// Table design menu
	tDesignAction = addTopLevelMenuButton(ui->menuBar, tr("Table &design"), SLOT(slot_pushButtonDesignClicked()));

//	Font menu
	addTopLevelMenuButton(ui->menuBar, tr("F&ont"), SLOT(slot_pickFont()));

// Style menu
	QMenu *pickColors = menuBar()->addMenu(tr("St&yle"));
// Alternate row colors checkbox
	useAltColorsAction = addMenuItem(pickColors, tr("&Use alternate row colors"), SLOT(slot_altColorsToggled()), false);
		useAltColorsAction->setCheckable(true);
		useAltColorsAction->setChecked(FileUseAltColors);
// First row color, alternate row color, default skin (style), Windows modern skin, Change skin
	addMenuItem(pickColors, tr("&First row color"), SLOT(slot_evenColorClicked()), false);
	addMenuItem(pickColors, tr("&Alternate row color"), SLOT(slot_oddColorClicked()), false);
	addMenuItem(pickColors, tr("&Gridcolor"), SLOT(slot_GridColorClicked()), false);
	useDefaultStyleAction = addMenuItem(pickColors, tr("&Default skin"), SLOT(slot_skinToggled()), false);
		useDefaultStyleAction->setCheckable(true);
		useDefaultStyleAction->setChecked(!UseModernStyle);
	useModernStyleAction = addMenuItem(pickColors, tr("&Windows skin"), SLOT(slot_skinToggled()), false);
		useModernStyleAction->setCheckable(true);
		useModernStyleAction->setChecked(UseModernStyle);
	addMenuItem(pickColors, tr("&Pick skin"), SLOT(slot_pickSkin()), false);

// Query menu
	queryMenu = menuBar()->addMenu(tr("&Query"));
	queryMenu->setToolTipsVisible(true);
//	queryMenu->setStyleSheet(MENU_NO_CHECKBOX);
// Show all on query menu
	QAction *showAllMenu = addMenuItem(queryMenu, tr("Show a&ll"), SLOT(slot_showallClicked()), true);
		showAllMenu->setToolTip(tr("<html><head/><body><p>The Hide vol/subdir records checkbox affects what records are displayed.</p></body></html>"));
// Simple query on query menu
	addMenuItem(queryMenu, tr("&Simple Query (^F or ^S)"), SLOT(slot_pushButtonQueryClicked()), true);
	menuHideCheckbox = addMenuItem(queryMenu, tr("&Hide vol/subdir records"), SLOT(slot_pushButtonKillIdClicked()), true);
		menuHideCheckbox->setCheckable(true);
		menuHideCheckbox->setChecked(NoId);
		menuHideCheckbox->setToolTip(QFA("<html><head/><body><p>Checking Hide vol/subdir records takes effect immediately, unchecking affects the next query.</p></body></html>"));
	addMenuItem(queryMenu, tr("&Invert selection"), SLOT(slot_pushButtonInvertClicked()), true);
	addMenuItem(queryMenu, tr("&Archives Queries"), SLOT(slot_pushButtonArchivesClicked()), true);
	addMenuItem(queryMenu, tr("&Filetype Queries"), SLOT(slot_pushButtonFiletypeClicked()), true);
	addMenuItem(queryMenu, tr("&Names Queries"), SLOT(slot_pushButtonNamesClicked()), true);
	addMenuItem(queryMenu, tr("&Range Queries"), SLOT(slot_pushButtonRangeClicked()), true);
	addMenuItem(queryMenu, tr("S&pecial Queries"), SLOT(slot_pushButtonSpecialClicked()), true);
// All queries on query menu
	QAction *allQueryAction = addMenuItem(queryMenu, tr("All &Queries"), SLOT(slot_pushButtonQueryClicked()), true);
		connect(allQueryAction, SIGNAL(triggered()),
			  this, SLOT(slot_pushButtonArchivesClicked()), Qt::QueuedConnection);
		connect(allQueryAction, SIGNAL(triggered()),
			  this, SLOT(slot_pushButtonFiletypeClicked()), Qt::QueuedConnection);
		connect(allQueryAction, SIGNAL(triggered()),
			  this, SLOT(slot_pushButtonNamesClicked()), Qt::QueuedConnection);
		connect(allQueryAction, SIGNAL(triggered()),
			  this, SLOT(slot_pushButtonRangeClicked()), Qt::QueuedConnection);
		connect(allQueryAction, SIGNAL(triggered()),
			  this, SLOT(slot_pushButtonSpecialClicked()), Qt::QueuedConnection);
// Close all queries on query menu
	QAction *closeQueriesAction = addMenuItem(queryMenu, tr("Close all query &windows"), SLOT(slot_closeQueries()), false);
		closeQueriesAction->setStatusTip(tr("Dialog positions not saved"));

// Sort menu
	sortMenu = menuBar()->addMenu(tr("&Sort"));
	sortMenu->setStyleSheet(MENU_NO_CHECKBOX);
// Simple sort on sort menu
	addMenuItem(sortMenu, tr("&Simple sorts"), SLOT(slot_pushButtonSortClicked()), true);
// Arbitrary sorts on sort menu
	addMenuItem(sortMenu, tr("&Custom sorts"), SLOT(slot_pushButtonFullSortClicked()), true);
// All sorts on sort menu
	QAction *allSortAction = addMenuItem(sortMenu, tr("&All Sort options"), SLOT(slot_pushButtonSortClicked()), true);
		connect(allSortAction, SIGNAL(triggered()),
			  this, SLOT(slot_pushButtonFullSortClicked()), Qt::QueuedConnection);
// Close all sorts on sort menu
	QAction *closeSortsAction = addMenuItem(sortMenu, tr("Close all sort &windows"), SLOT(slot_closeSorts()), false);
		closeSortsAction->setStatusTip(tr("Dialog positions not saved"));

// Index menu
	addTopLevelMenuButton(ui->menuBar, tr("&Index"), SLOT(slot_openIndex()));

// Volume list
	openVollist = addTopLevelMenuButton(ui->menuBar, tr("&Volumes"), SLOT(slot_openVolumes()));

// addSeparator doesn't do anything in Windows unless the style is overridden
// (which we do in the Style class and have added to windowsmodernstyle).
	helpRightSeparator = ui->menuBar->addSeparator();
	helpRightSeparator->setVisible(HelpMenuRight);
// Help menu
	QMenu *helpMenu = menuBar()->addMenu(tr("&Help"));
	helpMenu->setStyleSheet(MENU_NO_CHECKBOX);
// About
	addMenuItem(helpMenu, tr("&About"), SLOT(slot_about()), false);
// About Qt
// Can't use addMenuItem() here because slot is in qApp, not this
	QAction *aboutQtAction = new QAction(tr("About &Qt"), this);
	connect(aboutQtAction, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
	helpMenu->addAction(aboutQtAction);
// Welcome
	addMenuItem(helpMenu, tr("&Welcome and tips"), SLOT(slot_welcome()), false);
// Version check
	addMenuItem(helpMenu, tr("&Check for new version"), SLOT(slot_pushButtonVersioncheckClicked()), true);
	}

void MainWindow::createToolbars()
	{
	ui->toolBar->setStyleSheet(toolbarStyle);
	ui->toolBar->setMovable(false);
// Toolbar items
// File open (toolbar)
	fileOpenPair  = new raise2(this, ui->toolBar, QIcon(tr(":/Images/fileopen.png")),
		tr("Open database"), tr("Open/close/save/export database, exit"), SLOT(slot_openClicked()), true);

	pickFileTB = fileOpenPair->menu;
	pickFileTB->setStyleSheet(MENU_NO_CHECKBOX);
	addMenuItem(pickFileTB, tr("Open database"), SLOT(slot_openClicked()), true);
	openRecentFileTB = pickFileTB->addMenu(tr("    open a recently used database"));
	recentFileOpenTBActs.clear();
	for(int i=0; i<NrecentFiles; i++)
		{
		recentFileOpenTBActs.append(new PinMenuItem(pickFileTB, i));
		openRecentFileTB->addAction(recentFileOpenTBActs[i]);
		connect(recentFileOpenTBActs[i], SIGNAL(signal_textClicked(int)), this, SLOT(slot_openRecentFileClicked(int)));
		connect(recentFileOpenTBActs[i], SIGNAL(signal_pinClicked(int)), this, SLOT(slot_pinRecentFileClicked(int)));
		}
	connect(openRecentFileTB, SIGNAL(aboutToShow()), this, SLOT(slot_recentFileTBOpened()));
	openRecentDirTB = pickFileTB->addMenu(tr("    open from recently used directory"));
	recentDirOpenTBActs.clear();
	for(int i=0; i<NrecentDirs; i++)
		{
		recentDirOpenTBActs.append(new PinMenuItem(pickFileTB, i));
		openRecentDirTB->addAction(recentDirOpenTBActs[i]);
		connect(recentDirOpenTBActs[i], SIGNAL(signal_textClicked(int)), this, SLOT(slot_openRecentDirClicked(int)));
		connect(recentDirOpenTBActs[i], SIGNAL(signal_pinClicked(int)), this, SLOT(slot_pinRecentDirClicked(int)));
		}
	connect(openRecentDirTB, SIGNAL(aboutToShow()), this, SLOT(slot_recentDirTBOpened()));

	addMenuItem(pickFileTB, tr("Close database"), SLOT(slot_closeDbClicked()), true);
	addMenuItem(pickFileTB, tr("Save database"), SLOT(slot_saveClicked()), true);
	saveRecentDirTB = pickFileTB->addMenu(tr("    save to recently used directory"));
	recentDirSaveTBActs.clear();
	for(int i=0; i<NrecentDirs; i++)
		{
		recentDirSaveTBActs.append(new PinMenuItem(pickFileTB, i));
		saveRecentDirTB->addAction(recentDirSaveTBActs[i]);
		connect(recentDirSaveTBActs[i], SIGNAL(signal_textClicked(int)), this, SLOT(slot_saveRecentDirClicked(int)));
		connect(recentDirSaveTBActs[i], SIGNAL(signal_pinClicked(int)), this, SLOT(slot_pinRecentDirClicked(int)));
		}
	connect(saveRecentDirTB, SIGNAL(aboutToShow()), this, SLOT(slot_saveRecentDirTBOpened()));

	addMenuItem(pickFileTB, tr("Csv export"), SLOT(slot_csvExportClicked()), true);
	addMenuItem(pickFileTB, tr("File names (possibly unicode) to clipboard"), SLOT(slot_filenameClipboardClicked()), true);
		pickFileTB->addSeparator();
	addMenuItem(pickFileTB, tr("Exit"), SLOT(slot_closeClicked()), false);
	pickFileTB->setToolTip(tr("<html><head/><body><p>Any settings changes made from top-level menus will be saved.</p></body></html>"));

// Settings (toolbar)
	addIconAction(ui->toolBar, QIcon(tr(":/Images/settings.png")), tr("Settings"), SLOT(slot_settingsClicked()));

// Table design (toolbar)
	tDesignActionTB = addIconAction(ui->toolBar, QIcon(tr(":/Images/picnictable.png")), tr("Table design"), SLOT(slot_pushButtonDesignClicked()));

// Font (toolbar)
	addIconAction(ui->toolBar, QIcon(tr(":/Images/fontcombobox.png")), tr("Select font"), SLOT(slot_pickFont()));

// Toggle colors (toolbar)
	colorPair  = new raise2(this, ui->toolBar, QIcon(tr(":/Images/rowcolors.png")),
		tr("Toggle use of alternate row colors"), tr("Colors and skins"), SLOT(slot_altColorsToggled()), false);

	QMenu *skinTB = colorPair->menu;
	useAltColorsActionTB = addMenuItem(skinTB, tr("Use alternate row colors"), SLOT(slot_altColorsToggled()), false);
		useAltColorsActionTB->setCheckable(true);
		useAltColorsActionTB->setChecked(FileUseAltColors);
	addMenuItem(skinTB, tr("First row color"), SLOT(slot_evenColorClicked()), false);
	addMenuItem(skinTB, tr("Alternate row color"), SLOT(slot_oddColorClicked()), false);
	addMenuItem(skinTB, tr("&Gridcolor"), SLOT(slot_GridColorClicked()), false);

// Default skin (style)
	useDefaultStyleActionTB = addMenuItem(skinTB, tr("Default skin"), SLOT(slot_skinToggled()), false);
		useDefaultStyleActionTB->setCheckable(true);
		useDefaultStyleActionTB->setChecked(!UseModernStyle);
// Windows modern skin (style)
	useModernStyleActionTB = addMenuItem(skinTB, tr("Windows skin"), SLOT(slot_skinToggled()), false);
		useModernStyleActionTB->setCheckable(true);
		useModernStyleActionTB->setChecked(UseModernStyle);
// Change skin (style)
	addMenuItem(skinTB, tr("Pick skin"), SLOT(slot_pickSkin()), false);

// Simple Query (toolbar)
	queryPair  = new raise2(this, ui->toolBar, QIcon(tr(":/Images/searchfind.png")),
		tr("Simple database query"), tr("Query database"), SLOT(slot_pushButtonQueryClicked()), false);

	QMenu *pickQueryTB = queryPair->menu;
	pickQueryTB->setToolTipsVisible(true);
// Show all (toolbar)
	QAction *showAllTB = addMenuItem(pickQueryTB, tr("Show a&ll"), SLOT(slot_showallClicked()), true);
		showAllTB->setToolTip(tr("<html><head/><body><p>The Hide vol/subdir records checkbox affects what records are displayed.</p></body></html>"));
// Simple query (toolbar)
	addMenuItem(pickQueryTB, tr("Simple Query (^F or ^S)"), SLOT(slot_pushButtonQueryClicked()), true);
// Hide vol/subdir records (toolbar)
	ToolbarHideCheckbox = addMenuItem(pickQueryTB, tr("Hide vol/subdir records"), SLOT(slot_pushButtonKillIdClicked()), true);
		ToolbarHideCheckbox->setCheckable(true);
		ToolbarHideCheckbox->setChecked(NoId);
		ToolbarHideCheckbox->setToolTip(QFA("<html><head/><body><p>Checking Hide vol/subdir records takes effect immediately, unchecking affects the next query.</p></body></html>"));
// Invert selection (toolbar)
	addMenuItem(pickQueryTB, tr("Invert selection"), SLOT(slot_pushButtonInvertClicked()), true);
// Archives query (toolbar)
	addMenuItem(pickQueryTB, tr("Archives Queries"), SLOT(slot_pushButtonArchivesClicked()), true);
// Filetype query (toolbar)
	addMenuItem(pickQueryTB, tr("Filetype Queries"), SLOT(slot_pushButtonFiletypeClicked()), true);
// Names queries (toolbar)
	addMenuItem(pickQueryTB, tr("Names Queries"), SLOT(slot_pushButtonNamesClicked()), true);
// Range query (toolbar)
	addMenuItem(pickQueryTB, tr("Range Queries"), SLOT(slot_pushButtonRangeClicked()), true);
// Special query (toolbar)
	addMenuItem(pickQueryTB, tr("Special Queries"), SLOT(slot_pushButtonSpecialClicked()), true);
// All queries (toolbar)
	QAction *allQueryActionTB = new QAction(tr("All Queries"), this);
	pickQueryTB->addAction(allQueryActionTB);
		connect(allQueryActionTB, SIGNAL(triggered()),
			  this, SLOT(slot_pushButtonQueryClicked()), Qt::QueuedConnection);
		connect(allQueryActionTB, SIGNAL(triggered()),
			  this, SLOT(slot_pushButtonArchivesClicked()), Qt::QueuedConnection);
		connect(allQueryActionTB, SIGNAL(triggered()),
			  this, SLOT(slot_pushButtonFiletypeClicked()), Qt::QueuedConnection);
		connect(allQueryActionTB, SIGNAL(triggered()),
			  this, SLOT(slot_pushButtonNamesClicked()), Qt::QueuedConnection);
		connect(allQueryActionTB, SIGNAL(triggered()),
			  this, SLOT(slot_pushButtonRangeClicked()), Qt::QueuedConnection);
		connect(allQueryActionTB, SIGNAL(triggered()),
			  this, SLOT(slot_pushButtonSpecialClicked()), Qt::QueuedConnection);
// Close all queries (toolbar)
	QAction *closeQueriesActionTB = addMenuItem(pickQueryTB, tr("Close all query windows"), SLOT(slot_closeQueries()), true);
		closeQueriesActionTB->setStatusTip(tr("Dialog positions not saved"));

// Simple Sort (toolbar)
	sortPair = new raise2(this, ui->toolBar, QIcon(tr(":/Images/mysort.png")),
		tr("Simple sort options"), tr("Sort database"), SLOT(slot_pushButtonSortClicked()), false);

	QMenu *pickSortTB = sortPair->menu;
	pickSortTB->setStyleSheet(MENU_NO_CHECKBOX);
// Simple sort (toolbar)
	addMenuItem(pickSortTB, tr("Simple Sorts"), SLOT(slot_pushButtonSortClicked()), true);
// Arbitrary sorts (toolbar)
	addMenuItem(pickSortTB, tr("Custom Sorts"), SLOT(slot_pushButtonFullSortClicked()), true);
// All sorts (toolbar)
	QAction *allSortsActionTB = addMenuItem(pickSortTB, tr("All Sort options"), SLOT(slot_pushButtonSortClicked()), true);
		connect(allSortsActionTB, SIGNAL(triggered()),
			  this, SLOT(slot_pushButtonFullSortClicked()), Qt::QueuedConnection);
// Close all sorts (toolbar)
	QAction *closeSortsActionTB = addMenuItem(pickSortTB, tr("Close all sort windows"), SLOT(slot_closeSorts()), true);
		closeSortsActionTB->setStatusTip(tr("Dialog positions not saved"));

// Index (toolbar)
	addIconAction(ui->toolBar, QIcon(tr(":/Images/index.png")), tr("Index disks"), SLOT(slot_openIndex()));

// Volume list (toolbar)
	vollistActionTB = addIconAction(ui->toolBar, QIcon(tr(":/Images/icon_48x48_disks.png")),
		tr("List volumes"), SLOT(slot_openVolumes()));

// Go to first (toolbar)
	firstActionTB = addIconAction(ui->toolBar, QIcon(tr(":/Images/go-first.png")),
		tr("Go to first record"), SLOT(slot_pushButtonFirstClicked()));
		firstActionTB->setDisabled(true);

// Go to previous (toolbar)
	previousActionTB = addIconAction(ui->toolBar, QIcon(tr(":/Images/go-previous-24.png")),
		tr("Go to previous group of records"), SLOT(slot_pushButtonPreviousClicked()));
		previousActionTB->setDisabled(true);

// Go to next (toolbar)
	nextActionTB = addIconAction(ui->toolBar, QIcon(tr(":/Images/go-next-24.png")),
		tr("Go to next group of records"), SLOT(slot_pushButtonNextClicked()));
		nextActionTB->setDisabled(true);

// Go to last (toolbar)
	lastActionTB = addIconAction(ui->toolBar, QIcon(tr(":/Images/go-last.png")),
		tr("Go to last group of records"), SLOT(slot_pushButtonLastClicked()));
		lastActionTB->setDisabled(true);

// Fit width (toolbar)
	fitWidthActionTB = addIconAction(ui->toolBar, QIcon(tr(":/Images/fitwidth.png")),
		tr("Auto-fit column widths"), SLOT(slot_pushButtonFitWidthClicked()));
		fitWidthActionTB->setDisabled(true);

// Spacer (toolbar)
	QAction *nullActionTB = new QAction(QIcon(tr(":/Images/blank.png")), tr(""), this);
	nullActionTB->setDisabled(true);
	ui->toolBar->addAction(nullActionTB);

// Row height spinbox (toolbar)
	QSpinBox *hspinTB = new QSpinBox(this);
	hspinTB->setRange(5,99);
	hspinTB->setValue(File_row_height);
	hspinTB->setPrefix(tr("Row height: "));
	hspinTB->setToolTip(tr("<html><head/><body><p>Row height (you can also drag at the left margin to adjust height of individual rows)</p></body></html>"));
	QWidgetAction *spinActionTB = new QWidgetAction(ui->toolBar);
	spinActionTB->setDefaultWidget(hspinTB);
	ui->toolBar->addAction(spinActionTB);
	connect(hspinTB, SIGNAL(valueChanged(int)),
			  this, SLOT(slot_rowHeightSpinboxChanged(int)));

// Spacer (toolbar)
	QAction *nullActionTB2 = new QAction(QIcon(tr(":/Images/blank.png")), tr(""), this);
	nullActionTB2->setDisabled(true);
	ui->toolBar->addAction(nullActionTB2);

// Exit (toolbar)
	addIconAction(ui->toolBar, QIcon(tr(":/Images/exit.png")),
		tr("<html><head/><body><p>Save any settings changes that were made from top-level menus, then exit.</p></body></html>"), SLOT(slot_closeClicked()));
	}

void MainWindow::enableMenusIfDbLoaded()
	{
	bool enable = (Nofile > 0);

	queryMenu->setEnabled(enable);
	sortMenu->setEnabled(enable);
	openVollist->setEnabled(enable);
	queryPair->widebutton->setEnabled(enable);
	queryPair->pulldown->setEnabled(enable);
	sortPair->widebutton->setEnabled(enable);
	sortPair->pulldown->setEnabled(enable);
	vollistActionTB->setEnabled(enable);
	}
