// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "Dialogs/mainwindow.h"
#include "ui_mainwindow.h"
#include "Dialogs/settingsdialog.h"
#include "OtherWssi/tablelayout.h"
#include "OtherWssi/wssiproto.h"
#include "OtherWssi/wssiproto.h"
#include <QtDebug>

void MainWindow::slot_settingsClicked()
	{
	if(!settings_dialog)
		settings_dialog = new settingsDialog(this);
	settings_dialog->show();
	}

void MainWindow::slot_saveSettings()
	{
// By using different definitions of getputOption, the same list of variables
// can be used for both saving and restoring settings.  This doesn't work for
// fonts because the setter uses different syntax, so they get done separately.
#define getputOption(x,y) options.setValue(#x, x)
#include "OtherWssi/optionsVarlist.include"
#undef getputOption
	options.setValue("Extract_opt", Extract_opt);
	}

// This routine gets called when exiting (not by X at top right)
// to save values that are set by top-level menus.
void MainWindow::saveTopLevelSettings()
	{
// By using different definitions of getputOption, the same list of variables
// can be used for both saving and restoring settings.  This doesn't work for
// fonts because the setter uses different syntax, so they get done separately.
#define getputOption(x,y) options.setValue(#x, x)
	#define SAVING
	#include "OtherWssi/optionsVarlistTop.include"
	#undef SAVING
#undef getputOption
#define getputOption(x,y) options.setValue(#x, *x)
	getputOption(Font_filelist, dummy);
#undef getputOption
	}

void MainWindow::restoreTopLevelSettings()
	{
// By using different definitions of getputOption, the same list of variables
// can be used for both saving and restoring settings.  This doesn't work for
// fonts because the setter uses different syntax, so they get done separately.
#define getputOption(x,y) x = options.value(#x, x).y()
	#include "OtherWssi/optionsVarlistTop.include"
#undef getputOption
	FileRowColor[0] = options.value("FileRowColor[0]", FileRowColor[0]).value<QColor>();
	FileRowColor[1] = options.value("FileRowColor[1]", FileRowColor[1]).value<QColor>();
// A bug set both colors to zero (black), same as the font color, if there was no saved value.  Fix this if found.
// Comparing the color again Qt::black or QColor(0,0,0,255) doesn't work so pull out rgba.
	if((FileRowColor[0].red() + FileRowColor[0].green() + FileRowColor[0].blue() == 0 && FileRowColor[0].alpha() == 255)
	 || (FileRowColor[1].red() + FileRowColor[1].green() + FileRowColor[1].blue() == 0 && FileRowColor[1].alpha() == 255))
		{
		FileRowColor[0] = QColor(255,250,220);
		FileRowColor[1] = QColor(193,206,250);
		QMessageBox::information(this, tr("Bad saved colors"), tr("<html><head/><body><p>Saved background color is black, same as the font color. "
			"This is probably caused by a bug in an older version of Wssindex - on first use, no saved value exists.  The bug used "
			"zero (black).<p>Reasonable defaults have been set.  Use the red exit button to save this change."));
		}
	FileGridColor = options.value("FileGridColor", FileGridColor).value<QColor>();
	UpdatedColor = options.value("UpdatedColor", UpdatedColor).value<QColor>();
	Font_filelist->fromString(options.value("Font_Filelist", *Font_filelist).toString());
	ui->tableView->setFont(*Font_filelist);
	setFileGridColor();
	ui->tableView->horizontalHeader()->setFont(*Font_filelist);
	}

void MainWindow::restoreSettings()
	{
	setDefaultValues();
	restoreTopLevelSettings();

#define getputOption(x,y) x = options.value(#x, x).y()
	#include "OtherWssi/optionsVarlist.include"
#undef getputOption
// Make sure saved number of query threads is appropriate for the hardware we're running on
// Max_threads is set to 1 if the number of cores cannot be determined
	Max_threads = qMin(Max_threads, QThread::idealThreadCount());
	Max_threads = qMax(Max_threads, 1);
	Extract_opt = static_cast<ExtractOpts>((options.value("Extract_opt", 1).toInt()));

	QSettings recent("WSS-DDC", "Wssindex\\Recent");
	recentFiles->resize(NrecentFiles);
	if(recent.contains("Recent_files[0]"))
		{
		for(int i=0; i<NrecentFiles; i++)
			{
			QString fn = recent.value(tr("recent_files[%1]").arg(i), QString()).toString();
			bool ispinned = recent.value(tr("recent_files_pinned[%1]").arg(i), false).toBool();
			if(fn.isEmpty())
				break;
			recentFiles->addEnd(fn, ispinned);
			}
		}

	recentDirs->resize(NrecentDirs);
	if(recent.contains("Recent_dirs[0]"))
		{
		for(int i=0; i<NrecentDirs; i++)
			{
			QString dirn = recent.value(tr("recent_dirs[%1]").arg(i), QString()).toString();
			bool ispinned = recent.value(tr("recent_dirs_pinned[%1]").arg(i), false).toBool();
			if(dirn.isEmpty())
				break;
			recentDirs->addEnd(dirn, ispinned);
			}
		}
//	can't call updateRecentLists() yet, it references things not yet created.

	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	if(settings.contains("MainGeometrySize"))
		{
		QSize size = settings.value("MainGeometrySize").toSize();
		resize(size);
		}
	if(settings.contains("MainGeometryPos"))
		{
		QPoint pos = settings.value("MainGeometryPos").toPoint();
		move(pos);
		forceOnscreen(this);
		}

// Restore table design to a temp array, check validity, and use if ok
	QSettings tformat("WSS-DDC", "Wssindex");
	if(tformat.contains("TableFormat/10/enabled"))
		{
		struct fl tempdesign[NFIELDS];
		tformat.beginReadArray("TableFormat");
		for (int i=0; i<NFIELDS; ++i)
			{
			tformat.setArrayIndex(i);
			tempdesign[i].enabled = tformat.value("enabled").toBool();
			tempdesign[i].row = tformat.value("row").toInt() - 1;
			tempdesign[i].column = tformat.value("column").toInt() - 1;
			tempdesign[i].height = tformat.value("height").toInt();
			tempdesign[i].width = tformat.value("width").toInt();
			tempdesign[i].alignment = justEnumFromInt(tformat.value("justification").toInt());
			}
		tformat.endArray();
		if(isTableDesignConsistent(tempdesign))
			{
			rowsPerItem = 1;
			colsPerItem = 1;
			for (int i=0; i<NFIELDS; ++i)
				{
// Following line would clobber the field names, but they get (re)set later
				fieldlayout[i] = tempdesign[i];
				if(fieldlayout[i].enabled)
					{
					rowsPerItem = qMax(rowsPerItem,
												fieldlayout[i].row+fieldlayout[i].height);
					colsPerItem = qMax(colsPerItem,
												fieldlayout[i].column+fieldlayout[i].width);
					}
				}
			set_editable_cells();
			}
		else
			QMessageBox::warning(this, tr("Warning"), tr("Invalid saved table design ignored."));
		}
	QSettings extractopts("WSS-DDC", "Wssindex\\ExtractOpts");
	for(unsigned int i=0; i<NLIBTYPE; i++)
		{
		switch(Extract[i].type)
			{
			case TYPE_ARCHIVE:
				extractopts.beginGroup("Archives");
				break;
			case TYPE_FONT:
				extractopts.beginGroup("Fonts");
				break;
			case TYPE_GRAPHICS:
				extractopts.beginGroup("Graphics");
				break;
			case TYPE_SELF_EXTRACTING:
				extractopts.beginGroup("Selfextracting");
				break;
			case TYPE_SOUND:
				extractopts.beginGroup("Sounds");
				break;
			case TYPE_MOVIE:
				extractopts.beginGroup("Videos");
				break;
			}
		if(extractopts.contains(Extract[i].description))
			{
			int tflag = extractopts.value(Extract[i].description).toInt();
			if(0 <= tflag && tflag <= 7)
				Extract[i].flag = tflag;
			}
		extractopts.endGroup();
		}

	QSettings IndexFilterOpts("WSS-DDC", "Wssindex\\Options\\IndexTimeFiltering");
	IndexTimeFileExclude = IndexFilterOpts.value("IndexTimeFileExclude").toBool();
	IndexTimeFileInclude = IndexFilterOpts.value("IndexTimeFileInclude").toBool();
	int excludeFileListCount = IndexFilterOpts.value("ExcludeFileListCount").toInt();
	IndexFileExcludeList.clear();
	for(int i=0; i<excludeFileListCount; i++)
		{
		QString tstring = IndexFilterOpts.value(QString("ExcludeFileList[%1]").arg(i)).toString();
		if(tstring.size() > 0)
			IndexFileExcludeList.append(tstring);
		}
	IndexFileExcludeList.sort();
	IndexFileExcludeList.removeDuplicates();

	int includeFileListCount = IndexFilterOpts.value("IncludeFileListCount").toInt();
	IndexFileIncludeList.clear();
	for(int i=0; i<includeFileListCount; i++)
		{
		QString tstring = IndexFilterOpts.value(QString("IncludeFileList[%1]").arg(i)).toString();
		if(tstring.size() > 0)
			IndexFileIncludeList.append(tstring);
		}
	IndexFileIncludeList.sort();
	IndexFileIncludeList.removeDuplicates();

	IndexTimeDirExclude = IndexFilterOpts.value("IndexTimeDirExclude").toBool();
	IndexTimeDirInclude = IndexFilterOpts.value("IndexTimeDirInclude").toBool();
	int excludeDirListCount = IndexFilterOpts.value("ExcludeDirListCount").toInt();
	IndexDirExcludeList.clear();
	for(int i=0; i<excludeDirListCount; i++)
		{
		QString tstring = IndexFilterOpts.value(QString("ExcludeDirList[%1]").arg(i)).toString();
		if(tstring.size() > 0)
			IndexDirExcludeList.append(tstring);
		}
	IndexDirExcludeList.sort();
	IndexDirExcludeList.removeDuplicates();

	int includeDirListCount = IndexFilterOpts.value("IncludeDirListCount").toInt();
	IndexDirIncludeList.clear();
	for(int i=0; i<includeDirListCount; i++)
		{
		QString tstring = IndexFilterOpts.value(QString("IncludeDirList[%1]").arg(i)).toString();
		if(tstring.size() > 0)
			IndexDirIncludeList.append(tstring);
		}
	IndexDirIncludeList.sort();
	IndexDirIncludeList.removeDuplicates();
	}
