// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "Dialogs/messagelogwindow.h"
#include "ui_messagelogwindow.h"
#include "Dialogs/mainwindow.h"
#include "OtherWssi/initialpos.h"
#include "OtherWssi/wssiproto.h"

// This is based on the Qt examples\mainwindows\application example

MessageLogWindow::MessageLogWindow(QWidget *parent)
	 : QMainWindow(parent), ui(new Ui::MessageLogWindow)
	{
	ui->setupUi(this);
	this->setWindowFlags(MyWindowFlags);
	restoreSettings();

	setCurrentFile("");
	createActions();
	createMenus();
	createToolBars();

	clearIntroText = false;
	slot_logMessage(tr("Indexing status messages appear in this log window.\n"
		"You can save the log if messages indicate problems with files that need to be investigated.\n"
		"Simple editing functions are supported.\n"));
	ui->MessageLogPlainTextEdit->appendHtml((tr("<font color=red size=+2>"
		"If this window is closed during indexing, all but the most critical error messages will not be shown.</font><br>")));
	clearIntroText = true;
	ui->MessageLogPlainTextEdit->setLineWrapMode(Msglog_linewrap ? QPlainTextEdit::WidgetWidth : QPlainTextEdit::NoWrap);
	MessageLogWindowOpen = true;
	}

MessageLogWindow::~MessageLogWindow()
	{
	MessageLogWindowOpen = false;
	delete ui;
	}

void MessageLogWindow::slot_closeLog()
	{
	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	settings.setValue("MessageWindowPos", pos());
	settings.setValue("MessageWindowSize", size());
	QSettings options("WSS-DDC", "Wssindex\\Options");
#define getputOption(x,y) options.setValue(#x, x)
	getputOption(Msglog_linewrap, toBool);
#undef getputOption
	close();
	}

void MessageLogWindow::restoreSettings()
	{
	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	if(settings.contains("MessageWindowSize"))
		{
		QSize size = settings.value("MessageWindowSize").toSize();
		resize(size);
		}
	if(settings.contains("MessageWindowPos"))
		{
		QPoint pos = settings.value("MessageWindowPos").toPoint();
		move(pos);
		forceOnscreen(this);
		}
	else
		{
		this->show();
		move(mainWindowPtr->offCenter(this->frameGeometry(), INDEXLOG_OFFSET));
		}
	QSettings options("WSS-DDC", "Wssindex\\Options");
#define getputOption(x,y) x = options.value(#x, x).y()
	getputOption(Msglog_linewrap, toBool);
#undef getputOption
	}

void MessageLogWindow::slot_logMessage(QString message)
	{
	if(clearIntroText)
		{
		ui->MessageLogPlainTextEdit->clear();
		clearIntroText = false;
		}
	ui->MessageLogPlainTextEdit->appendPlainText(QDir::toNativeSeparators(message));
	}

void MessageLogWindow::createActions()
	{
	saveAct = new QAction(QIcon(":/Images/save.png"), tr("&Save"), this);
	saveAct->setShortcuts(QKeySequence::Save);
	saveAct->setStatusTip(tr("Save the document to disk"));
	connect(saveAct, SIGNAL(triggered()), this, SLOT(slot_save()));

	saveAsAct = new QAction(tr("Save &As..."), this);
	saveAsAct->setShortcuts(QKeySequence::SaveAs);
	saveAsAct->setStatusTip(tr("Save the document under a new name"));
	connect(saveAsAct, SIGNAL(triggered()), this, SLOT(slot_saveAs()));

	exitAct = new QAction(tr("E&xit"), this);
	exitAct->setShortcuts(QKeySequence::Quit);
	exitAct->setStatusTip(tr("Exit from the message logger"));
	connect(exitAct, SIGNAL(triggered()), this, SLOT(slot_closeLog()));

	cutAct = new QAction(QIcon(":/Images/cut.png"), tr("Cu&t"), this);
	cutAct->setShortcuts(QKeySequence::Cut);
	cutAct->setStatusTip(tr("Cut the current selection's contents to the "
								 "clipboard"));
	connect(cutAct, SIGNAL(triggered()), ui->MessageLogPlainTextEdit, SLOT(cut()));

	copyAct = new QAction(QIcon(":/Images/copy.png"), tr("&Copy"), this);
	copyAct->setShortcuts(QKeySequence::Copy);
	copyAct->setStatusTip(tr("Copy the current selection's contents to the "
								  "clipboard"));
	connect(copyAct, SIGNAL(triggered()), ui->MessageLogPlainTextEdit, SLOT(copy()));

	pasteAct = new QAction(QIcon(":/Images/paste.png"), tr("&Paste"), this);
	pasteAct->setShortcuts(QKeySequence::Paste);
	pasteAct->setStatusTip(tr("Paste the clipboard's contents into the current "
									"selection"));
	connect(pasteAct, SIGNAL(triggered()), ui->MessageLogPlainTextEdit, SLOT(paste()));

	wrapAct = new QAction(QIcon(":/Images/wrap.png"), tr("Toggle line&wrap"), this);
	wrapAct->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_W));
	updateWrapToggleStatus();
	connect(wrapAct, SIGNAL(triggered()), this, SLOT(slot_wrapToggle()));

	cutAct->setEnabled(false);
	copyAct->setEnabled(false);
	connect(ui->MessageLogPlainTextEdit, SIGNAL(copyAvailable(bool)),
		cutAct, SLOT(setEnabled(bool)));
	connect(ui->MessageLogPlainTextEdit, SIGNAL(copyAvailable(bool)),
		copyAct, SLOT(setEnabled(bool)));
	}

void MessageLogWindow::createMenus()
	{
	 fileMenu = menuBar()->addMenu(tr("&File"));
	 fileMenu->addAction(saveAct);
	 fileMenu->addAction(saveAsAct);
	 fileMenu->addSeparator();
	 fileMenu->addAction(exitAct);

	 editMenu = menuBar()->addMenu(tr("&Edit"));
	 editMenu->addAction(cutAct);
	 editMenu->addAction(copyAct);
	 editMenu->addAction(pasteAct);
	 editMenu->addAction(wrapAct);
	}

void MessageLogWindow::createToolBars()
	{
	 fileToolBar = addToolBar(tr("File"));
	 fileToolBar->addAction(saveAct);

	 editToolBar = addToolBar(tr("Edit"));
	 editToolBar->addAction(cutAct);
	 editToolBar->addAction(copyAct);
	 editToolBar->addAction(pasteAct);
	 editToolBar->addAction(wrapAct);
	}

void MessageLogWindow::updateWrapToggleStatus()
	{
	if(Msglog_linewrap)
		wrapAct->setStatusTip(tr("Toggle linewrap (now on)"));
	else
		wrapAct->setStatusTip(tr("Toggle linewrap (now off)"));
// Force the status tip to change even if still hovering over the toolbar button
	wrapAct->showStatusText();
	}

void MessageLogWindow::slot_wrapToggle()
	{
	Msglog_linewrap = !Msglog_linewrap;
	ui->MessageLogPlainTextEdit->setLineWrapMode(Msglog_linewrap ? QPlainTextEdit::WidgetWidth : QPlainTextEdit::NoWrap);
	updateWrapToggleStatus();
	}

bool MessageLogWindow::slot_save()
	{
	if(curFile.isEmpty())
		return slot_saveAs();
	else
		return saveFile(curFile);
	}

bool MessageLogWindow::slot_saveAs()
	{
	QString fileName = QFileDialog::getSaveFileName(this);
	if(fileName.isEmpty())
		return false;

	return saveFile(fileName);
	}

bool MessageLogWindow::saveFile(const QString &fileName)
	{
	QFile file(fileName);
	if (!file.open(QFile::WriteOnly | QFile::Text))
		{
		QMessageBox::warning(this, tr("Application"),
									  tr("Cannot write file %1:\n%2.")
									  .arg(fileName, file.errorString()));
		return false;
		}

	QTextStream out(&file);
	out << ui->MessageLogPlainTextEdit->toPlainText();

	setCurrentFile(fileName);
//	statusBar()->showMessage(tr("File saved"), 2000);
	return true;
	}

void MessageLogWindow::setCurrentFile(const QString &fileName)
	{
	curFile = fileName;
	ui->MessageLogPlainTextEdit->document()->setModified(false);
	setWindowModified(false);

	QString shownName = curFile;
	if (curFile.isEmpty())
		shownName = tr("untitled.txt");
	setWindowFilePath(shownName);
	}

void MessageLogWindow::slot_keyToMl(int key, int modifier)
	{
	this->raise();
	QApplication::sendEvent(ui->MessageLogPlainTextEdit->viewport(),
		new QKeyEvent(QEvent::KeyPress, key, static_cast<Qt::KeyboardModifier>(modifier)));
	QApplication::sendEvent(ui->MessageLogPlainTextEdit->viewport(),
		new QKeyEvent(QEvent::KeyRelease, key, static_cast<Qt::KeyboardModifier>(modifier)));
	}

void MessageLogWindow::slot_wheelToMl(QPoint pos, int delta, Qt::MouseButtons buttons,
									Qt::KeyboardModifiers modifiers, Qt::Orientation orientation)
	{
	this->raise();
	QApplication::sendEvent(ui->MessageLogPlainTextEdit->viewport(),
		new QWheelEvent(pos, delta, buttons, modifiers, orientation));
	}
