// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "Dialogs/indexwindow.h"
#include "Dialogs/mainwindow.h"
#include "Dialogs/vollistwindow.h"
#include "OtherWssi/wssiproto.h"

#include <QMessageBox>
#include <QObject>

// purge_dups - remove duplicate volumes from a Wssindex database.  Given a
//	duplication, the volume removed is
//		1. the least recently indexed
//		2. the one with the smaller number of files
//		3. the second one in the database
// If silent is true, don't generate any output if no deletions are done.

void MainWindow::purge_dups(bool silent)
	{
	if(Diskno < 2)
		return;

	unsigned long *vptr;
	try {
		vptr = new unsigned long [Diskno];
		}
	catch(std::bad_alloc const&)
		{
		QMessageBox::critical(this, tr("Error"), tr("Insufficient memory to purge duplicate volumes."));
		return;
		}

	unsigned long *vtptr = vptr;
	for(unsigned long i=0; i<Diskno; i++)
		*(vtptr++) = i;
	Vsrtkey = SORT_VOL_DUPS;
	doSort(vptr, Diskno, wssvcomp, 0);

	struct diskrec	*vrec_previous = Vrec + *vptr;
	unsigned long ndups = 0;
	unsigned long last_keep = 0;
	unsigned long starting_diskno = Diskno;
	for(unsigned long i=1; i<starting_diskno; i++)
		{
		unsigned long j = *(vptr + i);
		struct diskrec	*vrec_i = Vrec + j;
		if(0 == qstringCompare(vrec_previous->VolName, vrec_i->VolName))
			{
			if(removews(vrec_i->VolName, j))
				{
				QMessageBox::critical(this, tr("Error"), tr("Error: unable to remove duplicates"));
				break;
				}
			ndups++;
			for(unsigned long k=last_keep; k<starting_diskno; k++)
				if(*(vptr + k) > j)
					--*(vptr + k);
// Careful, the disk being kept may have been moved by the last deletion
			j = *(vptr + last_keep);
			vrec_previous = Vrec + j;
			continue;
			}
		else
			{
			last_keep = i;
			vrec_previous = vrec_i;
			}
		}
	delete [] vptr;
	if(ndups > 0)
		{
		QMessageBox::information(this, tr("Duplicates removed"),
			tr("%1 duplicate volume name%2 found and removed").arg(ndups).arg(1==ndups ? tr(" was") : tr("s were")));
		}
	else if(!silent)
		QMessageBox::information(this, tr("No duplicates"), tr("No duplicates were found"));
	return;
	}
