// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "OtherWssi/globalsettings.h"
#include "OtherWssi/wssiproto.h"
#include "Dialogs/settingsdialog.h"
#include "ui_settingsdialog.h"
#include "ExtractSubs/extractinfo.h"
#include "Dialogs/mainwindow.h"
#include "Dialogs/sortbenchdialog.h"

#include <QCheckBox>
#include <QFormLayout>
#include <QLabel>
#include <QGroupBox>
#include <QSettings>
//#include <QDebug>
#include <QComboBox>
#include <QLineEdit>
#include <QColor>

// This dialog is not destroyed after each use, so the constructor is only
// called once.  showEvent is used to update display for any values that
// might have been changed by other routines.  Any routine here not called
// from showEvent can use new without having to worry about deleting
// the objects created.
settingsDialog::settingsDialog(QWidget *parent) :
	QMainWindow(parent), ui(new Ui::settingsDialog)
	{
// Could use KiB, MiB, GiB here for binary size units, but who would understand?
	QString units[] =
		{tr("bytes"),
		 tr("K/M/G/TB (dec)"),
		 tr("K/M/G/TB (bin)")};

	checkboxes_created = false;
	ui->setupUi(this);
	this->setWindowModality(Qt::WindowModal);
	this->setWindowFlags(MyWindowFlags);
	addOptions();
	checkboxes_created = true;

// Indexing options tab
	ui->spinBox_category_length->setAlignment(Qt::AlignRight);
	ui->spinBox_comment_length->setAlignment(Qt::AlignRight);
	ui->spinBox_extra_length->setAlignment(Qt::AlignRight);
	ui->comboBox_archive_member_case->addItem(tr("Preserve"));
	ui->comboBox_archive_member_case->addItem(tr("Force uppercase"));
	ui->comboBox_archive_member_case->addItem(tr("Force lowercase"));
	widerCombo(ui->comboBox_archive_member_case);
	ui->comboBox_fileid_diz_comments->addItem(tr("Don't use"));
	ui->comboBox_fileid_diz_comments->addItem(tr("Apply to File_id.diz"));
	ui->comboBox_fileid_diz_comments->addItem(tr("Apply to zip file"));
	widerCombo(ui->comboBox_fileid_diz_comments);

// General options tab
// Populate the size units controls
	for(int i=0; i<3; i++)
		{
		ui->comboBox_DisksizeUnits->insertItem(i+1, units[i], i);
		ui->comboBox_FilesizeUnits->insertItem(i+1, units[i], i);
		}
	widerCombo(ui->comboBox_DisksizeUnits);
	widerCombo(ui->comboBox_FilesizeUnits);
	ui->spinBox_arc_indent->setAlignment(Qt::AlignRight);
	ui->spinBox_max_recent_files->setAlignment(Qt::AlignRight);
	ui->spinBox_max_recent_dirs->setAlignment(Qt::AlignRight);
	ui->spinBox_max_files_at_once->setAlignment(Qt::AlignRight);
	ui->spinBox_fadeout->setAlignment(Qt::AlignRight);
	ui->spinBox_toolTipFontSize->setAlignment(Qt::AlignRight);
	ui->spinBox_statusBarFontSize->setAlignment(Qt::AlignRight);
	ui->spinBox_MaxThreads->setAlignment(Qt::AlignRight);

// Memory optimization tab
	ui->spinBox_init_files->setAlignment(Qt::AlignRight);
	ui->spinBox_init_subs->setAlignment(Qt::AlignRight);
	ui->spinBox_init_disks->setAlignment(Qt::AlignRight);
	ui->spinBox_incr_files->setAlignment(Qt::AlignRight);
	ui->spinBox_incr_subs->setAlignment(Qt::AlignRight);
	ui->spinBox_incr_disks->setAlignment(Qt::AlignRight);
	ui->label_memory_opt->setWordWrap(true);
	ui->label_memory_opt->setText(tr(
		"Memory for the database is allocated to allow the above size limits, "
		"then expanded as needed.  During the expansion, memory usage is higher "
		"because two copies of the data exist at the same time.  Every time a "
		"reallocation is required, the corresponding increment is doubled (but "
		"you won't see any change here).  If you are "
		"hitting the 2GB process space limit for 32-bit Windows, it may help "
		"to make the initial allocations larger so reallocation is not needed,"
		"otherwise defaults here should be ok."));

	connect(ui->toolButton_threadBench, SIGNAL(clicked()), this, SLOT(slot_sortBenchClicked()));
	connect(ui->pushButton_cancel, SIGNAL(clicked()), this, SLOT(slot_cancelClicked()));
	connect(ui->pushButton_apply, SIGNAL(clicked()), this, SLOT(slot_applyClicked()));
	connect(ui->pushButton_save, SIGNAL(clicked()), this, SLOT(slot_saveClicked()));
	connect(this, SIGNAL(signal_saveSettings()), parent, SLOT(slot_saveSettings()));
	connect(this, SIGNAL(signal_updateDisplay(bool)),
		mainWindowPtr, SLOT(slot_updateDisplay(bool)));
	}

settingsDialog::~settingsDialog()
	{
	delete ui;
	}

void settingsDialog::populateGeneralOptions()
	{
	ui->checkBox_systemScale->setChecked(UseSystemScale);
	ui->checkBox_dateonly->setChecked(DateOnly);
	ui->checkBox_caseSensitive->setChecked(!Ignore_case_sort);
	ui->checkBox_natural_sort->setChecked(NaturalSort);
	ui->spinBox_arc_indent->setValue(Indent_members);
	ui->comboBox_DisksizeUnits->setCurrentIndex(disksize_unit);
	ui->comboBox_FilesizeUnits->setCurrentIndex(Filesize_unit);
	ui->spinBox_fadeout->setValue(Fade_time);
	ui->spinBox_toolTipFontSize->setValue(ToolTipFontSize);
	ui->spinBox_statusBarFontSize->setValue(StatusBarFontSize);
	ui->spinBox_MaxThreads->setValue(Max_threads);
	int maxthread = qMax(1, QThread::idealThreadCount());
//	maxthread = qMin(maxthread, Max_threads);
	ui->spinBox_MaxThreads->setMaximum(maxthread);
	ui->checkBox_makebackup->setChecked(Makebak);
	ui->checkBox_helpmenu_right->setChecked(HelpMenuRight);
	ui->spinBox_max_recent_files->setValue(NrecentFiles);
	ui->spinBox_max_recent_dirs->setValue(NrecentDirs);
	ui->spinBox_max_files_at_once->setValue(Max_bunch);
	}

void settingsDialog::populateMemoryOptions()
	{
	ui->spinBox_init_files->setValue(Maxfiles_initial);
	ui->spinBox_init_subs->setValue(Maxsubs_initial);
	ui->spinBox_init_disks->setValue(Maxdisks_initial);
	ui->spinBox_incr_files->setValue(Maxfiles_increment);
	ui->spinBox_incr_subs->setValue(Maxsubs_increment);
	ui->spinBox_incr_disks->setValue(Maxdisks_increment);
	}

void settingsDialog::populateIndexingOptions()
	{
	ui->spinBox_category_length->setValue(Max_category_length);
	ui->spinBox_comment_length->setValue(Max_comment_length);
	ui->spinBox_extra_length->setValue(Max_extra_length);
	ui->checkBox_IndexHidden->setChecked(Index_hidden);
	ui->checkBox_eject->setChecked(EjectAfterIndexing);
	ui->checkBox_skip_junctions->setChecked(Skip_NTFS_mounted);
	ui->checkBox_relative_paths->setChecked(Store_relative_path);
	ui->checkBox_archive_paths->setChecked(Store_arc_path);

	ui->comboBox_archive_member_case->setCurrentIndex(Case_option_indexing);

	ui->comboBox_fileid_diz_comments->setCurrentIndex(Extract_diz);
	ui->checkBox_always_diz->setChecked(Always_diz_contents);
	ui->checkBox_ask_extract->setChecked(Query_extract);

	ui->checkBox_IgnoreArcExtension->setChecked(Ignore_arc_ext);
	ui->checkBox_allow_subdir_change->setChecked(Copy_across_subdirs);
	}

void settingsDialog::restoreWindowSettings()
	{
	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	if(settings.contains("SettingsDialogSize"))
		{
		QSize size = settings.value("SettingsDialogSize").toSize();
		resize(size);
		}
	if(settings.contains("SettingsDialogPos"))
		{
		QPoint pos = settings.value("SettingsDialogPos").toPoint();
		move(pos);
		forceOnscreen(this);
		}
	}

void settingsDialog::saveWindowSettings()
	{
	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	settings.setValue("SettingsDialogSize", size());
	settings.setValue("SettingsDialogPos", pos());
	}

void settingsDialog::slot_cancelClicked()
	{
	if(mainWindowPtr->update_display_on_return_to_main)
		Q_EMIT signal_updateDisplay(true);
	close();
	}

void settingsDialog::slot_sortBenchClicked()
	{
	sortBenchDialog *s = new sortBenchDialog(this);
	s->setWindowModality(Qt::WindowModal);
	s->setAttribute(Qt::WA_DeleteOnClose);
//	s->show();		// show is done in constructor
	}

void settingsDialog::apply_settings()
	{
// If parameters that affect the display have changed, set
// flag to tell main window display needs to be updated.
	if(ui->checkBox_dateonly->isChecked() != DateOnly
		|| ui->spinBox_arc_indent->value() != Indent_members
		|| ui->comboBox_FilesizeUnits->currentIndex() != Filesize_unit
		|| ui->comboBox_DisksizeUnits->currentIndex() != disksize_unit
		|| ui->spinBox_max_files_at_once->value() != Max_bunch)
		mainWindowPtr->update_display_on_return_to_main = true;

	UseSystemScale = ui->checkBox_systemScale->isChecked();
	DateOnly = ui->checkBox_dateonly->isChecked();
	Ignore_case_sort = !ui->checkBox_caseSensitive->isChecked();
	collator->setCaseSensitivity(Ignore_case_sort ? Qt::CaseInsensitive : Qt::CaseSensitive);
	NaturalSort = ui->checkBox_natural_sort->isChecked();
	collator->setNumericMode(NaturalSort);
	Indent_members = ui->spinBox_arc_indent->value();
	Filesize_unit = ui->comboBox_FilesizeUnits->currentIndex();
	disksize_unit = ui->comboBox_DisksizeUnits->currentIndex();
	Fade_time = ui->spinBox_fadeout->value();
	ToolTipFontSize = ui->spinBox_toolTipFontSize->value();
	StatusBarFontSize = ui->spinBox_statusBarFontSize->value();
	Max_threads = ui->spinBox_MaxThreads->value();
	Makebak = ui->checkBox_makebackup->isChecked();
	HelpMenuRight = ui->checkBox_helpmenu_right->isChecked();
	mainWindowPtr->helpRightSeparator->setVisible(HelpMenuRight);
	Max_bunch = ui->spinBox_max_files_at_once->value();
	NrecentFiles = ui->spinBox_max_recent_files->value();
	NrecentDirs = ui->spinBox_max_recent_dirs->value();

	Maxfiles_increment = ui->spinBox_incr_files->value();
	Maxsubs_increment = ui->spinBox_incr_subs->value();
	Maxdisks_increment = ui->spinBox_incr_disks->value();
	Maxfiles_initial = ui->spinBox_init_files->value();
	Maxsubs_initial = ui->spinBox_init_subs->value();
	Maxdisks_initial = ui->spinBox_init_disks->value();

	Max_category_length = ui->spinBox_category_length->value();
	Max_comment_length = ui->spinBox_comment_length->value();
	Max_extra_length = ui->spinBox_extra_length->value();

	Index_hidden = ui->checkBox_IndexHidden->isChecked();
	EjectAfterIndexing = ui->checkBox_eject->isChecked();
	Skip_NTFS_mounted = ui->checkBox_skip_junctions->isChecked();
	Store_relative_path = ui->checkBox_relative_paths->isChecked();
	Store_arc_path = ui->checkBox_archive_paths->isChecked();
	Case_option_indexing = ui->comboBox_archive_member_case->currentIndex();
	Extract_diz = ui->comboBox_fileid_diz_comments->currentIndex();
	Always_diz_contents = ui->checkBox_always_diz->isChecked();
	Query_extract = ui->checkBox_ask_extract->isChecked();
	Ignore_arc_ext = ui->checkBox_IgnoreArcExtension->isChecked();
	Copy_across_subdirs = ui->checkBox_allow_subdir_change->isChecked();

	mainWindowPtr->setAppStyle();

	for(unsigned int i=0; i<NLIBTYPE; i++)
		{
		Extract[i].flag = dontExtract;
		if(extractContentsBox[i]->isChecked())
			Extract[i].flag |= extractMembers;
		if(TYPE_ARCHIVE == Extract[i].type || TYPE_SELF_EXTRACTING == Extract[i].type)
			{
			if(extractCommentBox[i]->isChecked())
				Extract[i].flag |= extractArcComment;
// Doesn't make sense to extract member contents unless member info is also extracted
			if(extractMemberCommentsBox[i]->isChecked() && extractContentsBox[i]->isChecked())
				Extract[i].flag |= extractMemberComments;
			}
		}
	}

void settingsDialog::slot_applyClicked()
	{
	apply_settings();
	saveWindowSettings();
	slot_cancelClicked();	// close window
	}

void settingsDialog::slot_saveClicked()
	{
	apply_settings();
	mainWindowPtr->recentFiles->resize(NrecentFiles);
	mainWindowPtr->recentDirs->resize(NrecentDirs);
	QSettings extractopts("WSS-DDC", "Wssindex\\ExtractOpts");
	for(unsigned int i=0; i<NLIBTYPE; i++)
		{
		switch(Extract[i].type)
			{
			case TYPE_ARCHIVE:
				extractopts.beginGroup("Archives");
				break;
			case TYPE_FONT:
				extractopts.beginGroup("Fonts");
				break;
			case TYPE_GRAPHICS:
				extractopts.beginGroup("Graphics");
				break;
			case TYPE_SELF_EXTRACTING:
				extractopts.beginGroup("Selfextracting");
				break;
			case TYPE_SOUND:
				extractopts.beginGroup("Sounds");
				break;
			case TYPE_MOVIE:
				extractopts.beginGroup("Videos");
				break;
			}
		extractopts.setValue(Extract[i].description, Extract[i].flag);
		extractopts.endGroup();
		}
	saveWindowSettings();
	Q_EMIT signal_saveSettings();
	slot_cancelClicked();	// close window
	}

void settingsDialog::addOptions()
	{
	QGridLayout *archivesLayout = new QGridLayout();
	QGridLayout *fontLayout = new QGridLayout();
	QGridLayout *graphicsLayout = new QGridLayout();
	QGridLayout *selfextractingLayout = new QGridLayout();
	QGridLayout *soundLayout = new QGridLayout();
	QGridLayout *videoLayout = new QGridLayout();

	int arch_row = 0;
	add_extract_title(archivesLayout, &arch_row, TYPE_ARCHIVE);

	int font_row = 0;
	add_extract_title(fontLayout, &font_row, TYPE_FONT);

	int graphics_row = 0;
	add_extract_title(graphicsLayout, &graphics_row, TYPE_GRAPHICS);

	int selfextracting_row = 0;
	add_extract_title(selfextractingLayout, &selfextracting_row, TYPE_SELF_EXTRACTING);

	int sound_row = 0;
	add_extract_title(soundLayout, &sound_row, TYPE_SOUND);

	int video_row = 0;
	add_extract_title(videoLayout, &video_row, TYPE_MOVIE);

	for(int i=0; i<NLIBTYPE; i++)
		{
		switch(Extract[i].type)
			{
			case TYPE_ARCHIVE:
				menu_spacer_lines(archivesLayout, i, &arch_row, true);
				break;
			case TYPE_FONT:
				menu_spacer_lines(fontLayout, i, &font_row, false);
				break;
			case TYPE_GRAPHICS:
				menu_spacer_lines(graphicsLayout, i, &graphics_row, false);
				break;
			case TYPE_SELF_EXTRACTING:
				menu_spacer_lines(selfextractingLayout, i, &selfextracting_row, true);
				break;
			case TYPE_SOUND:
				menu_spacer_lines(soundLayout, i, &sound_row, false);
				break;
			case TYPE_MOVIE:
				menu_spacer_lines(videoLayout, i, &video_row, false);
				break;
			}
		}

	add_toggle_button(archivesLayout, &arch_row, SLOT(slot_toggleArchMembers()), TYPE_ARCHIVE);
	add_toggle_button(fontLayout, &font_row, SLOT(slot_toggleFont()), TYPE_FONT);
	add_toggle_button(graphicsLayout, &graphics_row, SLOT(slot_toggleGraphics()), TYPE_GRAPHICS);
	add_toggle_button(selfextractingLayout, &selfextracting_row, SLOT(slot_toggleSelfextracting()), TYPE_SELF_EXTRACTING);
	add_toggle_button(soundLayout, &sound_row, SLOT(slot_toggleSound()), TYPE_SOUND);
	add_toggle_button(videoLayout, &video_row, SLOT(slot_toggleVideo()), TYPE_MOVIE);

	add_box(archivesLayout, ui->tab_archives, arch_row, tr("Options for extracting information from archive files"));
	add_box(fontLayout, ui->tab_fonts, font_row, tr("Options for extracting information from font files"));
	add_box(graphicsLayout, ui->tab_graphics, graphics_row, tr("Options for extracting information from graphics files"));
	add_box(selfextractingLayout, ui->tab_self_extracting, selfextracting_row, tr("Options for extracting information from self-extracting files"));
	add_box(soundLayout, ui->tab_sounds, sound_row, tr("Options for extracting information from sound files"));
	add_box(videoLayout, ui->tab_video, video_row, tr("Options for extracting information from video files"));
	}

void settingsDialog::add_extract_title(QGridLayout *glayout, int *row, int type)
	{
	QHBoxLayout *layout = new QHBoxLayout;
	switch(type)
		{
		case TYPE_ARCHIVE:
			{
			layout->addWidget(new QLabel(tr("Extension")));
			QLabel *mlabel = new QLabel(tr("Members"));
			mlabel->setAlignment(Qt::AlignCenter);
			layout->addWidget(mlabel);
			QLabel *alabel = new QLabel(tr("Archive\ncomment"));
			alabel->setAlignment(Qt::AlignCenter);
			layout->addWidget(alabel);
			QLabel *clabel = new QLabel(tr("Member\ncomments"));
			clabel->setAlignment(Qt::AlignCenter);
			layout->addWidget(clabel);
			}
			break;

		case TYPE_SELF_EXTRACTING:
			{
			layout->addWidget(new QLabel(tr("Compression\nmethod")));
			QLabel *mlabel = new QLabel(tr("Members"));
			mlabel->setAlignment(Qt::AlignCenter);
			layout->addWidget(mlabel);
			QLabel *alabel = new QLabel(tr("File\ncomment"));
			alabel->setAlignment(Qt::AlignCenter);
			layout->addWidget(alabel);
			QLabel *clabel = new QLabel(tr("Member\ncomments"));
			clabel->setAlignment(Qt::AlignCenter);
			layout->addWidget(clabel);
			}
			break;

		default:
			{
			layout->addWidget(new QLabel(tr("Extension")));
			QLabel *elabel = new QLabel(tr("Extract info"));
			elabel->setAlignment(Qt::AlignCenter);
			layout->addWidget(elabel);
			layout->addWidget(new QLabel());
			layout->addWidget(new QLabel());
			}
			break;
		}

	glayout->addLayout(layout, (*row)++, 0);
	}

void settingsDialog::menu_spacer_lines(QGridLayout *Layout, int i, int *row, bool is_archive)
	{
	QLabel *desc = new QLabel(Extract[i].description);
	QHBoxLayout *layout;
	if(1 == (*row % 3))
		{
		QLabel *line = new QLabel();
		line->setFixedHeight(3);	// 3 is smallest value that gives a visible line
		line->setFrameStyle(QFrame::HLine);
		line->setFrameShadow(QFrame::Sunken);
		layout = new QHBoxLayout();
		layout->addWidget(line);
		if(!is_archive)
			{
			QLabel *null_label = new QLabel();
			null_label->setFixedHeight(3);
			layout->addWidget(null_label);
			}
		Layout->addLayout(layout, (*row)++, 0);
		}
	layout = new QHBoxLayout();
	extractContentsBox[i] = new QCheckBox();
	extractContentsBox[i]->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);

	if(is_archive)
		{
		extractCommentBox[i] = new QCheckBox();
		extractCommentBox[i]->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
		if(!(Extract[i].valid_flag_values & extractArcComment))
			extractCommentBox[i]->setDisabled(true);
		extractMemberCommentsBox[i] = new QCheckBox();
		extractMemberCommentsBox[i]->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
		if(!(Extract[i].valid_flag_values & extractMemberComments))
			extractMemberCommentsBox[i]->setDisabled(true);
		}
	layout->addWidget(desc);
	if(!Extract[i].tooltip.isEmpty())
		desc->setToolTip(Extract[i].tooltip);
	add_centered_widget(layout, extractContentsBox[i]);
	if(is_archive)
		{
		add_centered_widget(layout, extractCommentBox[i]);
		add_centered_widget(layout, extractMemberCommentsBox[i]);
		}
	else
		{
		QLabel *null_label = new QLabel();
		null_label->setFixedHeight(3);
		layout->addWidget(null_label);
		null_label = new QLabel();
		null_label->setFixedHeight(3);
		layout->addWidget(null_label);
		}
	Layout->addLayout(layout, (*row)++, 0);
	if(extractInfo & Extract[i].flag)
		extractContentsBox[i]->setChecked(true);
	if(is_archive)
		{
		if(extractArcComment & Extract[i].flag)
			extractCommentBox[i]->setChecked(true);
		if(extractMemberComments & Extract[i].flag)
			extractMemberCommentsBox[i]->setChecked(true);
		connect(extractContentsBox[i], SIGNAL(toggled(bool)), this, SLOT(slot_validateArchiveSettings()));
		connect(extractMemberCommentsBox[i], SIGNAL(toggled(bool)), this, SLOT(slot_validateArchiveSettings()));
		}
	}

void settingsDialog::add_centered_widget(QHBoxLayout *layout, QWidget *box)
	{
	QHBoxLayout *cb = new QHBoxLayout();
	cb->addWidget(new QLabel());
	cb->addWidget(box);
	cb->addWidget(new QLabel());
	layout->addLayout(cb);
	}

void settingsDialog::slot_toggleArchMembers()
	{
	toggle_arch(0, TYPE_ARCHIVE);
	}

void settingsDialog::slot_toggleArchComments()
	{
	toggle_arch(1, TYPE_ARCHIVE);
	}

void settingsDialog::slot_toggleMemberComments()
	{
	toggle_arch(2, TYPE_ARCHIVE);
	}

void settingsDialog::toggle_arch(int column, int type)
	{
	QCheckBox *box = nullptr;	// initialized to quiet compiler warning
	bool new_state = false; // initialized to quiet compiler warning
	bool new_state_known = false;
	for(unsigned int i=0; i<NLIBTYPE; i++)
		{
		if(type != Extract[i].type)
			continue;
		switch(column)
			{
			case 0:
				box = extractContentsBox[i];
				break;
			case 1:
				box = extractCommentBox[i];
				break;
			case 2:
				box = extractMemberCommentsBox[i];
				break;
			}
		if(!box->isEnabled())
			continue;
		if(!new_state_known)
			{
			new_state = !box->isChecked();
			new_state_known = true;
			}
		box->setChecked(new_state);
		}
	}

void settingsDialog::slot_toggleFont()
	{
	toggle_boxes(TYPE_FONT);
	}

void settingsDialog::slot_toggleGraphics()
	{
	toggle_boxes(TYPE_GRAPHICS);
	}

void settingsDialog::slot_toggleSelfextracting()
	{
	toggle_arch(0, TYPE_SELF_EXTRACTING);
	}

void settingsDialog::slot_toggleSeComments()
	{
	toggle_arch(1, TYPE_SELF_EXTRACTING);
	}

void settingsDialog::slot_toggleSeMemberComments()
	{
	toggle_arch(2, TYPE_SELF_EXTRACTING);
	}

void settingsDialog::slot_toggleSound()
	{
	toggle_boxes(TYPE_SOUND);
	}

void settingsDialog::slot_toggleVideo()
	{
	toggle_boxes(TYPE_MOVIE);
	}

void settingsDialog::toggle_boxes(int boxtype)
	{
	bool new_state = false;	// initialized to quiet compiler warning
	bool new_state_known = false;

	for(unsigned int i=0; i<NLIBTYPE; i++)
		{
		if(boxtype != Extract[i].type)
			continue;
		if(!new_state_known)
			{
			new_state = !extractContentsBox[i]->isChecked();
			new_state_known = true;
			}
		extractContentsBox[i]->setChecked(new_state);
		}
	}

void settingsDialog::slot_validateArchiveSettings()
	{
// Don't try to read checkbox states before the checkboxes are created
	if(!checkboxes_created)
		return;
	for(unsigned int i=0; i<NLIBTYPE; i++)
		{
		if(TYPE_ARCHIVE != Extract[i].type && TYPE_SELF_EXTRACTING != Extract[i].type)
			continue;
		if(!extractContentsBox[i]->isChecked())
			extractMemberCommentsBox[i]->setChecked(false);
		}
	}

void settingsDialog::set_checkboxes()
	{
	for(unsigned int i=0; i<NLIBTYPE; i++)
		{
		extractContentsBox[i]->setChecked(Extract[i].flag & extractMembers);
		if(TYPE_ARCHIVE == Extract[i].type || TYPE_SELF_EXTRACTING == Extract[i].type)
			{
			extractCommentBox[i]->setChecked(Extract[i].flag & extractArcComment);
			extractMemberCommentsBox[i]->setChecked(Extract[i].flag & extractMemberComments);
			}
		}
	slot_validateArchiveSettings();
	}

void settingsDialog::add_toggle_button(QGridLayout *layout, int *row, const char *target_slot, int type)
	{
	QPushButton *toggleButton1 = new QPushButton(tr("&Toggle"));
	toggleButton1->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	QPushButton *toggleButton2 = new QPushButton(tr("T&oggle"));
	toggleButton2->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	QPushButton *toggleButton3 = new QPushButton(tr("To&ggle"));
	toggleButton3->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	QHBoxLayout *buttonLayout = new QHBoxLayout();
	buttonLayout->addWidget(new QLabel());
	add_centered_widget(buttonLayout, toggleButton1);
	if(TYPE_ARCHIVE == type || TYPE_SELF_EXTRACTING == type)
		{
		add_centered_widget(buttonLayout, toggleButton2);
		add_centered_widget(buttonLayout, toggleButton3);
		}
	else
		{
		buttonLayout->addWidget(new QLabel());
		buttonLayout->addWidget(new QLabel());
		}
	layout->addLayout(buttonLayout, (*row)++, 0);
	connect(toggleButton1, SIGNAL(clicked()), this, target_slot);
	if(TYPE_ARCHIVE == type)
		{
		connect(toggleButton2, SIGNAL(clicked()), this, SLOT(slot_toggleArchComments()));
		connect(toggleButton3, SIGNAL(clicked()), this, SLOT(slot_toggleMemberComments()));
		}
	if(TYPE_SELF_EXTRACTING == type)
		{
		connect(toggleButton2, SIGNAL(clicked()), this, SLOT(slot_toggleSeComments()));
		connect(toggleButton3, SIGNAL(clicked()), this, SLOT(slot_toggleSeMemberComments()));
		}
	}

void settingsDialog::add_box(QGridLayout *glayout, QWidget *tab, int row, QString boxText)
	{
	QSpacerItem *vertspace = new QSpacerItem(0, 1000, QSizePolicy::Minimum, QSizePolicy::Maximum);
	glayout->addItem(vertspace, row, 0);

	QGroupBox *groupBox = new QGroupBox(boxText);
	groupBox->setLayout(glayout);
	QVBoxLayout *boxLayout = new QVBoxLayout();
	boxLayout->addWidget(groupBox);
	tab->setLayout(boxLayout);
	}

void settingsDialog::showEvent(QShowEvent *e)
	{
	restoreWindowSettings();
	populateIndexingOptions();
	populateGeneralOptions();
	populateMemoryOptions();
	set_checkboxes();
// Scroller is drawn transparent, see
// https://bugreports.qt-project.org/browse/QTBUG-21808
// Fixed with style setting background color (which depends on overall style, so grab color from upper right-hand corner)
	QPixmap qPix = this->grab();
	QImage image(qPix.toImage());
	QColor color(image.pixel(ui->centralwidget->width()-1, 1));
	QString tabBackColor = color.name(QColor::HexRgb);
	ui->settingsDialogWidget->setStyleSheet(tr("QTabWidget::pane {" BLUE_WASH "} "
			"QTabBar QToolButton {background-color: %1; margin-top: 1px; margin-bottom: 0px; border: none;}").arg(tabBackColor));
	QWidget::showEvent(e); // execute the normal showevent
	}
