// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "Dialogs/simplesortdialog.h"
#include "ui_simplesortdialog.h"
#include "ui_mainwindow.h"
#include "Dialogs/mainwindow.h"
#include "OtherWssi/initialpos.h"
#include "OtherWssi/wssiproto.h"

SimpleSortDialog::SimpleSortDialog(QWidget *parent)
	: QMainWindow(parent), sd(new Ui::SimpleSortDialog)
	{
	sd->setupUi(this);
	this->setWindowFlags(MyWindowFlags);
	restoreSettings();
	sd->pushButton_sort_fn_ext->setDefault(true);

	connect(this, SIGNAL(signal_sortNeeded()),
		mainWindowPtr, SLOT(slot_sortdatabase()));

	connect(sd->pushButton_sort_advanced, SIGNAL(clicked()),
		mainWindowPtr, SLOT(slot_pushButtonFullSortClicked()));
	connect(sd->pushButton_sort_close, SIGNAL(clicked()),
		this, SLOT(slot_pushButtonSortCloseClicked()));
	connect(sd->pushButton_sort_ext_fn, SIGNAL(clicked()),
		this, SLOT(slot_pushButtonSortExtFnClicked()));
	connect(sd->pushButton_sort_fn_ext, SIGNAL(clicked()),
		this, SLOT(slot_pushButtonSortFnExtClicked()));
	connect(sd->pushButton_sort_timestamp_fn_ext, SIGNAL(clicked()),
		this, SLOT(slot_pushButtonSortTimestampFnExtClicked()));

	connect(this, SIGNAL(signal_keyToTable(int,int)),
		parent, SLOT(slot_keyToTable(int,int)));
	connect(this, SIGNAL(signal_wheelToTable(QPoint,int,Qt::MouseButtons,Qt::KeyboardModifiers,Qt::Orientation)),
		mainWindowPtr, SLOT(slot_wheelToTable(QPoint,int,Qt::MouseButtons,Qt::KeyboardModifiers,Qt::Orientation)));

	connect(mainWindowPtr, SIGNAL(signal_close_sorts()),
			  this, SLOT(close()));
	}

SimpleSortDialog::~SimpleSortDialog()
	{
	delete sd;
	}

void SimpleSortDialog::restoreSettings()
	{
	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	if(settings.contains("SimpleSortSize"))
		{
		QSize size= settings.value("SimpleSortSize").toSize();
		resize(size);
		}
	if(settings.contains("SimpleSortPos"))
		{
		QPoint pos = settings.value("SimpleSortPos").toPoint();
		move(pos);
		forceOnscreen(this);
		}
	else
		{
// frameGeometry is the same as geometry until the dialog is displayed
		this->show();
		move(mainWindowPtr->offCenter(this->frameGeometry(), SIMPLE_SORT_OFFSET));
		}
	}

// ------------------------------- Sort buttons -------------------------------
void SimpleSortDialog::slot_pushButtonSortFnExtClicked()
	{
	if(0 == Noselect)
		return;
	int sortdir = sd->checkBox_reverse_sort->isChecked() ? -1 : 1;
	Nkey = 2;
	SortKey[0] = SortKey_FN;
	Sortdir[0] = sortdir;
	SortKey[1] = SortKey_EXT;
	Sortdir[1] = 1;
	Q_EMIT signal_sortNeeded();
	}

void SimpleSortDialog::slot_pushButtonSortExtFnClicked()
	{
	if(0 == Noselect)
		return;
	int sortdir = sd->checkBox_reverse_sort->isChecked() ? -1 : 1;
	Nkey = 2;
	SortKey[0] = SortKey_EXT;
	Sortdir[0] = sortdir;
	SortKey[1] = SortKey_FN;
	Sortdir[1] = 1;
	Q_EMIT signal_sortNeeded();
	}

void SimpleSortDialog::slot_pushButtonSortTimestampFnExtClicked()
	{
	if(0 == Noselect)
		return;
	int sortdir = sd->checkBox_reverse_sort->isChecked() ? -1 : 1;
	Nkey = 3;
	SortKey[0] = SortKey_TTAG;
	Sortdir[0] = sortdir;
	SortKey[1] = SortKey_FN;
	Sortdir[1] = 1;
	SortKey[2] = SortKey_EXT;
	Sortdir[2] = 1;
	Q_EMIT signal_sortNeeded();
	}

void SimpleSortDialog::slot_pushButtonSortCloseClicked()
	{
	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	settings.setValue("SimpleSortPos", pos());
	settings.setValue("SimpleSortSize", size());
	this->close();
	}

// ------------------------------ Event handlers ------------------------------
void SimpleSortDialog::keyPressEvent(QKeyEvent *event)
	{
	switch(event->key())
		{
		case Qt::Key_Home:
		case Qt::Key_End:
			if(!(Qt::ControlModifier & event->modifiers()))
				break;
			Q_FALLTHROUGH();
		case Qt::Key_PageDown:
		case Qt::Key_PageUp:
			Q_EMIT signal_keyToTable(event->key(), static_cast<int>(event->modifiers()));
			break;
		case Qt::Key_Escape:
			this->close();
			break;
		default:
			return;
		}
	event->ignore();
	return;
	}

void SimpleSortDialog::wheelEvent(QWheelEvent *event)
	{
	Q_EMIT signal_wheelToTable(event->pos(), event->delta(), event->buttons(),
										event->modifiers(), event->orientation());
	event->ignore();
	return;
	}

void SimpleSortDialog::closeEvent(QCloseEvent *event)
	{
	event->accept();
	}
