// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "Dialogs/specialquerydialog.h"
#include "ui_specialquerydialog.h"
#include "OtherWssi/wssindex.h"
#include "Dialogs/mainwindow.h"
#include "OtherWssi/initialpos.h"
#include "OtherWssi/wssiproto.h"

#include <QtDebug>
#include <QMessageBox>

SpecialQueryDialog::SpecialQueryDialog(QWidget *parent)
	: QMainWindow(parent), spq(new Ui::SpecialQueryDialog)
	{
	spq->setupUi(this);
	spq->comboBox_vols->setModel(mainWindowPtr->Vcm);
	spq->comboBox_vols->setCurrentIndex(-1);
	this->setWindowFlags(MyWindowFlags);
	restoreSettings();

	mainWindowPtr->Vcm->diskListValid = false;

	connect(this, SIGNAL(signal_sortNeeded()),
		mainWindowPtr, SLOT(slot_sortdatabase()));
	connect(this, SIGNAL(signal_updateDisplay(bool)),
		mainWindowPtr, SLOT(slot_updateDisplay(bool)));
	connect(spq->pushButton_close, SIGNAL(clicked()),
		this, SLOT(slot_pushButtonCloseClicked()));
	connect(spq->pushButton_query_dup, SIGNAL(clicked()),
		this, SLOT(slot_pushButtonQueryDupClicked()));
	connect(spq->pushButton_query_uniq, SIGNAL(clicked()),
		this, SLOT(slot_pushButtonQueryUniqClicked()));
	connect(this, SIGNAL(signal_keyToTable(int,int)),
		mainWindowPtr, SLOT(slot_keyToTable(int,int)));
	connect(this, SIGNAL(signal_wheelToTable(QPoint,int,Qt::MouseButtons,Qt::KeyboardModifiers,Qt::Orientation)),
		mainWindowPtr, SLOT(slot_wheelToTable(QPoint,int,Qt::MouseButtons,Qt::KeyboardModifiers,Qt::Orientation)));
	connect(mainWindowPtr, SIGNAL(signal_close_queries()),
			  this, SLOT(close()));
	connect(mainWindowPtr, SIGNAL(signal_diskListChanged()),
			  this, SLOT(slot_diskListChanged()));
	connect(spq->pushButton_query_backup, SIGNAL(clicked()),
				this, SLOT(slot_pushButtonQueryBackupClicked()));
	}

SpecialQueryDialog::~SpecialQueryDialog()
	{
	delete spq;
	}

void SpecialQueryDialog::restoreSettings()
	{
	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	if(settings.contains("SpecialQuerySize"))
		{
		QSize size = settings.value("SpecialQuerySize").toSize();
		resize(size);
		}
	if(settings.contains("SpecialQueryPos"))
		{
		QPoint pos = settings.value("SpecialQueryPos").toPoint();
		move(pos);
		forceOnscreen(this);
		}
	else
		{
// frameGeometry is the same as geometry until the dialog is displayed
		this->show();
		move(mainWindowPtr->offCenter(this->frameGeometry(), SPECIAL_OFFSET));
		}
	}

// ------------------------------ Worker routines -----------------------------
void SpecialQueryDialog::dupUnique(bool finding_duplicates,
		bool same_size_required, bool same_time_required)
	{
	Nkey = 6;
	SortKey[0] = SortKey_FN;	Sortdir[0] = 1;		// file name
	SortKey[1] = SortKey_EXT;	Sortdir[1] = 1;		// extension
	SortKey[2] = SortKey_SIZE;	Sortdir[2] = 1;		// size
	SortKey[3] = SortKey_TTAG;	Sortdir[3] = 1;		// timestamp
	SortKey[4] = SortKey_DISK;	Sortdir[4] = 1;		// disk
	SortKey[5] = SortKey_SUB;	Sortdir[5] = 1;		// subdirectory
	if(!spq->checkBox_only_displayed->isChecked())
		selectAllUnsorted();
	mainWindowPtr->hideVolSub(true);
	Q_EMIT signal_sortNeeded();
	bool ignoreOneHour = spq->checkBox_ignoreOneHour->isChecked();
	unsigned long current = Sortindx[0];
	struct filerec *dr_current = Dr + current;
	unsigned long next = 0;
	for(unsigned long i=0; i<Noselect; i++)
		{
		struct filerec *dr_previous = dr_current;
		current = Sortindx[i];
		dr_current = Dr + current;
// same as following one?
		if(i+1 < Noselect)
			{
			next = Sortindx[i+1];
			struct filerec *dr_next = Dr + next;
			if((!same_size_required || (dr_current->Dsize == dr_next->Dsize)) &&
				(!same_time_required ||
				 (timeEqual(dr_current->Ddate, dr_current->Dtime, dr_next->Ddate, dr_next->Dtime, ignoreOneHour))))
				{
				if(0 == simpleCollator->compare(dr_current->Lfn, dr_next->Lfn))
					{
// It's a duplicate, exclude if looking for unique names
					if(!finding_duplicates)
						Sortindx[i] = Nofile;
					continue;
					}
				}
			}
// same as previous one?
		if(i > 0)
			{
			if((!same_size_required || (dr_current->Dsize == dr_previous->Dsize)) &&
				(!same_time_required ||
				 (timeEqual(dr_current->Ddate, dr_current->Dtime, dr_previous->Ddate, dr_previous->Dtime, ignoreOneHour))))
				{
				if(0 == simpleCollator->compare(dr_current->Lfn, dr_previous->Lfn))
					{
// It's a duplicate, exclude if looking for unique names
					if(!finding_duplicates)
						Sortindx[i] = Nofile;
					continue;
					}
				}
			}
// Unique
		if(finding_duplicates)
			Sortindx[i] = Nofile;
		}
	mainWindowPtr->sortListPurge();
	}

// ------------------------------ Action buttons ------------------------------
void SpecialQueryDialog::slot_pushButtonQueryDupClicked()
	{
	if(0 == Diskno)
		return;
	dupUnique(true, spq->checkBox_dup_size->isChecked(),
				  spq->checkBox_dup_time->isChecked());
	}

void SpecialQueryDialog::slot_pushButtonQueryUniqClicked()
	{
	if(0 == Diskno)
		return;
	dupUnique(false, spq->checkBox_uniq_size->isChecked(),
				  spq->checkBox_uniq_time->isChecked());
	}

void SpecialQueryDialog::slot_pushButtonQueryBackupClicked()
	{
	if(0 == Diskno)
		return;
	int index = spq->comboBox_vols->currentIndex();
	if(index < 0)
		return;
	if(!spq->checkBox_only_displayed->isChecked())
		selectAllUnsorted();
	Keydisk = mainWindowPtr->Vcm->vptr[index];
	Nkey = 4;
	SortKey[0] = SortKey_FN;		Sortdir[0] = 1;	// file name
	SortKey[1] = SortKey_EXT;	Sortdir[1] = 1;	// extension
	SortKey[2] = SORTDIM;			Sortdir[2] = 1;	// key disk first
	SortKey[3] = SortKey_TTAG;	Sortdir[3] = -1;	// date & time, backwards
	Q_EMIT signal_sortNeeded();
	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
	for(unsigned int i=0; i<Noselect; i++)
		{
		struct filerec *dr_k = Dr + Sortindx[i];

		if(dr_k->Volno != Keydisk)
			{
			Sortindx[i] = Nofile;
			continue;
			}

		if(isDotID(dr_k))
			{
			Sortindx[i] = Nofile;
			continue;
			}

		for(unsigned int l=i+1; l<Noselect; l++)
			{
			struct filerec *dr_m = Dr + Sortindx[l];
			if(dr_m->Volno == Keydisk)
				continue;
			if(namecomp(dr_m, dr_k, COMPARE_FILENAME, COMPARE_EXTENSION))
				break;
			if(dr_m->Ddate < dr_k->Ddate)
				break;
			if(dr_m->Ddate == dr_k->Ddate && (dr_m->Dtime.secsTo(dr_k->Dtime) > 0))
				break;
			Sortindx[i] = Nofile;
			break;
			}
		}
	mainWindowPtr->sortListPurge();
	QApplication::restoreOverrideCursor();
	}

void SpecialQueryDialog::slot_pushButtonCloseClicked()
	{
	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	settings.setValue("SpecialQueryPos", pos());
	settings.setValue("SpecialQuerySize", size());
	this->close();
	}

void SpecialQueryDialog::slot_diskListChanged()
	{
	mainWindowPtr->Vcm->diskListValid = false;
	spq->comboBox_vols->setCurrentIndex(-1);
	}

// ------------------------------ Event handlers ------------------------------
void SpecialQueryDialog::closeEvent(QCloseEvent *event)
	{
	event->accept();
	}

void SpecialQueryDialog::keyPressEvent(QKeyEvent *event)
	{
	switch(event->key())
		{
		case Qt::Key_Home:
		case Qt::Key_End:
			if(!(Qt::ControlModifier & event->modifiers()))
				break;
			Q_FALLTHROUGH();
		case Qt::Key_PageDown:
		case Qt::Key_PageUp:
			Q_EMIT signal_keyToTable(event->key(), static_cast<int>(event->modifiers()));
			break;
		case Qt::Key_Escape:
			this->close();
			break;
// There is no obvious default button in this dialog, so
// don't handle enter and return keys.
		default:
			return;
		}
	event->ignore();
	}

void SpecialQueryDialog::wheelEvent(QWheelEvent *event)
	{
	Q_EMIT signal_wheelToTable(event->pos(), event->delta(), event->buttons(),
										event->modifiers(), event->orientation());
	event->ignore();
	return;
	}
