// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "Dialogs/mainwindow.h"
#include "Dialogs/tabledesigndialog.h"
#include "ui_tabledesigndialog.h"
#include "Dialogs/tablepreview.h"
#include "OtherWssi/tablelayout.h"
#include "OtherWssi/wssiproto.h"
#include "3rdparty/clickablelabel.h"

TableDesignDialog::TableDesignDialog(MainWindow *parent)
	: QMainWindow(parent), td(new Ui::TableDesignDialog)
	{
	td->setupUi(this);
	this->setWindowFlags(MyWindowFlags);
	tdparent = parent;
	restoreSettings();

// Find the various controls created in the designer so we can reference them
// from code.  Controls must follow naming convention xxx...NN for this to work.
//	Set fieldname[x] = td->fieldname_0x;
	QList<ClickableLabel*> labels = this->findChildren<ClickableLabel*>(QRegularExpression(QFA("fieldname*")));
	foreach (ClickableLabel* label, labels)
		{
		fieldname[label->objectName().toLatin1().right(2).toInt()] = label;
		connect(label, SIGNAL(clicked()), this, SLOT(slot_fieldnameLabelClicked()));
		}

//	Set rowspin[x] = td->spinBox_row_0x;
	QList<QSpinBox*> rSpins = this->findChildren<QSpinBox*>(QRegularExpression(QFA("spinBox_row*")));
	foreach (QSpinBox* rSpin, rSpins)
		rowspin[rSpin->objectName().toLatin1().right(2).toInt()] = rSpin;

//	Set heightspin[x] = td->spinBox_height_0x;
	QList<QSpinBox*> hSpins = this->findChildren<QSpinBox*>(QRegularExpression(QFA("spinBox_height*")));
	foreach (QSpinBox* hSpin, hSpins)
		heightspin[hSpin->objectName().toLatin1().right(2).toInt()] = hSpin;

//	Set colspin[x] = td->spinBox_col_0x;
	QList<QSpinBox*> cSpins = this->findChildren<QSpinBox*>(QRegularExpression(QFA("spinBox_col*")));
	foreach (QSpinBox* cSpin, cSpins)
		colspin[cSpin->objectName().toLatin1().right(2).toInt()] = cSpin;

//	Set widthspin[x] = td->spinBox_width_0x;
	QList<QSpinBox*> wSpins = this->findChildren<QSpinBox*>(QRegularExpression(QFA("spinBox_width*")));
	foreach (QSpinBox* wSpin, wSpins)
		widthspin[wSpin->objectName().toLatin1().right(2).toInt()] = wSpin;

//	Set fieldcheck[x] = td->checkBox_0x;
	QList<QCheckBox*> checkBoxes = this->findChildren<QCheckBox*>(QRegularExpression(QFA("checkBox*")));
	foreach (QCheckBox* checkBox, checkBoxes)
		fieldcheck[checkBox->objectName().toLatin1().right(2).toInt()] = checkBox;

//	Set justcombo[x] = td->comboBox_just_0x;
	QList<QComboBox*> comboBoxes = this->findChildren<QComboBox*>(QRegularExpression(QFA("comboBox_just*")));
	foreach (QComboBox* comboBox, comboBoxes)
		justcombo[comboBox->objectName().toLatin1().right(2).toInt()] = comboBox;

	connect(td->pushButton_accept, SIGNAL(clicked()),
		this, SLOT(slot_pushButtonAcceptClicked()));
	connect(td->pushButton_cancel, SIGNAL(clicked()),
		this, SLOT(slot_pushButtonCancelClicked()));
	connect(this, SIGNAL(signal_tableRedesign(bool)),
		parent, SLOT(slot_setTableHeaders(bool)));
	connect(td->pushButton_restore, SIGNAL(clicked()),
		this, SLOT(slot_pushButtonRestoreClicked()));
	connect(td->pushButton_save, SIGNAL(clicked()),
		this, SLOT(slot_pushButtonSaveClicked()));

	connect(td->pushButton_simple, SIGNAL(clicked()),
		this, SLOT(slot_layoutSimple()));
	connect(td->pushButton_simple_pv, SIGNAL(clicked()),
		this, SLOT(slot_layoutSimplePv()));
	connect(td->pushButton_complete_1r, SIGNAL(clicked()),
		this, SLOT(slot_layoutComplete1r()));
	connect(td->pushButton_complete_2r, SIGNAL(clicked()),
		this, SLOT(slot_layoutComplete2r()));

	connect(this, SIGNAL(signal_keyToTable(int,int)),
		parent, SLOT(slot_keyToTable(int,int)));

	setup_preview();
	set_control_values();
	}

TableDesignDialog::~TableDesignDialog()
	{
	delete td;
	}

void TableDesignDialog::restoreSettings()
	{
	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	if(settings.contains("TableDesignPos"))
		{
		QPoint pos = settings.value("TableDesignPos").toPoint();
		move(pos);
		forceOnscreen(this);
		}
	if(settings.contains("TableDesignSize"))
		{
		QSize size= settings.value("TableDesignSize").toSize();
		resize(size);
		}
	}

// ------------------------------ Setup routines ------------------------------
void TableDesignDialog::set_control_values()
	{
	tp->set_preview_update_enabled(false);
	for(int i=0; i<NFIELDS; i++)
		{
		fieldcheck[i]->setChecked(fieldlayout[i].enabled);
		rowspin[i]->setValue(1+fieldlayout[i].row);
		colspin[i]->setValue(1+fieldlayout[i].column);
		heightspin[i]->setValue(fieldlayout[i].height);
		widthspin[i]->setValue(fieldlayout[i].width);
		justcombo[i]->clear();
		justcombo[i]->addItem(QFA("Left"));
		justcombo[i]->addItem(QFA("Center"));
		justcombo[i]->addItem(QFA("Right"));
		justcombo[i]->setCurrentIndex(tdparent->justIntFromEnum(fieldlayout[i].alignment));
		}
	tp->set_preview_update_enabled(true);
	Q_EMIT signal_drawPreview();
	}

void TableDesignDialog::setup_preview()
	{
	tp = new TablePreview(tdparent, this);

	for(int i=0; i<NFIELDS; i++)
		{
		connect(fieldcheck[i], SIGNAL(stateChanged(int)),
			tp, SLOT(slot_drawPreview()));
		connect(rowspin[i], SIGNAL(valueChanged(int)),
			tp, SLOT(slot_drawPreview()));
		connect(colspin[i], SIGNAL(valueChanged(int)),
			tp, SLOT(slot_drawPreview()));
		connect(heightspin[i], SIGNAL(valueChanged(int)),
			tp, SLOT(slot_drawPreview()));
		connect(widthspin[i], SIGNAL(valueChanged(int)),
			tp, SLOT(slot_drawPreview()));
		connect(justcombo[i], SIGNAL(currentIndexChanged(int)),
			tp, SLOT(slot_drawPreview()));
		}

	tp->setAttribute(Qt::WA_DeleteOnClose);
	tp->show();

	connect(this, SIGNAL(signal_drawPreview()),
		tp, SLOT(slot_drawPreview()));
	connect(this, SIGNAL(signal_closePreview(bool)),
		tp, SLOT(slot_closePreview(bool)));
	}

// ---------------------------------- Presets ---------------------------------
void TableDesignDialog::slot_layoutSimple()
	{
	for(int i=0; i<NFIELDS; i++)
		{
		fieldcheck[i]->setChecked(false);
		rowspin[i]->setValue(1);
		heightspin[i]->setValue(1);
		widthspin[i]->setValue(1);
		justcombo[i]->setCurrentIndex(0);
		}

// long file name
	fieldcheck[0]->setChecked(true);
	colspin[0]->setValue(1);

// size (right justified)
	fieldcheck[5]->setChecked(true);
	colspin[5]->setValue(2);
	justcombo[5]->setCurrentIndex(2);

// timestamp (centered)
	fieldcheck[6]->setChecked(true);
	colspin[6]->setValue(3);
	justcombo[6]->setCurrentIndex(1);

	Q_EMIT signal_drawPreview();
	}

void TableDesignDialog::slot_layoutSimplePv()
	{
	for(int i=0; i<NFIELDS; i++)
		{
		fieldcheck[i]->setChecked(false);
		rowspin[i]->setValue(1);
		heightspin[i]->setValue(1);
		widthspin[i]->setValue(1);
		justcombo[i]->setCurrentIndex(0);
		}

// long file name
	fieldcheck[0]->setChecked(true);
	colspin[0]->setValue(1);

// size (right justified)
	fieldcheck[5]->setChecked(true);
	colspin[5]->setValue(2);
	justcombo[5]->setCurrentIndex(2);

// timestamp (centered)
	fieldcheck[6]->setChecked(true);
	colspin[6]->setValue(3);
	justcombo[6]->setCurrentIndex(1);

// volume
	fieldcheck[2]->setChecked(true);
	colspin[2]->setValue(4);

// subdirectory
	fieldcheck[3]->setChecked(true);
	colspin[3]->setValue(5);

	Q_EMIT signal_drawPreview();
	}

void TableDesignDialog::slot_layoutComplete1r()
	{
	for(int i=0; i<NFIELDS; i++)
		{
		fieldcheck[i]->setChecked(true);
		rowspin[i]->setValue(1);
		heightspin[i]->setValue(1);
		widthspin[i]->setValue(1);
		justcombo[i]->setCurrentIndex(0);
		}

// long file name
	colspin[0]->setValue(1);

// extension
	colspin[1]->setValue(2);

// volume
	colspin[2]->setValue(3);

// subdirectory
	colspin[3]->setValue(4);

// archive
	colspin[4]->setValue(5);

// size (right justified)
	colspin[5]->setValue(6);
	justcombo[5]->setCurrentIndex(2);

// timestamp (centered)
	colspin[6]->setValue(7);
	justcombo[6]->setCurrentIndex(1);

// category
	colspin[7]->setValue(8);

// comment
	colspin[8]->setValue(9);

// extra info
	colspin[9]->setValue(10);

	Q_EMIT signal_drawPreview();
	}

void TableDesignDialog::slot_layoutComplete2r()
	{
	for(int i=0; i<NFIELDS; i++)
		{
		fieldcheck[i]->setChecked(true);
		rowspin[i]->setValue(1);
		heightspin[i]->setValue(1);
		widthspin[i]->setValue(1);
		justcombo[i]->setCurrentIndex(0);
		}

// long file name
	colspin[0]->setValue(1);

// extension
	colspin[1]->setValue(2);

// volume
	colspin[2]->setValue(5);

// subdirectory
	rowspin[3]->setValue(2);
	colspin[3]->setValue(3);

// archive
	rowspin[4]->setValue(2);
	colspin[4]->setValue(4);

// size (right justified)
	colspin[5]->setValue(3);
	justcombo[5]->setCurrentIndex(2);

// timestamp (centered)
	colspin[6]->setValue(4);
	justcombo[6]->setCurrentIndex(1);

// category
	rowspin[7]->setValue(2);
	colspin[7]->setValue(2);

// comment
	rowspin[8]->setValue(2);
	colspin[8]->setValue(5);

// extra info
	rowspin[9]->setValue(2);
	colspin[9]->setValue(1);

	Q_EMIT signal_drawPreview();
	}

// ------------------------------- Save/Restore -------------------------------
void TableDesignDialog::slot_pushButtonSaveClicked()
	{
// The settings that are saved here are the ones in the design form.
// They may not be the ones that are used unless the accept button is clicked.

	if(!consistent)
		{
		QMessageBox msgBox;
		msgBox.setText(tr("<html><head/><body><p>The table design is not consistent.  You can save"
			" the design for later editing, but it will not be loaded automatically"
			" and at program startup you will get a warning that it was skipped.<p>"
			"Do you want to save this table design?</p></body></html>"));
		msgBox.setStandardButtons(QMessageBox::Save);
		QPushButton *dontSaveButton = msgBox.addButton(tr("Don't save"), QMessageBox::ActionRole);
		msgBox.setDefaultButton(dontSaveButton);
		msgBox.exec();
		if(msgBox.clickedButton() == dontSaveButton)
			return;
		}

	QSettings settings("WSS-DDC", "Wssindex");
	settings.beginWriteArray("TableFormat");
	for (int i=0; i<NFIELDS; ++i)
		{
		settings.setArrayIndex(i);
		settings.setValue("enabled", fieldcheck[i]->isChecked());
		settings.setValue("row", rowspin[i]->value());
		settings.setValue("column", colspin[i]->value());
		settings.setValue("height", heightspin[i]->value());
		settings.setValue("width", widthspin[i]->value());
		settings.setValue(("justification"), justcombo[i]->currentIndex());
		}
	settings.endArray();
	}

void TableDesignDialog::slot_pushButtonRestoreClicked()
	{
	QSettings settings("WSS-DDC", "Wssindex");
	if(settings.contains("TableFormat/10/enabled"))
		{
		settings.beginReadArray("TableFormat");
		for (int i=0; i<NFIELDS; ++i)
			{
			settings.setArrayIndex(i);
			fieldcheck[i]->setChecked(settings.value("enabled", false).toBool());
// setValue forces these spinbox values to be within range
			rowspin[i]->setValue(settings.value("row", 1).toInt());
			colspin[i]->setValue(settings.value("column", 1).toInt());
			heightspin[i]->setValue(settings.value("height", 1).toInt());
			widthspin[i]->setValue(settings.value("width", 1).toInt());
			justcombo[i]->setCurrentIndex(settings.value("justification", 0).toInt());
			}
		settings.endArray();
		}
	else
		QMessageBox::warning(this, tr("Warning"), tr("No saved table design was found"));
	}

// ------------------------------ Action buttons ------------------------------
// Label next to checkbox has been clicked.  We know the lable name has the format
// fieldname_nn where nn ranges from 00-09, so look at the last character to tell
// which label was clicked.  Then, toggle the associated checkbox.
void TableDesignDialog::slot_fieldnameLabelClicked()
	{
	ClickableLabel *cl = static_cast<ClickableLabel *>(sender());
	int boxno = cl->objectName().rightRef(1).toInt();
	fieldcheck[boxno]->toggle();
	}

void TableDesignDialog::slot_pushButtonAcceptClicked()
	{
	if(!consistent)
		{
		QMessageBox::warning(this, tr("Error"), tr("Table design has problems.  "
			"Any red cells indicate more than one value is assigned to the cell."));
		}
	else
		{
		rowsPerItem = 1;
		colsPerItem = 1;
		for(int i=0; i<NFIELDS; i++)
			{
			fieldlayout[i].enabled = fieldcheck[i]->isChecked();
			fieldlayout[i].row = rowspin[i]->value()-1;
			fieldlayout[i].column = colspin[i]->value()-1;
			fieldlayout[i].height = heightspin[i]->value();
			fieldlayout[i].width = widthspin[i]->value();
			fieldlayout[i].alignment = tdparent->justEnumFromInt(justcombo[i]->currentIndex());

			if(fieldlayout[i].enabled)
				{
				rowsPerItem = qMax(rowsPerItem,
											fieldlayout[i].row+fieldlayout[i].height);
				colsPerItem = qMax(colsPerItem,
											fieldlayout[i].column+fieldlayout[i].width);
				}
			}
		tdparent->set_editable_cells();
		tdparent->fileBalance->colWidthsValid = false;
		Q_EMIT signal_tableRedesign(true);
		slot_pushButtonCancelClicked();
		}
	}

void TableDesignDialog::slot_pushButtonCancelClicked()
	{
	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	settings.setValue("TableDesignPos", pos());
	settings.setValue("TableDesignSize", size());
	Q_EMIT signal_closePreview(true);
	this->close();
	}

// ------------------------------ Event handlers ------------------------------
void TableDesignDialog::closeEvent(QCloseEvent *event)
	{
	Q_EMIT signal_closePreview(false);
	event->accept();
	}

void TableDesignDialog::keyPressEvent(QKeyEvent *event)
	{
	switch(event->key())
		{
		case Qt::Key_Home:
		case Qt::Key_End:
			if(!(Qt::ControlModifier & event->modifiers()))
				break;
			Q_FALLTHROUGH();
		case Qt::Key_PageDown:
		case Qt::Key_PageUp:
			Q_EMIT signal_keyToTable(event->key(), static_cast<int>(event->modifiers()));
			break;
		case Qt::Key_Escape:
			this->close();
			break;
		default:
			return;
		}
	event->ignore();
	return;
	}
