// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "OtherWssi/wssiproto.h"
#include "Dialogs/tabledesigndialog.h"
#include "Dialogs/tablepreview.h"
#include "ui_tablepreview.h"
#include "Dialogs/mainwindow.h"

#include <QCloseEvent>
#include <QSettings>
#include <QtDebug>

TablePreview::TablePreview(MainWindow *parent, TableDesignDialog *designparent)
	: QMainWindow(parent), tp(new Ui::TablePreview)
	{
	ok_to_close = false;
	tp->setupUi(this);
	resizeTimer = new QTimer(this);
	this->setWindowFlags(MyWindowFlags);
	restoreSettings();
	design = designparent;
	mw = parent;
	previewBalance = new balanceColumns(tp->tableWidget_preview);

	connect(this, SIGNAL(signal_keyToTable(int,int)),
		parent, SLOT(slot_keyToTable(int,int)));
	connect(resizeTimer, SIGNAL(timeout()), this, SLOT(slot_resizeTimeout()));
	}

TablePreview::~TablePreview()
	{
	delete resizeTimer;
	delete previewBalance;
	delete tp;
	}

void TablePreview::restoreSettings()
	{
	QSettings settings("WSS-DDC", "Wssindex\\Geometry");
	if(settings.contains("PreviewGeometrySize"))
		{
		QSize size = settings.value("PreviewGeometrySize").toSize();
		resize(size);
		}
	if(settings.contains("PreviewGeometryPos"))
		{
		QPoint pos = settings.value("PreviewGeometryPos").toPoint();
		move(pos);
		forceOnscreen(this);
		}
	}

// -------------- Slots receiving commands from tabledesigndialog -------------
void TablePreview::slot_closePreview(bool save_settings)
	{
	if(save_settings)
		{
		QSettings settings("WSS-DDC", "Wssindex\\Geometry");
		settings.setValue("PreviewGeometryPos", pos());
		settings.setValue("PreviewGeometrySize", size());
		}
	ok_to_close = true;
	this->close();
	}

void TablePreview::slot_drawPreview()
	{
	if(!preview_update_enabled)
		return;
	tp->tableWidget_preview->clearSpans();
	tp->tableWidget_preview->clearContents();
	int rowmax = 1;
	for(int i=0; i<NFIELDS; i++)
		{
		if(!design->fieldcheck[i]->isChecked())
			continue;
// Make sure spanned cells don't run off edge or bottom of table (=crash)
// Changing control values triggers calling this routine, so exit to avoid unwanted recursion
		if(design->colspin[i]->value() + design->widthspin[i]->value()-1 > MAXCOLS)
			{
			design->widthspin[i]->setValue(design->widthspin[i]->value()-1);
			return;
			}
		if(design->rowspin[i]->value() + design->heightspin[i]->value()-1 > MAXROWS)
			{
			design->heightspin[i]->setValue(design->heightspin[i]->value()-1);
			return;
			}
		rowmax = qMax(rowmax,
						design->rowspin[i]->value()+design->heightspin[i]->value()-1);
		}
	rowmax = qMin(rowmax, MAXROWS);

	int colmax = 0;
	for(int i=0; i<NFIELDS; i++)
		{
		if(!design->fieldcheck[i]->isChecked())
			continue;
		colmax = qMax(colmax,
						 design->colspin[i]->value()+design->widthspin[i]->value()-1);
		}
	colmax = qMin(colmax, MAXCOLS);

	tp->tableWidget_preview->setRowCount(rowmax);
	tp->tableWidget_preview->setColumnCount(colmax);

	int celluse[MAXROWS][MAXCOLS];
	for(int i=0; i<MAXROWS; i++)
		for(int j=0; j<MAXCOLS; j++)
			celluse[i][j] = 0;
	design->consistent = colmax > 0;
	for(int i=0; i<NFIELDS; i++)
		{
		if(!design->fieldcheck[i]->isChecked())
			continue;

		for(int k=design->colspin[i]->value()-1;
			k<design->colspin[i]->value()+design->widthspin[i]->value()-1; k++)
			{
			if(k>=MAXCOLS)
				continue;
			for(int j=design->rowspin[i]->value()-1;
				j<design->rowspin[i]->value()+design->heightspin[i]->value()-1; j++)
				{
				if(j>=MAXROWS)
					continue;
				celluse[j][k]++;
				if(celluse[j][k] > 1)
					design->consistent = false;
				}
			}
		}

	for(int i=0; i<NFIELDS; i++)
		{
		if(!design->fieldcheck[i]->isChecked())
			continue;
		QString tname = design->fieldname[i]->text();
		tname.replace("&", "");
		for(int j=0; j<design->widthspin[i]->value(); j++)
			for(int k=0; k<design->heightspin[i]->value(); k++)
				previewInsert(tname, j+design->colspin[i]->value()-1, k+design->rowspin[i]->value()-1,
								  mw->justEnumFromInt(design->justcombo[i]->currentIndex()));
		}
	tp->tableWidget_preview->resizeRowsToContents();
	previewBalance->doBalance();
	}

void TablePreview::previewInsert(QString tname, int col, int row, int alignment)
	{
	QTableWidgetItem *cellContents = tp->tableWidget_preview->takeItem(row, col);
	if(nullptr == cellContents)
		{
		QTableWidgetItem *qtwi = new QTableWidgetItem(tr("%1").arg(tname));
		qtwi->setTextAlignment(alignment);
		qtwi->setBackground(QColor(255, 204, 153));
		tp->tableWidget_preview->setItem(row, col, qtwi);
		return;
		}
	cellContents->setText(cellContents->text() +"\n" + tname);
	cellContents->setBackground(Qt::red);
	cellContents->setTextAlignment(Qt::AlignCenter);
	tp->tableWidget_preview->setItem(row, col, cellContents);
	}

// ------------------------------ Event handlers ------------------------------
void TablePreview::closeEvent(QCloseEvent *event)
	{
	if(ok_to_close)
		event->accept();
	else
		event->ignore();
	}

void TablePreview::keyPressEvent(QKeyEvent *event)
	{
	switch(event->key())
		{
		case Qt::Key_Home:
		case Qt::Key_End:
			if(!(Qt::ControlModifier & event->modifiers()))
				break;
			Q_FALLTHROUGH();
		case Qt::Key_PageDown:
		case Qt::Key_PageUp:
			Q_EMIT signal_keyToTable(event->key(), static_cast<int>(event->modifiers()));
			break;
		case Qt::Key_Escape:
			this->close();
			break;
		default:
			return;
		}
	event->ignore();
	return;
	}

void TablePreview::resizeEvent(QResizeEvent * /* event */)
	{
	resizeTimer->stop();
	resizeTimer->start(RESIZING_DELAY_MS);
	}

void TablePreview::slot_resizeTimeout()
	{
	resizeTimer->stop();
	previewBalance->colWidthsValid = false;
	previewBalance->doBalance();
	}
