// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

// This routine draws heavily on example code in lzma1604.7z and lzma1801.7z

#include "OtherWssi/wssindex.h"
#include "OtherWssi/globalsettings.h"
#include "OtherWssi/wssiproto.h"
#include "OtherWssi/wsxxxadd.h"
#include "Dialogs/indexwindow.h"
#include "Dialogs/mainwindow.h"

#include "3rdparty/7zip-2201/7z.h"
#include "3rdparty/7zip-2201/7zFile.h"
#include "3rdparty/7zip-2201/7zAlloc.h"
#include "3rdparty/7zip-2201/7zCrc.h"

#define kInputBufSize (static_cast<size_t>(1) << 18)

// GetAttribString was copied from 7zMain.c
static void GetAttribString(UInt32 wa, bool isDir, char *s)
{
  #ifdef USE_WINDOWS_FILE
  s[0] = static_cast<char>(((wa & FILE_ATTRIBUTE_DIRECTORY) != 0 || isDir) ? 'D' : '.');
  s[1] = static_cast<char>(((wa & FILE_ATTRIBUTE_READONLY ) != 0) ? 'R': '.');
  s[2] = static_cast<char>(((wa & FILE_ATTRIBUTE_HIDDEN   ) != 0) ? 'H': '.');
  s[3] = static_cast<char>(((wa & FILE_ATTRIBUTE_SYSTEM   ) != 0) ? 'S': '.');
  s[4] = static_cast<char>(((wa & FILE_ATTRIBUTE_ARCHIVE  ) != 0) ? 'A': '.');
  s[5] = 0;
  #else
  s[0] = (char)(((wa & (1 << 4)) != 0 || isDir) ? 'D' : '.');
  s[1] = 0;
  #endif
}

int IndexWindow::zip7add(QString path, QString filename, int /* mode */)
	{
	if(skip_extract(filename))
		return(SKIPPED);

	unsigned long basefile = Nofile - 1;
	int return_code = EXTRACTED_OK;

	QString name_with_path = path;
	name_with_path += filename;

	CFileInStream archiveStream;
	CLookToRead2 lookStream;
	CSzArEx db;
	SRes res;
	ISzAlloc allocImp;
	ISzAlloc allocTempImp;
	UInt16 *temp = nullptr;
	size_t tempSize = 0;
	allocImp.Alloc = SzAlloc;
	allocImp.Free = SzFree;

	allocTempImp.Alloc = SzAllocTemp;
	allocTempImp.Free = SzFreeTemp;

	wchar_t* ArcNameW = new wchar_t[static_cast<qulonglong>(name_with_path.length()+1)];
	int nameLength = name_with_path.toWCharArray(ArcNameW);
	ArcNameW[nameLength] = 0;
	if (InFile_OpenW(&archiveStream.file, ArcNameW))
		{
		delete [] ArcNameW;
		return(CANT_OPEN);
		}
	delete [] ArcNameW;

	struct filerec *dr_nofile;
	FileInStream_CreateVTable(&archiveStream);
	LookToRead2_CreateVTable(&lookStream, False);
	lookStream.buf = nullptr;

//	res = SZ_OK;

	{
	lookStream.buf = static_cast<Byte *>(ISzAlloc_Alloc(&allocImp, kInputBufSize));
//	if (!lookStream.buf)
//		res = SZ_ERROR_MEM;
//	else
	if (lookStream.buf)
		{
		lookStream.bufSize = kInputBufSize;
		lookStream.realStream = &archiveStream.vt;
		LookToRead2_Init(&lookStream)
		}
	}

	CrcGenerateTable();

	SzArEx_Init(&db);

	res = SzArEx_Open(&db, &lookStream.vt, &allocImp, &allocTempImp);

	if(SZ_OK != res)
		goto corrupt;

	for(unsigned int i=0; i<db.NumFiles; i++)
		{
		size_t len;
		unsigned isDir = SzArEx_IsDir(&db, i);
		if(isDir)
			continue;
		len = SzArEx_GetFileNameUtf16(&db, i, nullptr);

		if (len > tempSize)
			{
			SzFree(nullptr, temp);
			tempSize = len;
			temp = static_cast<UInt16 *>(SzAlloc(nullptr, tempSize * sizeof(temp[0])));
			if (!temp)
				{
//				res = SZ_ERROR_MEM;
				break;
				}
			}

		SzArEx_GetFileNameUtf16(&db, i, temp);

		char attr[8];
		GetAttribString(SzBitWithVals_Check(&db.Attribs, i) ? db.Attribs.Vals[i] : 0, static_cast<bool>(isDir), attr);

		UInt64 fileSize = SzArEx_GetFileSize(&db, i);

		dr_nofile = Dr + Nofile;
		switch(Case_option_indexing)
			{
			case CASE_PRESERVE:
				break;
			case CASE_FORCE_UP:
				_wcsupr(reinterpret_cast<WCHAR *>(temp));
				break;
			case CASE_FORCE_LOW:
				_wcslwr(reinterpret_cast<WCHAR *>(temp));
				break;
			}

		time1600(dr_nofile, db.MTime.Vals[i].Low, db.MTime.Vals[i].High);
		dr_nofile->Dsize = fileSize;
		dr_nofile->Volno = Diskno;
		dr_nofile->Subdirno = basefile;
		dr_nofile->Comment = QString();
		dr_nofile->Category = QString();
		dr_nofile->Bitflags = 0;
		dr_nofile->Diskbits = 0;
		setInArc(dr_nofile);
		QString fname = QString::fromWCharArray(reinterpret_cast<WCHAR *>(temp));
		if(!Store_arc_path)
			no_path(&fname);
		unbad(fname, 0);
		dr_nofile->Lfn = QDir::toNativeSeparators(fname);
		dr_nofile->ExtOffset = static_cast<unsigned short>(lastdot(dr_nofile->Lfn));
		setHasLFN(dr_nofile);

		if(mainWindowPtr->incfile_and_check_for_error(false))
			goto no_core;
		}

norm_ret:
	SzArEx_Free(&db, &allocImp);
	SzFree(nullptr, temp);
	File_Close(&archiveStream.file);
	qApp->processEvents();
	return(return_code);

no_core:
	return_code = DATABASE_FULL;
	QMessageBox::critical(this, tr("Error"), tr("Out of memory in 7zipadd"));
	Stop_indexing = true;
	goto norm_ret;

corrupt:
	return_code = CORRUPT;
	Q_EMIT signal_logMessage(tr("%1%2 is corrupted or not a 7-zip archive.")
			.arg(path, filename));
	goto norm_ret;
	}
