// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "OtherWssi/globalsettings.h"
#include "OtherWssi/wssiproto.h"
#include "OtherWssi/wsxxxadd.h"
#include "Dialogs/indexwindow.h"
#include "Dialogs/mainwindow.h"

#define NOVOSIELSKI	0
#define LUPC			1

#define UBIT8(Z) (static_cast<unsigned char>(0x ## Z))

// lbradd - read a .LBR file and add information to wssindex database.
//
//	.LBR files do not contain comments, so argument dummy is not used.
//
//		LU format may have both creation and update dates;
//		the former is used for the time tag.  Missing dates become 1/1/1980
//		or 12/31/1977 depending on format.
//
// Directory format information extracted from doc file for Vern Berg's LUU213.
// LU86 format matches LUDEF5 document.  CP/M date format is days since 12/31/77.
//
//         Directory format
//         ----------------
//  0 - File status; 0=active, 254=deleted, 255=free
//  1 - Member name; filename in FCB format, dir name is blanks
// 12 - Offset to data in sectors of 128 bytes
// 14 - Length of member in sectors.
// 16 - all zero for 'old LU' format
//
//         LUPC format extension
//         ---------------------
//  1 - master directory name is ********DIR
// 16 - Original member file creation date, MM/DD/YY
// 24 - Original member file creation time, HH:MM:SS
//
//         LU86 Directory Format
//         ---------------------
//
//  0 - File status; 0=active, 254=deleted, 255=free
//  1 - Member name; filename in FCB format, dir name is blanks
// 12 - Offset to data in sectors of 128 bytes
// 14 - Length of member in sectors.
// 16 - CRC word, or zero if Cyclic Redundancy Checking not used
// 18 - File create date, CP/M format
// 20 - File create time, DOS format
// 22 - Date file last updated (replaced), CP/M format
// 24 - Time file last updated, DOS format
// 26 - Number of bytes in last sector (??? padding bytes ???)
// 27 - reserved/unused/zero

int IndexWindow::lbradd(QString path, QString filename, int /* dummy */)
	{
	struct lu_directory
		{
		unsigned char status;	// 0=active, 0xff=unused, 0xfe or other=deleted
		char fn[8];					// file name, blank padded
		char ext[3];				// extension, blank padded
		unsigned short index;	// pointer to data
		unsigned short length;	// number of 128 byte sectors
		union {
			unsigned short crc;	// cyclic redundancy check
			char lupc_date[1];	// mm/dd/yy hh:mm:ss format date
										//				(really 16 chars)
			} lud;
		unsigned short creation_date;	// days since 12/31/77
		unsigned short change_date;	// days since 12/31/77
		unsigned short creation_time;	// standard MS-DOS packed format
		unsigned short change_time;	// standard MS-DOS packed format
		char pad_count;					// number of pad characters added, 0-0x7f
		char filler[5];					// zeroes
		} lu_buffer;

	int num_entry, type;
	FILE *fp;
	struct filerec *dr_nofile;

	if(skip_extract(filename))
		return(SKIPPED);

	unsigned long basefile = Nofile - 1;
	int return_code = EXTRACTED_OK;

	if(nullptr == (fp = arc_open(path, filename)))
		return(CANT_OPEN);

	if(1 != fread(&lu_buffer, 32, 1, fp)
		|| lu_buffer.status != 0
		|| lu_buffer.index != 0
		|| lu_buffer.length == 0)
		goto bad_lbr;

	if(strncmp(lu_buffer.fn, "           ", 11) == 0)
		type = NOVOSIELSKI;
	else if(strncmp(lu_buffer.fn, "********DIR", 11) == 0)
		type = LUPC;
	else
		goto bad_lbr;

	num_entry = lu_buffer.length << 2;
	for(int i=1; i<num_entry; i++)
		{
		if(1 != fread(&lu_buffer, 32, 1, fp))
			goto bad_lbr;

		if(UBIT8(ff) == lu_buffer.status)	// unused entry means done
			goto norm_ret;
		if(0 != lu_buffer.status)		// skip deleted entries
			continue;
		dr_nofile = Dr + Nofile;
		char fn[12];
		strncpy(fn, lu_buffer.fn, 8);
		fn[8] = EOS;
		trimblnk(fn);

		char ext[4];
		ext[0] = EOS;
		strncat(ext, lu_buffer.ext, 3);
		ext[3] = EOS;
		trimblnk(ext);
		strcat(fn, ext);

		switch(Case_option_indexing)
			{
			case CASE_PRESERVE:
				break;
			case CASE_FORCE_UP:
				_strupr(fn);
				break;
			case CASE_FORCE_LOW:
				_strlwr(fn);
				break;
			}

// date & time format depends on library type
		if(NOVOSIELSKI == type)
			{
			dr_nofile->Ddate = QDate(1977, 12, 31);
			dr_nofile->Ddate = dr_nofile->Ddate.addDays(lu_buffer.creation_date);
			dr_nofile->Dtime = timextract(lu_buffer.creation_time);
			}
		else
			{
			int year, month, day, hour, minute, second;
			sscanf(lu_buffer.lud.lupc_date, "%d/%d/%2d%d:%d:%2d", &month,
				&day, &year, &hour, &minute, &second);
			dr_nofile->Ddate = QDate(year+1900, month, day);
			dr_nofile->Dtime = QTime(hour, minute, second);
			}
		dr_nofile->Dsize = static_cast<quint64>(lu_buffer.length*128L-lu_buffer.pad_count);
		dr_nofile->Volno = static_cast<unsigned long>(Diskno);
		dr_nofile->Subdirno = static_cast<unsigned long>(basefile);
		dr_nofile->Comment = QString();
		dr_nofile->Category = QString();
		dr_nofile->Bitflags = 0;
		dr_nofile->Diskbits = 0;
		setInArc(dr_nofile);
		unbad(fn, 0);
		dr_nofile->Lfn = fn;
		dr_nofile->ExtOffset = static_cast<unsigned short>(lastdot(dr_nofile->Lfn));
		setHasLFN(dr_nofile);

		if(mainWindowPtr->incfile_and_check_for_error(false))
			goto no_core;
		}

// Fall out of above loop when all entries processed
norm_ret:
		fclose(fp);
//		update_db_totals();
		qApp->processEvents();
		return(return_code);

bad_lbr:
		return_code = CORRUPT;
		Q_EMIT signal_logMessage(tr("%1%2 is not a valid Novosielski .LBR file.")
				.arg(path, filename));
		goto norm_ret;

no_core:
		return_code = DATABASE_FULL;
		QMessageBox::critical(this, tr("Error"), tr("Out of memory in lbradd"));
		Stop_indexing = true;
		goto norm_ret;
	}
