// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "OtherWssi/globalsettings.h"
#include "OtherWssi/wssiproto.h"
#include "OtherWssi/wsxxxadd.h"
#include "Dialogs/indexwindow.h"
#include "Dialogs/mainwindow.h"

#include <fcntl.h>
#include <io.h>

#define MAXSFX	4000

// lhaadd - read a .LZH file and add information to wssindex database.
//
//	.LZH files do not contain comments so dummy is not used.

#pragma pack(1)
#define TEMPBUF_SIZE 4096

int IndexWindow::lhaadd(QString path, QString filename, int /* dummy */)
	{
	char tempbuf[TEMPBUF_SIZE];
	unsigned char header_length;
	struct lzh
		{
		char header_checksum;		// checksum from next char through crc
											//				or 2nd byte of length for type 2
		char comp_name[5];			// -lh0- or -lh1-
		long compressed_size;		// compressed size, includes path
											//			information if type 1 header
		long uncompressed_size;		// uncompressed size
// For type 2 header, time and date are combined into one field holding
//	seconds since 1-Jan-1970

		union
			{
			struct
				{
				unsigned short time_stamp;	// original file time, usual MS-DOS format
				unsigned short date_stamp;	// original file date, usual MS-DOS format
				} msdostime;
			unsigned long secs1970;
			} timeunion;
		char attribute;					// file attributes
		char header_type;					// header format, 0, 1 or 2
		unsigned char name_length;		// length of file name
		} lzh_header;

	char *full_name;	// name of file in archive, may include a path
							//				if header type 0
	unsigned int	length;		// length of file name in archive
	FILE *fp;						// file pointer for reading archive
	long				arcpos,
						lastpos,
						seek_offset;		// offset to skip to next header
	int	return_code;

	if(skip_extract(filename))
		return(SKIPPED);

	unsigned long basefile = Nofile - 1;
	return_code = EXTRACTED_OK;

	if(nullptr == (fp = arc_open(path, filename)))
		return(CANT_OPEN);

	struct filerec *dr_nofile = Dr + basefile;
	if(0 == QFA("E").compare(dr_nofile->Lfn.midRef(dr_nofile->ExtOffset,1), Qt::CaseInsensitive))
		{
		arcpos = 0;
		lastpos = _filelength(_fileno(fp)) - 5;
		if (lastpos > MAXSFX)
			lastpos = MAXSFX;
		for ( ; arcpos < lastpos; arcpos++)
			{
			fseek(fp, arcpos, SEEK_SET);
			do {
				while (arcpos < lastpos)
					{
					arcpos++;
					if('-' == fgetc(fp))
						break;
					}
				if ('l' != fgetc(fp))
					break;
				if ('h' != fgetc(fp))
					break;
				fgetc(fp);
				if ('-' != fgetc(fp))
					break;
				fseek(fp, -7, SEEK_CUR);
				goto found_header;
				} while (false);
			}
		return_code = NOT_RECOGNIZED;
		goto norm_ret;
		}

found_header:
	for(;;)
		{
		header_length = static_cast<unsigned char>(fgetc(fp));
		if(header_length == 0)
			goto norm_ret;

		if(1 != fread(&lzh_header, sizeof(struct lzh), 1, fp))
			break;
		if(2 == lzh_header.header_type)
			{
			fseek(fp, 2, SEEK_CUR);
			length = static_cast<unsigned int>(fgetc(fp)) - 3;
			fseek(fp, 2, SEEK_CUR);
			}
		else
			length = static_cast<unsigned int>(lzh_header.name_length);
		if(0 == length || length > 255)	// >255 can't really happen
			break;
		if(1 != fread(tempbuf, static_cast<size_t>(length), 1, fp))
			break;
		*(tempbuf + length) = EOS;
		dr_nofile = Dr + Nofile;

		full_name = tempbuf;
		if(!Store_arc_path)
			full_name = no_path(tempbuf, static_cast<int>(length));

		switch(Case_option_indexing)
			{
			case CASE_PRESERVE:
				break;
			case CASE_FORCE_UP:
				_strupr(full_name);
				break;
			case CASE_FORCE_LOW:
				_strlwr(full_name);
				break;
			}
		if(2 == lzh_header.header_type)
			{
#if QT_VERSION < QT_VERSION_CHECK(5, 8, 0)
			QDateTime tstamp = QDateTime(QDate(1970, 1, 1));
#else
			QDateTime tstamp = QDateTime::fromSecsSinceEpoch(0);
#endif
			tstamp = tstamp.addSecs(lzh_header.timeunion.secs1970);
			dr_nofile->Ddate = tstamp.date();
			dr_nofile->Dtime = tstamp.time();
			}
		else
			{
			dr_nofile->Ddate = datextract(lzh_header.timeunion.msdostime.date_stamp);
			dr_nofile->Dtime = timextract(lzh_header.timeunion.msdostime.time_stamp);
			}
		dr_nofile->Dsize = static_cast<quint64>(lzh_header.uncompressed_size);
		dr_nofile->Volno = static_cast<unsigned long>(Diskno);
		dr_nofile->Subdirno = static_cast<unsigned long>(basefile);
		dr_nofile->Comment = QString();
		dr_nofile->Category = QString();
		dr_nofile->Bitflags = 0;
		dr_nofile->Diskbits = 0;
		setInArc(dr_nofile);
		unbad(full_name, 0);
		dr_nofile->Lfn = full_name;
		dr_nofile->ExtOffset = static_cast<unsigned short>(lastdot(dr_nofile->Lfn));
		setHasLFN(dr_nofile);

		if(mainWindowPtr->incfile_and_check_for_error(false))
			goto no_core;

// Space forward to next header, method depends on header type
		seek_offset = lzh_header.compressed_size;
		if(seek_offset < 0)
			goto corrupt;
		switch(lzh_header.header_type)
			{
// Header 0, just skip over CRC (and compressed data)
			case 0:
				seek_offset += 2;
				break;

// Header 1, skip over CRC and 1 byte field which may be a header checksum,
//	then read length of block containing subdir name, skip over all but last
//	two bytes which contain a length field for another block to be skipped.
			case 1:
				seek_offset += 5;
				break;

// Header 2, first header byte has total header length, so just subtract off
//	the length already read
			case 2:
				seek_offset += header_length - length - 27;
				break;

			default:
				return_code = CORRUPT;
				Q_EMIT signal_logMessage(tr("%1%2 header format not recognized or not an LZH archive.")
						.arg(path, filename));
				goto norm_ret;
			}
		if(0 != fseek(fp, seek_offset, SEEK_CUR))
			break;		// this is an error, but not a severe one
		}

// Fall out of above loop only if file is corrupt.
corrupt:
		return_code = CORRUPT;
		Q_EMIT signal_logMessage(tr("%1%2 is corrupted or not a LZH archive.")
				.arg(path, filename));
// fall through to norm_ret

norm_ret:
		fclose(fp);
//		update_db_totals();
		qApp->processEvents();
		return(return_code);

no_core:
		return_code = DATABASE_FULL;
		QMessageBox::critical(this, tr("Error"), tr("Out of memory in lhaadd"));
		Stop_indexing = true;
		goto norm_ret;
	}
