// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "OtherWssi/wssiproto.h"
#include "OtherWssi/wsxxxadd.h"
#include "Dialogs/indexwindow.h"

#define	PC_OS
#include	"3rdparty/fscdefs.h"
#include	"3rdparty/sfnt.h"

// ttfadd - read a TrueType font file and add information to wssindex
//	Font name is added as extra info field.
#define TEMPBUF_SIZE 4096

int IndexWindow::ttfadd(QString path, QString filename, int /* dummy */)
	{
	char						tempbuf[TEMPBUF_SIZE];
	FILE						*fp;
	struct filerec			*dr_nofile;
	unsigned int			cTables;
	sfnt_OffsetTable		OffsetTable;
	sfnt_DirectoryEntry	Table;
	sfnt_NamingTable		NamingTable;
	unsigned short			numNames;
	sfnt_NameRecord		NameRecord;

	if(skip_extract(filename))
		return(SKIPPED);

	unsigned long basefile = Nofile -1 ;
	int return_code = EXTRACTED_OK;

	if(nullptr == (fp = arc_open(path, filename)))
		return(CANT_OPEN);

	dr_nofile = Dr + basefile;
	fread(&OffsetTable, sizeof(OffsetTable)-sizeof(sfnt_DirectoryEntry), 1, fp);
	cTables = static_cast<unsigned int>(SWAPW(OffsetTable.numOffsets));

	for(int i=0; i<static_cast<int>(cTables) && i<40; i++)
		{
		if(sizeof(Table) != fread(&Table, 1, sizeof(Table), fp))
			goto corrupt;
		if(Table.tag == tag_NamingTable)	// defined in sfnt_en.h
			{
// Now that we've found the entry for the name table, seek to that
// position in the file and read in the initial header for this
// particular table.
			unsigned long tabOff = static_cast<unsigned long>(SWAPL(Table.offset));
			if(tabOff+static_cast<unsigned long>(sizeof(NamingTable)) > dr_nofile->Dsize)
				goto corrupt;
			fseek(fp, static_cast<long>(tabOff), SEEK_SET);
			fread(&NamingTable, sizeof(NamingTable), 1, fp);
			numNames = static_cast<unsigned short>(SWAPW(NamingTable.count));
// Largest value seen in Windows 7 was 106
// Set a moderately large limit here to avoid getting stuck on a bad file
			if(numNames > 1024)
				goto corrupt;
			while(numNames--)
				{
				fread(&NameRecord, sizeof(NameRecord), 1, fp);
				unsigned long nameOffset, nameLength;
				if(4 == SWAPW(NameRecord.nameID))
					{
					switch(SWAPW(NameRecord.platformID))
						{
						case 1:	// font name is in ascii
							nameOffset = static_cast<unsigned long>(SWAPW(NameRecord.offset) + SWAPW(NamingTable.stringOffset) + SWAPL(Table.offset));
							if(nameOffset > dr_nofile->Dsize)
								goto corrupt;
							nameLength = static_cast<unsigned long>(SWAPW(NameRecord.length));
// Longest seen in Windows 7 was 37
							if(nameLength > 256)
								goto corrupt;
							fseek(fp, static_cast<long>(nameOffset), SEEK_SET);
							fread(tempbuf, nameLength, 1, fp);
							tempbuf[SWAPW(NameRecord.length)] = '\0';
							trimblnk(tempbuf);
							strcat(tempbuf, " TrueType font");
							dr_nofile->ExtraInfo = tempbuf;
							setHasExtra(dr_nofile);
							goto norm_ret;

						case 3:	// font name is in unicode
							nameOffset = static_cast<unsigned long>(SWAPW(NameRecord.offset) + SWAPW(NamingTable.stringOffset) + SWAPL(Table.offset));
							if(nameOffset > dr_nofile->Dsize)
								goto corrupt;
							fseek(fp, static_cast<long>(nameOffset), SEEK_SET);
							nameLength = static_cast<unsigned long>(SWAPW(NameRecord.length));
// Longest seen in Windows 7 was 48
							if(nameLength > 256)
								goto corrupt;
							fread(tempbuf+2, nameLength, 1, fp);
// Insert a big-endian byte order marker, otherwise Qt (under Windows)
// incorrectly assumes little-endian
							tempbuf[0] = '\xfe';
							tempbuf[1] = '\xff';
							tempbuf[nameLength+2] = '\0';
							tempbuf[nameLength+3] = '\0';
							dr_nofile->ExtraInfo = QString::fromUtf16(reinterpret_cast<unsigned short *>(tempbuf)) + tr(" TrueType font");
							setHasExtra(dr_nofile);
							goto norm_ret;

						default:
							continue;
						}
					}
				}
			}
		}

	corrupt:
		return_code = CORRUPT;
		Q_EMIT signal_logMessage(tr("%1%2 is corrupted or not a TrueType font.")
				.arg(path, filename));

	norm_ret:
		fclose(fp);
		return(return_code);
	}
