// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "OtherWssi/globalsettings.h"
#include "OtherWssi/wssiproto.h"
#include "OtherWssi/wsxxxadd.h"
#include "Dialogs/indexwindow.h"
#include "Dialogs/mainwindow.h"

// zooadd - read a .ZOO file and add information to wssindex database.

#define ZOO_TAG 0xfdc4a7dcUL

#pragma pack(1)
#define TEMPBUF_SIZE 4096

struct zh
	{
	char text[20];					// typeable identifier
	unsigned long tag;			// 0xfdc4a7dc
	long start;				  		// offset of first entry
	long minus;						// -start for checking/recovery
	char major;						// major zoo version
	char minor;						// minor zoo version
// following fields only present if offset to first entry is not 34
	char htype;						// header type
	long zip_comment_pos;		// offset to archive comment
	unsigned short comment_size;	// length of comment
// ZOO source says version_data is an int (16-bit), but the comments say
// byte.  It really is just one byte.  Otherwise, seek to first file
// record goes backwards which should be forbidden to prevent infinite
// looping in broken zoo files.
	unsigned char version_data;	// version info
	};

struct ze
	{
	unsigned long tag;		// 0xfdc4a7dc
	char type;					// directory type
	char pack_method;			// packing method
	long next_pos;				// position of next entry
	long this_pos;				// position of data portion of this entry
									//				or zero if end of zoo file
	unsigned short date;		// date stamp, DOS format
	unsigned short time;		// time stamp, DOS format
	short crc;					// cyclic redundancy check, not used here
	long uncompressed_size;	// uncompressed size
	long compressed_size;	// compressed file size (bytes)
	char major;					// major zoo version
	char minor;					// minor zoo version
	char delete_flag;			// 0 if good, 1 if deleted
	char file_structure;
	long comment_pos;			// position of comment for file
	short comment_size;		// length of comment
	char ZipName[13];		// filename.ext
// plus variable length stuff not of interest here
	};

int IndexWindow::zooadd(QString path, QString filename, int mode)
	{
	char tempbuf[TEMPBUF_SIZE];
	struct zh zoo_header;
	struct ze zoo_entry;
	int temp;
	FILE *fp;
	struct filerec *dr_nofile;

	if(skip_extract(filename))
		return(SKIPPED);

	unsigned long basefile = Nofile - 1;
	int return_code = EXTRACTED_OK;

	if(nullptr == (fp = arc_open(path, filename)))
		return(CANT_OPEN);

	if(1 != fread(&zoo_header, sizeof(struct zh), 1, fp)
		|| zoo_header.tag != ZOO_TAG)
		{
		goto corrupt;			// this isn't an error, just a warning
		}

	if((extractArcComment & mode) && (temp = zoo_header.comment_size) > 0)
		{
		long save_position = ftell(fp);
		if(temp > TEMPBUF_SIZE-1)
			temp = TEMPBUF_SIZE-1;
		if(zoo_header.zip_comment_pos < save_position)
			goto corrupt;
		if(0 != fseek(fp, zoo_header.zip_comment_pos, SEEK_SET)
			|| 1 != fread(tempbuf, static_cast<size_t>(temp), 1, fp))
			{
			goto corrupt;
			}
		dr_nofile = Dr + basefile;
		if(com_arch(tempbuf, dr_nofile, temp))
			goto no_core;
// Comment could be anywhere (probably at end of file).
// Go back to where we were so we won't have to seek backwards to get
// to the first file entry.
		fseek(fp, save_position, SEEK_SET);
		}

	zoo_entry.next_pos = zoo_header.start;
	for(;;)
		{
		if(zoo_entry.next_pos < ftell(fp))
			goto corrupt;
		if(0 != fseek(fp, zoo_entry.next_pos, SEEK_SET)
			|| 1 != fread(&zoo_entry, sizeof(struct ze), 1, fp)
			|| ZOO_TAG != zoo_entry.tag)
			{
			goto corrupt;		// this is an error, but not a severe one
			}
		if(0L == zoo_entry.this_pos)
			goto norm_ret;

		if(0 != zoo_entry.delete_flag)
			continue;

		dr_nofile = Dr + Nofile;
		switch(Case_option_indexing)
			{
			case CASE_PRESERVE:
				break;
			case CASE_FORCE_UP:
				_strupr(zoo_entry.ZipName);
				break;
			case CASE_FORCE_LOW:
				_strlwr(zoo_entry.ZipName);
				break;
			}

		dr_nofile->Ddate = datextract(zoo_entry.date);
		dr_nofile->Dtime = timextract(zoo_entry.time);
		dr_nofile->Dsize = static_cast<quint64>(zoo_entry.uncompressed_size);
		dr_nofile->Volno = Diskno;
		dr_nofile->Subdirno = static_cast<unsigned long>(basefile);
		dr_nofile->Category = QString();
		dr_nofile->Comment = QString();
		dr_nofile->Bitflags = 0;
		dr_nofile->Diskbits = 0;
		setInArc(dr_nofile);
		unbad(zoo_entry.ZipName, 12);
		dr_nofile->Lfn = zoo_entry.ZipName;
		dr_nofile->ExtOffset = static_cast<unsigned short>(lastdot(dr_nofile->Lfn));
		setHasLFN(dr_nofile);

		if((extractMemberComments & mode) && (temp = zoo_entry.comment_size) > 0)
			{
			if(temp > TEMPBUF_SIZE-1)
				temp = TEMPBUF_SIZE-1;
			if(zoo_entry.comment_pos < ftell(fp))
				goto corrupt;
			if(0 != fseek(fp, zoo_entry.comment_pos, SEEK_SET)
				|| 1 != fread(tempbuf, static_cast<size_t>(temp), 1, fp))
				{
				goto corrupt;			// this is a severe error
				}
			if(com_arch(tempbuf, dr_nofile, temp))
				goto no_core;
			}

		if(mainWindowPtr->incfile_and_check_for_error(false))
			goto no_core;
		}

// never fall out of above loop
corrupt:
		return_code = CORRUPT;
		Q_EMIT signal_logMessage(tr("%1%2 is corrupted or not a ZOO archive.")
				.arg(path, filename));
// fall through to norm_ret

norm_ret:
		fclose(fp);
//		update_db_totals();
		qApp->processEvents();
		return(return_code);

no_core:
		return_code = DATABASE_FULL;
		QMessageBox::critical(this, tr("Error"), tr("Out of memory in zooadd"));
		Stop_indexing = true;
		goto norm_ret;
	}
