// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "ModelView/voltablemodel.h"
#include "OtherWssi/tablelayout.h"
#include "OtherWssi/wssindex.h"
#include "Dialogs/mainwindow.h"
#include "ui_mainwindow.h"
#include "OtherWssi/globalsettings.h"
#include "OtherWssi/wssiproto.h"
#include <QAbstractItemModel>
#include <QtDebug>

volTableModel::volTableModel(QObject *parent) : QAbstractTableModel(parent)
	{
// This gets executed before there are any database entries, so pick a
// number of disks that may not require later reallocation.
	volmax = 1000;
	vptr = new unsigned long[volmax];
	volidnum = new unsigned long[volmax];
	redVol = 0;
	anyRed = false;
	volModelValid = false;
	volWindowPtr = nullptr;
	}

void volTableModel::setTablePtr(VollistWindow *vp)
	{
	volWindowPtr = vp;
	}

void volTableModel::signalLayoutChanged()
	{
	Q_EMIT QAbstractItemModel::layoutChanged();
	}

int volTableModel::rowCount(const QModelIndex & /*parent*/) const
	{
	return volModelValid ? modelRows : 0;
	}

void volTableModel::setVolModelValidity(bool isvalid)
	{
	volModelValid = isvalid;
	}

int volTableModel::columnCount(const QModelIndex & /*parent*/) const
	{
	return volModelValid ? modelCols : 0;
	}

void volTableModel::setSizeAndValidity(int rows, int cols, unsigned long volStart)
	{
	beginResetModel();
	volModelValid = false;
	modelRows = rows;
	modelCols = cols;
	firstVol = volStart;
	sortDisksIfNeeded();
	endResetModel();
	}

QVariant volTableModel::data(const QModelIndex &index, int role) const
	{
// index.row() < 0 happens after clicking header to sort and then typing ^C.
// No other keystroke seems to trigger this.
	if(!volModelValid || index.row() < 0)
		return QVariant();

	int col = index.column();

	unsigned long volno = firstVol + static_cast<unsigned long>(index.row());
	unsigned long k = vptr[volno];
	unsigned long first_record_on_volume = Vrec[k].firstOnDisk;
	struct filerec *dr_k = Dr + first_record_on_volume;
	struct diskrec *vr_k = Vrec + k;

	switch(role)
		{
		case Qt::DisplayRole:
			Q_FALLTHROUGH();
		case Qt::EditRole:
			switch(col)
				{
				case VOL_DISP_VOLUME:
					return vr_k->VolName;

				case VOL_DISP_SIZE:
					return formattedsize(vr_k->Vbytes, disksize_unit);

				case VOL_DISP_FREE:
					return formattedsize(vr_k->Vfree, disksize_unit);

				case VOL_DISP_FILES:
					return tr("%L1").arg(vr_k->Vfiles);

				case VOL_DISP_SUBS:
					return tr("%L1").arg(vr_k->Vsubdir);

				case VOL_DISP_INDEXDATE:
					return vr_k->Vdate.toString(ShortDateFormat);

				case VOL_DISP_CATEGORY:
					if(!isDotID(dr_k))
						break;
					return dr_k->Category;

				case VOL_DISP_COMMENT:
					if(!isDotID(dr_k))
						break;
					return dr_k->Comment;

				case VOL_DISP_EXTRA_INFO:
					if(!isDotID(dr_k))
						break;
					return dr_k->ExtraInfo;

				default:
				case -1:
					break;
				}
			break;

		case Qt::BackgroundRole:
			{
			if(anyRed && volno == redVol)
				return QColor(Qt::red);
			QColor rowcolor;
			if(VolUseAltColors)
				rowcolor = VolRowColor[volno&1];
			else
				rowcolor = VolRowColor[0];
			return rowcolor;
			}

		case Qt::ForegroundRole:
			return QColor(Qt::black);

		case Qt::TextAlignmentRole:
			switch(col)
				{
				case 0:	// Volume
				case 6:	// Category
				case 7:	// Comment
				case 8:	// Extra info
					return QVariant(Qt::AlignLeft | Qt::AlignVCenter);

				case 1:	// Disk size
				case 2:	// Bytes free
				case 3:	// Files
				case 4:	// Subdirectories
				case 5:	// Date indexed
					return QVariant(Qt::AlignRight | Qt::AlignVCenter);

				default:
					break;
				}
			break;

		case Qt::FontRole:
			return *Font_vollist;

		case Qt::ToolTipRole:
			if(index.row() != volWindowPtr->volSelection->currentIndex().row()
					|| index.column() != volWindowPtr->volSelection->currentIndex().column())
				break;
			switch(col)
				{
				case 0:	// Volume
				case 6:	// Category
				case 7:	// Comment
				case 8:	// Extra info
					return(tr("F2 or double-click to edit"));

				default:
					break;
				}
			break;

		default:
			break;
		}
	return QVariant();
	}

QVariant volTableModel::headerData(int section, Qt::Orientation orientation, int role) const
	{
	if (Qt::DisplayRole != role || Qt::Horizontal != orientation)
		return QVariant();
	return theader[section];
	}

bool volTableModel::setHeaderData(int section, Qt::Orientation orientation,
											  const QVariant &value, int role)
	{
	if (Qt::DisplayRole != role || Qt::Horizontal != orientation)
		return false;
	theader[section] = value.toString();
	Q_EMIT headerDataChanged(orientation, section, section);
	return true;
	}

void volTableModel::setRed(unsigned long reddisk)
	{
	anyRed = true;
	redVol = reddisk;
	}

void volTableModel::noRed()
	{
	anyRed = false;
	redVol = 0;
	}

void volTableModel::refreshData()
	{
	Q_EMIT dataChanged(QModelIndex(), QModelIndex());
	}

// Category, comment, extra info and volume cells are editable, others are not
Qt::ItemFlags volTableModel::flags(const QModelIndex &index) const
	{
	int col = index.column();
	switch(col)
		{
		case 0:	// Volume
		case 6:	// Category
		case 7:	// Comment
		case 8:	// Extra info
			return(Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsEditable);

		default:
			break;
		}

	return(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
	}

bool volTableModel::setData(const QModelIndex & index, const QVariant & value, int role)
	{
	if (role == Qt::EditRole)
		{
	  Q_EMIT signal_volEditCompleted(index, value.toString());
		}
	return true;
	}

void volTableModel::sortDisksIfNeeded()
	{
	beginResetModel();
	if(!volModelValid)
		{
		if(Diskno > volmax)
			{
			delete [] vptr;
			delete [] volidnum;
			volmax = Diskno + 100;
			vptr = new unsigned long[volmax];
			volidnum = new unsigned long[volmax];
			}

		unsigned long *viptr = volidnum;
		for(unsigned long i=0; i<Diskno; i++)
			*(viptr++) = Diskno;

		for(unsigned long i=0; i<Nofile; i++)
			{
			struct filerec *dr_i = Dr + i;
			if(!isDotID(dr_i))
				continue;
			if(dr_i->Subdirno > 0)
				continue;
			volidnum[dr_i->Volno] = i;
			}

		unsigned long *tptr = vptr;
		for(unsigned long i=0; i<Diskno; i++)
			*(tptr++) = i;
		if(0 != Vsrtdir)
			mainWindowPtr->doSort(vptr, Diskno, wssvcomp, 0);
		volModelValid = true;
		}
	endResetModel();
	}
