// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "Non_member_functions/customio.h"
#include <stdio.h>

// Read and return 16-bit int
unsigned short getshort(FILE *fp)
	{
	char *p;
	short w;

	p = reinterpret_cast<char *>(&w);
	for (int i=sizeof(short); --i>=0; )
		*p++ = static_cast<char>(getc(fp));
	if (feof(fp))
		return(static_cast<unsigned short>(EOF));
	return(static_cast<unsigned short>(w));
	}

// Read 16-bit int, return 32-bits
unsigned int fgetWord(FILE *fp)
	{
	unsigned int b0, b1;

	b0 = static_cast<unsigned int>(fgetc(fp));
	b1 = static_cast<unsigned int>(fgetc(fp));
	return (b1 << 8) + b0;
	}

// getlong - if read_int is true, read a 16-bit int from fp, otherwise read
//	32 bits.  If reading an int, and the high bit is found to be set, set the
//	high bit in the returned long, but do not sign extend.  On error or EOF,
//	return EOF;
unsigned long getlong(FILE *fp, bool read_int)
	{
	unsigned short lsw;
	unsigned long msw = 0;

	lsw = getshort(fp);
	if(read_int)
		{
		msw = lsw & 0x8000;
		lsw &= 0x7fff;
		}
	else
		msw = getshort(fp);
	if(WSSERROR_OR_EOF(fp))
		return(static_cast<unsigned long>(EOF));
	else
		return(lsw+(msw<<16));
	}

// Read 32-bit int
unsigned long fgetLongword(FILE *fp)
	{
	unsigned long b0, b1, b2, b3;

	b0 = static_cast<unsigned long>(fgetc(fp));
	b1 = static_cast<unsigned long>(fgetc(fp));
	b2 = static_cast<unsigned long>(fgetc(fp));
	b3 = static_cast<unsigned long>(fgetc(fp));
	return (b3 << 24) + (b2 << 16) + (b1 << 8) + b0;
	}

// Read a 64-bit integer from fp
quint64 get64(FILE *fp)
	{
	quint64 lsw = 0;
	quint64 msw = 0;

	lsw = getlong(fp, false);
	msw = getlong(fp, false);
	if(WSSERROR_OR_EOF(fp))
		return(static_cast<quint64>(EOF));
	else
		return(lsw+(msw<<32));
	}

// Read 16-bit int with swapped byte order
unsigned int fgetSwappedWord(FILE *fp)
	{
	unsigned int b0, b1;

	b1 = static_cast<unsigned int>(fgetc(fp));
	b0 = static_cast<unsigned int>(fgetc(fp));
	return (b1 << 8) + b0;
	}

// Read 32-bit int with swapped byte order
unsigned long fgetSwappedLong(FILE *fp)
	{
	unsigned long b0, b1, b2, b3;

	b3 = static_cast<unsigned long>(fgetc(fp));
	b2 = static_cast<unsigned long>(fgetc(fp));
	b1 = static_cast<unsigned long>(fgetc(fp));
	b0 = static_cast<unsigned long>(fgetc(fp));
	return (b3 << 24) + (b2 << 16) + (b1 << 8) + b0;
	}

// getNullTerminatedString - read a null terminated string from a stream.
//	If buffer is too small, stop filling but continue reading until the null
//	is found (or eof or error).  On success, return pointer to string; on
//	failure, return NULL

char *getNullTerminatedString(char *string, int n, FILE *stream)
	{
	int c = 1;

	char *P = string;

	while(EOS != c && --n > 0  &&  (c=getc(stream)) != EOF)
		*(P++) = static_cast<char>(c);

	if(EOF == c)
		return(nullptr);
	*P = 0;
	return (ferror(stream)) ? nullptr : string;
	}

// crlfFgets - like standard fgets, but recognizes either CR, LF or both
//	as a line terminator.
char *crlfFgets (char *s, int n, FILE *fp)
	{
	int c = 0;
	char *rc;

	char *p = s;

	while ('\n' != c && '\r' != c && --n > 0  &&  (c = getc(fp)) != EOF)
		*p++ = static_cast<char>(c);

	if(EOF == c && p == s)
		return( nullptr );
// Check the return code before peeking ahead one character because doing
//	that might trigger error or eof.
	rc = (ferror (fp)) ? nullptr : s;
	if('\n' != (c = getc(fp)) && '\r' != c)
		ungetc(c, fp);
	*p = 0;
	return rc;
	}
