// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include <string.h>
#include "OtherWssi/wssindex.h"
#include "OtherWssi/wssiproto.h"
#include "Threads/threadsafe.h"
#include "OtherWssi/globalsettings.h"
#include "ExtractSubs/extractinfo.h"

// Prototypes for local routines
static bool member_or_container(struct filerec *dr, int type, int ithread);

// wsreject - flag unselected entries in wssindex database
//	Return REJECT if rejected, else ACCEPT.

bool wsreject(unsigned long k, int ithread)
	{
	struct filerec *dr_k, *dr_indirect, *dr_next;
	struct diskrec *vrec_volno;
	unsigned long disk_number, k_indirect;
	unsigned short j;
	bool not_match_yet;

	dr_k = Dr + k;
	if(NoId && isDotID(dr_k))
		return(REJECT);

	if(Usesel[SEL_FROM_PREVIOUS] &&
		!(dr_k->Bitflags & PREVIOUSLY_SELECTED))
		return(REJECT);

	if(Usesel[SEL_REM_FROM_PREVIOUS] &&
		!(dr_k->Bitflags & PREVIOUSLY_SELECTED))
		return(REJECT);

	if(Usesel[SEL_ADD_TO_PREVIOUS] &&
		(dr_k->Bitflags & PREVIOUSLY_SELECTED))
		return(ACCEPT);

	int match_action = ACCEPT;
	int not_match_action = REJECT;
	if(Usesel[SEL_REM_FROM_PREVIOUS])
		{
		match_action = REJECT;
		not_match_action = ACCEPT;
		}

	disk_number = dr_k->Volno;
	vrec_volno = Vrec + disk_number;
	if(inArc(dr_k))
		{
		k_indirect = dr_k->Subdirno;
		dr_indirect = Dr + k_indirect;
		}
	else
		{
//		k_indirect = k;
		dr_indirect = dr_k;
		}

// Each case in this loop may either
//	1. return(REJECT) to unconditionally exclude the file.
//	2. return(ACCEPT) to unconditionally select the file.
//	3. Set not match yet flag and break if file fails this test but other
//		selection criteria need to be considered.  This is done if there is
//    the possibility of serveral allowed conditions (font or graphics or
//    movie...)
//	4. Do nothing and break to allow other criteria to be applied.  This is
//    done where a range might be selected; e.g., we're not too high but
//    haven't yet tested whether we're too low.
	not_match_yet = false;
	for(j=0; j<Nselect; j++)
		{
		switch(Sel[j])
			{
// File names and extensions
			case SEL_FILENAME:
				if(matchLfnRegexp(MatchNameCopy[ithread], dr_k))
					return(match_action);
				return(not_match_action);

// File names and extensions, accents ignored
			case SEL_FILENAME_NODIACRIT:
				if(matchLfnNodiacritRegexp(MatchNameCopy[ithread], dr_k, ithread))
					return(match_action);
				return(not_match_action);

// Subdirectory
			case SEL_SUBDIR:
				if(CompsubCopy[ithread].exactMatch(SubDir[dr_indirect->Subdirno]))
					return(match_action);
				return(not_match_action);

// Subdirectory, accents ignored
			case SEL_SUBDIR_NODIACRIT:
				if(matchNodiacritRegexp(CompsubCopy[ithread], SubDir[dr_indirect->Subdirno], ithread))
					return(match_action);
				return(not_match_action);

// disk volume name
			case SEL_VOLUME:
				if(CompdskCopy[ithread].exactMatch(vrec_volno->VolName))
					return(match_action);
				return(not_match_action);

// disk volume name, accents ignored
			case SEL_VOLUME_NODIACRIT:
				if(matchNodiacritRegexp(CompdskCopy[ithread], vrec_volno->VolName, ithread))
					return(match_action);
				return(not_match_action);

// File names starting from - not implemented
//			case SEL_FROM_FILENAME:
//				if(fixed_namecomp(StartFile, "", dr_k, COMPARE_FILENAME,
//										COMPARE_NOMORE, !Ignore_case_sort) > 0)
//					return(not_match_action);
//				break;

// File extensions starting from - not implemented
//			case SEL_FROM_EXTENSION:
//				if(fixed_namecomp("", StartExt, dr_k, COMPARE_EXTENSION,
//										COMPARE_NOMORE, !Ignore_case_sort) > 0)
//					return(not_match_action);
//				break;

// Subdirs starting from - not implemented
//			case SEL_FROM_SUBDIR:
//				if(qstringCompare(StartSubdir, SubDir[dr_indirect->Subdirno]) > 0)
//					return(not_match_action);
//				break;

// Volumes starting from - not implemented
//			case SEL_FROM_VOLUME:
//				if(qstringCompare(StartDisk, vrec_volno->VolName) > 0)
//					return(not_match_action);
//				break;

// Volume numbers, lower limit
			case SEL_VOL_NUM_LOW_LIMIT:
				if(StartDiskNum >
					static_cast<unsigned long>(numberKey(vrec_volno->VolName)))
					return(not_match_action);
				break;

// Volume numbers, upper limit
			case SEL_VOL_NUM_HI_LIMIT:
				if(StopDiskNum <
					static_cast<unsigned long>(numberKey(vrec_volno->VolName)))
					return(not_match_action);
				break;

// Comment
			case SEL_COMMENT:
				if(!Compcom.isEmpty() && dr_k->Comment.isEmpty())
					return(not_match_action);
				if(Compcom.isEmpty() && !dr_k->Comment.isEmpty())
					return(not_match_action);
				if(CompcomCopy[ithread].exactMatch(dr_k->Comment))
					return(match_action);
				return(not_match_action);

// Comment, accents ignored
			case SEL_COMMENT_NODIACRIT:
				if(!Compcom.isEmpty() && dr_k->Comment.isEmpty())
					return(not_match_action);
				if(Compcom.isEmpty() && !dr_k->Comment.isEmpty())
					return(not_match_action);
				if(matchNodiacritRegexp(CompcomCopy[ithread], dr_k->Comment, ithread))
					return(match_action);
				return(not_match_action);

// Category
			case SEL_CATEGORY:
				if(!Compcat.isEmpty() && dr_k->Category.isEmpty())
					return(not_match_action);
				if(Compcat.isEmpty() && !dr_k->Category.isEmpty())
					return(not_match_action);
				if(CompcatCopy[ithread].exactMatch(dr_k->Category))
					return(match_action);
				return(not_match_action);

// Category, accents ignored
			case SEL_CATEGORY_NODIACRIT:
				if(!Compcat.isEmpty() && dr_k->Category.isEmpty())
					return(not_match_action);
				if(Compcat.isEmpty() && !dr_k->Category.isEmpty())
					return(not_match_action);
				if(matchNodiacritRegexp(CompcatCopy[ithread], dr_k->Category, ithread))
					return(match_action);
				return(not_match_action);

// Extra info
			case SEL_EXTRA_INFO:
				if(!Compextra.isEmpty() && dr_k->ExtraInfo.isEmpty())
					return(not_match_action);
				if(Compextra.isEmpty() && !dr_k->ExtraInfo.isEmpty())
					return(not_match_action);
				if(CompextraCopy[ithread].exactMatch(dr_k->ExtraInfo))
					return(match_action);
				return(not_match_action);

// Extra info, accents ignored
			case SEL_EXTRA_INFO_NODIACRIT:
				if(!Compextra.isEmpty() && dr_k->ExtraInfo.isEmpty())
					return(not_match_action);
				if(Compextra.isEmpty() && !dr_k->ExtraInfo.isEmpty())
					return(not_match_action);
				if(matchNodiacritRegexp(CompextraCopy[ithread], dr_k->ExtraInfo, ithread))
					return(match_action);
				return(not_match_action);

// Date, lower limit
			case SEL_DATE_LOW_LIMIT:
				if(dr_k->Ddate < Compd1.date())
					return(not_match_action);
				break;

// Date, upper limit
			case SEL_DATE_HI_LIMIT:
				if(dr_k->Ddate > Compd2.date())
					return(not_match_action);
				break;

// Size, lower limit
			case SEL_SIZE_LOW_LIMIT:
				if(dr_k->Dsize < Compsiz1)
					return(not_match_action);
				break;

// Size, upper limit
			case SEL_SIZE_HI_LIMIT:
				if(dr_k->Dsize > Compsiz2)
					return(not_match_action);
				break;

// "PCBoard Zippy", file name, extension, comment or category
			case SEL_ZIPPY:
				if(ZipNameCopy[ithread].isEmpty())
					return(not_match_action);
				if(matchLfnRegexp(ZipNameCopy[ithread], dr_k))
					return(match_action);
				if(ZipNameCopy[ithread].exactMatch(dr_k->Comment))
					return(match_action);
				if(ZipNameCopy[ithread].exactMatch(dr_k->Category))
					return(match_action);
				if(ZipNameCopy[ithread].exactMatch(dr_k->ExtraInfo))
					return(match_action);
				return(not_match_action);

// Like zippy but ignore diacritical marks and ligatures
			case SEL_ZIPPY_NODIACRIT:
				if(ZipNameCopy[ithread].isEmpty())
					return(not_match_action);
				if(matchLfnNodiacritRegexp(ZipNameCopy[ithread], dr_k, ithread))
					return(match_action);
				if(matchNodiacritRegexp(ZipNameCopy[ithread], dr_k->Comment, ithread))
					return(match_action);
				if(matchNodiacritRegexp(ZipNameCopy[ithread], dr_k->Category, ithread))
					return(match_action);
				if(matchNodiacritRegexp(ZipNameCopy[ithread], dr_k->ExtraInfo, ithread))
					return(match_action);
				return(not_match_action);

// Include archives (and self-extracting files) but not members
			case SEL_ARC_FILES:
				if(inArc(dr_k))
					return(not_match_action);
				Q_FALLTHROUGH();

// Include archive files and members
			case SEL_ARC_AND_MEMB:
				if(member_or_container(dr_indirect, TYPE_ARCHIVE, ithread))
					return(match_action);
				return(not_match_action);

// Archives and members by name of archive: must match specified archive
//	wildcard, and extension must match one of the archive file types.
			case SEL_ARCMEM_BY_ARCNAME:
				if(!matchLfnRegexp(ComparcCopy[ithread], dr_indirect))
					return(not_match_action);
				if(member_or_container(dr_indirect, TYPE_ARCHIVE, ithread))
					return(match_action);
				return(not_match_action);

// Self-extracting files and contents
			case SEL_SELF_EXTRACTING:
				if(dr_indirect->Lfn.endsWith(".exe",Qt::CaseInsensitive))
					{
					if(inArc(dr_k))
						return(match_action);
					if(Nofile-1 == k)
						{
						not_match_yet = true;
						break;
						}
					dr_next = Dr + k + 1;
					if(inArc(dr_next))
						return(match_action);
					}
				not_match_yet = true;
				break;

// Not implemented
//			case -SEL_SELF_EXTRACTING:
//				if(dr_indirect->Lfn.endsWith(".exe",Qt::CaseInsensitive))
//					{
//					if(inArc(dr_k))
//						return(not_match_action);
//					if(Nofile-1 == k)
//						break;
//					dr_next = Dr + k + 1;
//					if(inArc(dr_next))
//						return(not_match_action);
//					}
//				break;

// Font files
			case SEL_FONT:
				if(member_or_container(dr_k, TYPE_FONT, ithread))
					return(match_action);
				not_match_yet = true;
				break;

// Graphics files
			case SEL_GRAPHICS:
				if(member_or_container(dr_k, TYPE_GRAPHICS, ithread))
					return(match_action);
				not_match_yet = true;
				break;

// Movie files
			case SEL_MOVIES:
				if(member_or_container(dr_k, TYPE_MOVIE, ithread))
					return(match_action);
				not_match_yet = true;
				break;

// Sound files
			case SEL_SOUND:
				if(member_or_container(dr_k, TYPE_SOUND, ithread))
					return(match_action);
				not_match_yet = true;
				break;

// Files marked for deletion
// (Not currently implemented)
//			case SEL_TAGGED:
//				if(dr_k->Bitflags & TAGGED_ANY_REASON)
//					return(match_action);
//				return(not_match_action);

// Files not marked for deletion
// (Not currently implemented)
//			case SEL_NOT_TAGGED:
//				if(dr_k->Bitflags & TAGGED_ANY_REASON)
//					return(not_match_action);
//				return(match_action);

			default:
				break;
			}
		}

	if(not_match_yet)
		return(not_match_action);

	return(match_action);
	}

// member_or_container - check whether extension matches a special file type
//	(ignore case).  For archive members, when being in an archive is a selection
// condition, this is called with the archive as an argument, so don't need to
// check whether an archive member.

static bool member_or_container(struct filerec *dr, int type, int ithread)
	{
	QString extension = lfnpiece(dr, COMPARE_EXTENSION);
	for(int i=0; i<NLIBTYPE; i++)
		{
		if(type != Extract[i].type)
			continue;
		if(nameRegexpCopy[ithread][i].indexIn(extension) >= 0)
			return(true);
		}
	return(false);
	}
