// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include <stdio.h>
#include <string.h>
#ifndef EOS
	#define EOS '\0'
#endif

// unbad(string) - truncate a string at a newline or carriage return
//	(translate first of either to a null).  Also translate hex ff to blank.
// Force length to be at most maxLength by inserting a null (so the
// allocated length better be one more than this to allow for the null).
// maxLength = 0 means use existing null terminator.
// This is to be used on ascii strings, not unicode.

#include "OtherWssi/wssindex.h"
#include <QString>

void unbad(char *string, int maxLength)
	{
	char *p;

	if(maxLength > 0)
		string[maxLength] = EOS;
	if((p = strchr(string, '\n')) != nullptr)
		*p = EOS;
	if((p = strchr(string, '\r')) != nullptr)
		*p = EOS;
	while((p = strchr(string, 0xff)) != nullptr)
		*p = ' ';

	return;
	}

void unbad(QString qstring, int maxLength)
	{
	if(maxLength > 0)
		qstring.truncate(maxLength);
	int cloc = qstring.indexOf("\n");
	if(cloc > 0)
		qstring.truncate(cloc);
	cloc = qstring.indexOf("\r");
	if(cloc > 0)
		qstring.truncate(cloc);
	qstring.replace(QChar(255)," ");

	return;
	}

// makePrintable(string) - remove any unprintable characters (control
//	characters 0-1fh, 7fh and ffh) from a string by changing them to blanks.
//	However, don't insert more than one blank for multiple unprintable
//	characters, and remove strings of blanks longer than 2.

// states
#define CHAR_LAST		0
#define BLANK_LAST	1
#define CTL_LAST		2
#define TWO_BLANKS	3

// actions
#define COPY			0
#define ADD_BLANK		1
#define SKIP			2

// input possibilities
#define UNPRINTABLE	0
#define PRINTABLE		1
#define BLANK_CHAR	2

void makePrintable(char *string)
	{
	char *outstring;
	int state, intype;
//										UNPRINTABLE PRINTABLE BLANK
	static int action[4][3] = { {ADD_BLANK, COPY, ADD_BLANK},	// CHAR_LAST
										{SKIP, COPY, ADD_BLANK},			// BLANK_LAST
										{SKIP, COPY, SKIP},			      // CTL_LAST
										{SKIP, COPY, SKIP} };		      // TWO_BLANKS

	static int newstate[4][3] = { {BLANK_LAST, CHAR_LAST, BLANK_LAST},
										{BLANK_LAST, CHAR_LAST, TWO_BLANKS},
										{CTL_LAST, CHAR_LAST, CTL_LAST},
										{TWO_BLANKS, CHAR_LAST, TWO_BLANKS} };

	state = TWO_BLANKS;		// starting in this state eliminates leading blanks

	for(outstring=string; ; string++)
		{
		unsigned char inchar = static_cast<unsigned char>(*string);
		if(EOS == inchar)
			{
			*outstring = EOS;
			return;
			}
		if(inchar < ' '									// control character
			|| inchar == 0x7f								// delete
			|| (inchar >= 0xb0 && inchar <= 0xdf)	// box forming character
			|| inchar == 0xff)							// pseudo-blank
			intype=UNPRINTABLE;
		else if(inchar == ' ')
			intype = BLANK_CHAR;
		else
			intype = PRINTABLE;

		switch(action[state][intype])
			{
			case COPY:
				*(outstring++) = *string;
				break;

			case ADD_BLANK:
				*(outstring++) = ' ';
				break;

			case SKIP:
				break;
			}

		state = newstate[state][intype];
		}
	}

// This only replaces control characters with blanks
void makePrintable(QString string)
	{
	for(int i=0; i<string.size(); i++)
		{
		if(string[i] < 32)
			string.replace(i, 1, " ");
		}
	}
