// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include <string.h>
#include "OtherWssi/wssindex.h"
#include "OtherWssi/wssiproto.h"
#include "OtherWssi/globalsettings.h"

// wssicomp - comparison routine for wssindex sort
//					returns -1 if first < second
//								0 if equal		(if sort stabilized, == never happens)
//								1 if first > second

int wssicomp(const unsigned long ptr1,	const unsigned long ptr2)
	{
	int i, fnmatches;
	int j;
	struct filerec *p1, *p2, *p1a, *p2a;
	struct diskrec *vrec_p1a, *vrec_p2a;
	bool inarc1, inarc2;

	p1 = p1a = Dr + ptr1;
	p2 = p2a = Dr + ptr2;

	if((inarc1 = inArc(p1)))
		p1a = Dr + p1->Subdirno;
	if((inarc2 = inArc(p2)))
		p2a = Dr + p2->Subdirno;

	for(i=fnmatches=0; i<Nkey; i++)
		{
// This first switch makes sure that the .ID dummy entries sort first
//	when the key is any of the file name/extension keys.
		switch(SortKey[i])
			{
			case SortKey_FN:			// file name
			case SortKey_EXT:		// file extension
			case SortKey_ARCNAM:	// archive file name
			case SortKey_ARCEXT:	// archive file extension
				if(isDotID(p1))
					{
					if(isDotID(p2))
						continue;
					else
						return(-1);
					}
// can't get here unless p1 is not a .ID entry
				if(isDotID(p2))
					return(1);
			}

		switch(SortKey[i])
			{
			case SortKey_FN:	// file name
				if(2 == fnmatches)	// true if archive name and extension have matched
					{
					fnmatches++;
					if(inArc(p2) && !inArc(p1) && (ptr1 == p2->Subdirno))
						return(-1);
					if(inArc(p1) && !inArc(p2)	&& (ptr2 == p1->Subdirno))
						return(1);
					}
				if(0 == (j = namecomp(p1, p2, COMPARE_FILENAME, COMPARE_NOMORE)))
					continue;
				else
					return(j*Sortdir[i]);

			case SortKey_EXT:	// file extension
				if(2 == fnmatches)
					{
					fnmatches++;
					if(inArc(p2) && !inArc(p1)	&& (ptr1 == p2->Subdirno))
						return(-1);
					if(inArc(p1) && !inArc(p2) && (ptr2 == p1->Subdirno))
						return(1);
					}
				if(0 == (j = namecomp(p1, p2, COMPARE_EXTENSION, COMPARE_NOMORE)))
					continue;
				else
					return(j*Sortdir[i]);

			case SortKey_DISK:	// disk name
				vrec_p1a = Vrec + p1a->Volno;
				vrec_p2a = Vrec + p2a->Volno;
				if((j = qstringCompare(vrec_p1a->VolName, vrec_p2a->VolName)) == 0)
					continue;
				else
					return(j*Sortdir[i]);

// Never used
//			case SortKey_DISKNO:	// numeric part of disk name
//				vrec_p1a = Vrec + p1a->Volno;
//				vrec_p2a = Vrec + p2a->Volno;
//				lj = numberKey(vrec_p1a->VolName)-numberKey(vrec_p2a->VolName);
//				if(lj > 0) return (Sortdir[i]);
//				if(lj < 0) return (-Sortdir[i]);
//				continue;

			case SortKey_SUB:	// subdirectory name
				if((j = qstringCompare(SubDir[p1a->Subdirno], SubDir[p2a->Subdirno]))==0)
					continue;
				else
					return(j*Sortdir[i]);

			case SortKey_SIZE:	// file size
				if(p1->Dsize == p2->Dsize)
					continue;
				if(p1->Dsize > p2->Dsize)
					return(Sortdir[i]);
				else
					return(-Sortdir[i]);

			case SortKey_TTAG:	// date and time
				if(p1->Ddate > p2->Ddate)
					return(Sortdir[i]);
				if(p1->Ddate == p2->Ddate)
					{
					if(p1->Dtime > p2->Dtime)
						return(Sortdir[i]);
					else
						return(-Sortdir[i]);
					}
				else
					return(-Sortdir[i]);

			case SortKey_CAT:	// category, case insensitive
				if(p1->Category.isEmpty())
					{
					if(p2->Category.isEmpty())
						continue;
					else
						return(-Sortdir[i]);
					}
				if(p2->Category.isEmpty())
					return(Sortdir[i]);
				if((j = qstringCompare(p1->Category, p2->Category)) == 0)
					continue;
				else
					return(j*Sortdir[i]);

			case SortKey_COM:	// comment, case insensitive
				if(p1->Comment.isEmpty())
					{
					if(p2->Comment.isEmpty())
						continue;
					else
						return(-Sortdir[i]);
					}
				if(p2->Comment.isEmpty())
					return(Sortdir[i]);
				if((j = qstringCompare(p1->Comment, p2->Comment)) == 0)
					continue;
				else
					return(j*Sortdir[i]);

			case SortKey_EXTRA:	// extra info, case insensitive
				if(p1->ExtraInfo.isEmpty())
					{
					if(p2->ExtraInfo.isEmpty())
						continue;
					else
						return(-Sortdir[i]);
					}
				if(p2->ExtraInfo.isEmpty())
					return(Sortdir[i]);
				if((j = qstringCompare(p1->ExtraInfo, p2->ExtraInfo)) == 0)
					continue;
				else
					return(j*Sortdir[i]);

			case SortKey_ARCNAM:	// archive file name
// Not in archive < inarchive
// Equal if both not in archive
				if(!inarc1)
					{
					if(!inarc2)
						continue;
					else
						return(-Sortdir[i]);
					}
				if(!inarc2)
					return(Sortdir[i]);
				if(0 == (j = namecomp(p1a, p2a, COMPARE_FILENAME, COMPARE_NOMORE)))
					{
					fnmatches++;
					continue;
					}
				else
					return(j*Sortdir[i]);

			case SortKey_ARCEXT:	// archive extension
				if(!inarc1)
					{
					if(!inarc2)
						continue;
					else
						return(-Sortdir[i]);
					}
				if(!inarc2)
					return(Sortdir[i]);
				if(0 == (j = namecomp(p1a, p2a, COMPARE_EXTENSION, COMPARE_NOMORE)))
					{
					fnmatches++;
					continue;
					}
				else
					return(j*Sortdir[i]);

			case SORTDIM:	// key disk first
				if(p1a->Volno == Keydisk)
					{
					if(p2a->Volno == Keydisk)
						return(0);
					else
						return(-1);
					}
				else
					{
					if(p2a->Volno == Keydisk)
						return(1);
					else
						continue;
					}
			}
		}
// Items compare as equal
	return(0);
	}
