// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "OtherWssi/wssiproto.h"
#include "Dialogs/vollistwindow.h"
#include "string.h"

// wssvcomp - sort key routine for wssindex volume sort
//	In all cases except removing duplicates, sort by volume name if primary
//	keys match.

int wssvcomp(const unsigned long ptr1, const unsigned long ptr2)
	{
	qint64	dif64;
	long		dif32;
	int i;

	struct diskrec	*vrec_ptr1 = Vrec + ptr1;
	struct diskrec	*vrec_ptr2 = Vrec + ptr2;

	switch(Vsrtkey)
		{
		case SORT_VOL_NAME:
			return(Vsrtdir*collator->compare(vrec_ptr1->VolName, vrec_ptr2->VolName));

		case SORT_VOL_USED:
			dif64 = static_cast<qint64>(vrec_ptr1->Vbytes - vrec_ptr2->Vbytes);
			if(dif64 > 0) return (Vsrtdir);
			if(dif64 < 0) return (-Vsrtdir);
			break;

		case SORT_VOL_FREE:   // sort by free space
			dif64 = static_cast<qint64>(vrec_ptr1->Vfree - vrec_ptr2->Vfree);
			if(dif64 > 0) return (Vsrtdir);
			if(dif64 < 0) return (-Vsrtdir);
			break;

		case SORT_VOL_FILES:
			dif32 = static_cast<long>(vrec_ptr1->Vfiles - vrec_ptr2->Vfiles);
			if(dif32 > 0) return (Vsrtdir);
			if(dif32 < 0) return (-Vsrtdir);
			break;

		case SORT_VOL_SUBS:
			dif32 = static_cast<long>(vrec_ptr1->Vsubdir - vrec_ptr2->Vsubdir);
			if(dif32 > 0) return (Vsrtdir);
			if(dif32 < 0) return (-Vsrtdir);
			break;

		case SORT_VOL_DATE_INDEXED:
			if(vrec_ptr1->Vdate > vrec_ptr2->Vdate)
				return Vsrtdir;
			if(vrec_ptr1->Vdate < vrec_ptr2->Vdate)
				return -Vsrtdir;
			break;

// If there is no id record to hold a comment or category, volidnum
// will have been set to Diskno.  No comment and no comment possible
// compare as equal.  Otherwise, no comment possible compares as less
// than any comment (for ascending order).  Similar treatment for category.
		case SORT_VOL_COMMENT:
			if(Diskno == volidnum[ptr1])
				{
				if(Diskno == volidnum[ptr2])
					break;
				if(!hasComment(Dr+volidnum[ptr2]))
					break;
				return Vsrtdir;
				}
			if(Diskno == volidnum[ptr2])
				{
				if(!hasComment(Dr+volidnum[ptr1]))
					break;
				return -Vsrtdir;
				}
			return(Vsrtdir*collator->compare(Dr[volidnum[ptr1]].Comment, Dr[volidnum[ptr2]].Comment));

		case SORT_VOL_CATEGORY:
			if(Diskno == volidnum[ptr1])
				{
				if(Diskno == volidnum[ptr2])
					break;
				if(!hasCategory(Dr+volidnum[ptr2]))
					break;
				return Vsrtdir;
				}
			if(Diskno == volidnum[ptr2])
				{
				if(!hasCategory(Dr+volidnum[ptr1]))
					break;
				return -Vsrtdir;
				}
			return(Vsrtdir*collator->compare(Dr[volidnum[ptr1]].Category, Dr[volidnum[ptr2]].Category));

		case SORT_VOL_EXTRA_INFO:
			if(Diskno == volidnum[ptr1])
				{
				if(Diskno == volidnum[ptr2])
					break;
				if(!hasExtraInfo(Dr+volidnum[ptr2]))
					break;
				return Vsrtdir;
				}
			if(Diskno == volidnum[ptr2])
				{
				if(!hasExtraInfo(Dr+volidnum[ptr1]))
					break;
				return -Vsrtdir;
				}
			return(Vsrtdir*collator->compare(Dr[volidnum[ptr1]].ExtraInfo, Dr[volidnum[ptr2]].ExtraInfo));

// Sort key for duplicate removal.  When duplicates are found, the one which
//	sorts second is the one deleted.
//		1. volume name
//		2. date indexed, most recent first
//		3. number of files on disk, largest first
//		4. unsorted order in database
		case SORT_VOL_DUPS:
			if(0 != (i = qstringCompare(vrec_ptr1->VolName, vrec_ptr2->VolName)))
				return(i);
			if(vrec_ptr2->Vdate > vrec_ptr1->Vdate)
				return 1;
			if(vrec_ptr2->Vdate < vrec_ptr1->Vdate)
				return -1;
			if(0L == (dif32 = (static_cast<long>(vrec_ptr2->Vfiles)) - (static_cast<long>(vrec_ptr1->Vfiles))))
				dif32 = static_cast<long>(ptr1 - ptr2);
			return(dif32>0);
		}
	return 0;
	}
