// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "OtherWssi/globalsettings.h"
#include "Dialogs/mainwindow.h"
#include <exception>

// incfile - increment number of files in database and check against maximum.
// If new value of Nofile is equal to (or greater than) Maxfiles, allocate
// additional memory.  If no files are added after this, the additional memory
// allocation wasn't necessary.

bool MainWindow::incfile_and_check_for_error(bool isSubOrVol)
	{
	if(Nofile+1 >= Maxfiles)
		{
		if(increase_max_filecount_to(no_overflow_increment(Maxfiles, static_cast<unsigned long>(Maxfiles_increment), MaxfilesIncMultiplier)))
			return true;
		MaxfilesIncMultiplier *= 2;
		}
	Nofile++;
	if(!isSubOrVol)
		NoPlainFiles++;
	return false;
	}

// Increase maximum number of files to specified value
// Returns TRUE if memory allocation failed, FALSE on success
bool MainWindow::increase_max_filecount_to(unsigned long newmax)
	{
// This routine cannot decrease Maxfiles
	if(newmax <= Maxfiles)
		return false;

	struct filerec *dr_new = nullptr;
	unsigned long *sortindx_new = nullptr;

	try {
#if !defined(VER_64_BIT)
		if(newmax * static_cast<qint64>(sizeof(struct filerec)) > 0xfffffff)
			{
			QMessageBox::critical(this, tr("Error"),
				tr("Unable to set maximum number of files to %L1 because this would require more than 4GB of memory").arg(newmax));
			return true;
			}
#endif
		dr_new = new struct filerec[newmax];
		sortindx_new = new unsigned long[newmax];
		}
	catch(std::bad_alloc&)
		{
		delete [] dr_new;
		delete [] sortindx_new;
		QMessageBox::critical(this, tr("Error"),
			tr("Not enough memory to set maximum number of files to %L1, using %L2").arg(newmax).arg(Maxfiles));
		return true;
		}

	for(unsigned int i=0; i<Maxfiles; i++)
		dr_new[i] = Dr[i];
	delete [] Dr;
	Dr = dr_new;

	for(unsigned int i=0; i<Maxfiles; i++)
		sortindx_new[i] = Sortindx[i];
	delete [] Sortindx;
	Sortindx = sortindx_new;
	Maxfiles = newmax;
	return false;
	}

// incsub - increment number of subdirs in database and check against maximum.
bool MainWindow::incsub_and_check_for_error(void)
	{
	if(Subno+1 >= Maxsubs)
		{
		if(increase_max_subcount_to(no_overflow_increment(Maxsubs, static_cast<unsigned long>(Maxsubs_increment), MaxsubsIncMultiplier)))
			return true;
		MaxsubsIncMultiplier *= 2;
		}
	Subno++;
	return false;
	}

// Increase maximum number of subdirectories to specified value
// Returns TRUE if memory allocation failed, FALSE on success
bool MainWindow::increase_max_subcount_to(unsigned long newmax)
	{
// This routine cannot decrease Maxsubs
	if(newmax <= Maxsubs)
		return false;

	QString *SubDir_new = nullptr;
	unsigned long *SubDisk_new = nullptr;

	try {
#if !defined(VER_64_BIT)
		if(newmax * static_cast<qint64>(sizeof(QString)) > 0xfffffff)
			{
			QMessageBox::critical(this, tr("Error"),
				tr("Unable to set maximum number of subdirectories to %L1 because this would require more than 4GB of memory").arg(newmax));
			return true;
			}
#endif
		SubDir_new = new QString[newmax];
		SubDisk_new = new unsigned long[newmax];
		}
	catch(std::bad_alloc&)
		{
		delete [] SubDir_new;
		delete [] SubDisk_new;
		QMessageBox::critical(this, tr("Error"),
			tr("Not enough memory to set maximum number of subdirectories to %L1, using %L2").arg(newmax).arg(Maxsubs));
		return true;
		}

	for(unsigned int i=0; i<Maxsubs; i++)
		SubDir_new[i] = SubDir[i];
	delete [] SubDir;
	SubDir = SubDir_new;

	for(unsigned int i=0; i<Maxsubs; i++)
		SubDisk_new[i] = SubDisk[i];
	delete [] SubDisk;
	SubDisk = SubDisk_new;
	Maxsubs = newmax;
	return false;
	}

// incdisk - check whether number of disks in database can be incremented,
//	but don't do the increment!

bool MainWindow::incdisk_error_check(void)
	{
	if(Diskno >= Maxdisks)		// Diskno not incremented!
		{
		if(increase_max_diskcount_to(no_overflow_increment(Maxdisks, static_cast<unsigned long>(Maxdisks_increment), MaxdisksIncMultiplier)))
			return true;
		MaxdisksIncMultiplier *= 2;
		}
	return false;
	}

// Increase maximum number of disks to specified value
// Returns TRUE if memory allocation failed, FALSE on success
bool MainWindow::increase_max_diskcount_to(unsigned long newmax)
	{
// This routine cannot decrease Maxdisks
	if(newmax <= Maxdisks)
		return false;

	struct diskrec *vrec_new = nullptr;

	try {
#if !defined(VER_64_BIT)
		if(newmax * static_cast<qint64>(sizeof(struct diskrec)) > 0xfffffff)
			{
			QMessageBox::critical(this, tr("Error"),
				tr("Unable to set maximum number of disks to %L1 because this would require more than 4GB of memory").arg(newmax));
			return true;
			}
#endif
		vrec_new = new struct diskrec[newmax];
		}
	catch(std::bad_alloc&)
		{
		delete [] vrec_new;
		QMessageBox::critical(this, tr("Error"),
			tr("Not enough memory to set maximum number of disks to %L1, using %L2").arg(newmax).arg(Maxdisks));
		return true;
		}

	for(unsigned int i=0; i<Maxdisks; i++)
		vrec_new[i] = Vrec[i];
	delete [] Vrec;
	Vrec = vrec_new;
	Maxdisks = newmax;
	return false;
	}

// returns val + inc*mult unless this would overflow a 32-bit signed integer,
// in which case it returns 0x7fffffff
unsigned long MainWindow::no_overflow_increment(unsigned long val, unsigned long inc, unsigned long mult)
	{
	unsigned long long val64 = val;
	unsigned long long inc64 = inc;
	unsigned long long mult64 = mult;
	unsigned long long result = val64 + inc64*mult64;
	if(result > 0x7fffffffUL)
		result = 0x7fffffffUL;
	return static_cast<unsigned long>(result);
	}
