// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/pinlist.h"

#include <QtDebug>

// Create empty list and set maximum size
pinList::pinList(int maxcount)
	{
	strList.clear();
	listSize = maxcount;
	}

pinList::~pinList()
	{
	}

// Return number of strings in list
int pinList::size()
	{
	return strList.size();
	}

// Change the maximum number of strings, possibly deleting some at end
void pinList::resize(int newsize)
	{
	if(newsize >= listSize)
		{
		listSize = newsize;
		return;
		}
	while(strList.size() > newsize)
		{
		strList.removeLast();
		pinState.removeLast();
		}
	listSize = newsize;
	}

// Pinning an item moves it ahead of all unpinned items
void pinList::pin(int itemNumber)
	{
	if(itemNumber < strList.size())
		{
		pinState[itemNumber] = true;
		reorder();
		}
	}

// Unpinning an item moves it after of all pinned items
void pinList::unpin(int itemNumber)
	{
	if(itemNumber < strList.size())
		{
		pinState[itemNumber] = false;
		reorder();
		}
	}

// Change the pin state of an item and reorder list accordingly
void pinList::togglePin(int itemNumber)
	{
	if(itemNumber < strList.size())
		{
		pinState[itemNumber] = !pinState[itemNumber];
		reorder();
		}
	}

// Find str in the list and move it as high as is consistent with
// its pin state.  Use case-insensitive comparison.
// Return true if str was in list, otherwise do nothing and return false.
bool pinList::moveTop(QString str)
	{
	int oldPos = -1;
	for(int i=0; i<strList.size(); i++)
		{
		if(0 == str.compare(strList[i], Qt::CaseInsensitive))
			{
			oldPos = i;
			break;
			}
		}
	if(oldPos < 0)
		return false;
	if(pinState[oldPos])
		{
		strList.move(oldPos, 0);
		pinState.move(oldPos, 0);
		return true;
		}

// str is not pinned, find where to move it
	int lastPinned = -1;
	for(int i=strList.size()-1; i>=0; i--)
		{
		if(pinState[i])
			{
			lastPinned = i;
			break;
			}
		}
	strList.move(oldPos, lastPinned+1);
	pinState.move(oldPos, lastPinned+1);

	return true;
	}

// Reorder list so all pinned items are before unpinned items.
// This is a very short list, so a recursive bubble sort suffices.
// (It's not even a good implementation of bubble sort...)
void pinList::reorder()
	{
	if(strList.size() < 2)
		return;
	bool anotherPass = false;
	for(int i=0; i<strList.size()-1; i++)
		{
		if(pinState[i+1] && !pinState[i])
			{
			strList.move(i+1, i);
			pinState.move(i+1, i);
			anotherPass = true;
			}
		}
	if(anotherPass)
		reorder();
	}

bool pinList::isDuplicate(QString str)
	{
	return strList.indexOf(QRegExp(str, Qt::CaseInsensitive))>=0;
	}

// Add unpinned item after last pinned item, before first unpinned item
// If list is already at maximum length last unpinned item is lost.
// If list is at maximum length and everything is pinned, nothing is added.
// If already in list, move up as high as consistent with pin state.
void pinList::add(QString str)
	{
	if(isDuplicate(str))
		{
		moveTop(str);
		return;
		}
	int firstUnpinned = strList.size();
	for(int i=0; i<strList.size(); i++)
		{
		if(!pinState[i])
			{
			firstUnpinned = i;
			break;
			}
		}
	if(firstUnpinned < listSize)
		{
		strList.insert(firstUnpinned, str);
		pinState.insert(firstUnpinned, false);
		if(strList.size() > listSize)
			{
			strList.removeLast();
			pinState.removeLast();
			}
//		return firstUnpinned;
		return;
		}
//	return -1;
	return;
	}

// Add item to list pinned and after last pinned item if ispinned is true,
// otherwise at very end and not pinned.  Do nothing if this is a duplicate.
void pinList::addEnd(QString str, bool ispinned)
	{
	if(isDuplicate(str))
		return;

	int insertionPoint = strList.size();
	if(ispinned)
		{
		for(int i=0; i<strList.size(); i++)
			{
			if(!pinState[i])
				{
				insertionPoint = i;
				break;
				}
			}
		}
	strList.insert(insertionPoint, str);
	pinState.insert(insertionPoint, ispinned);
	if(strList.size() > listSize)
		{
		strList.removeLast();
		pinState.removeLast();
		}
	}

// Return pin state for an item, or false if out of bounds
bool pinList::pinned(int itemNumber)
	{
	if(itemNumber < strList.size())
		return pinState[itemNumber];
	return false;
	}

// Return text for an item, or an empty string if out of bounds
QString pinList::text(int itemNumber)
	{
	if(itemNumber < strList.size())
		return strList[itemNumber];
	return QString();
	}
