// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#ifndef WSSINDEX_H
#define WSSINDEX_H

#ifdef EXTERN
	#define INIT_VALUE(x) =x
#else
	#define INIT_VALUE(x)
	#define EXTERN extern
#endif

#include <QDateTime>
#include <QCollator>
#include <QStyle>

// If building old version for XP, Q_FALLTHROUGH() was not defined before Qt 5.8
#if QT_VERSION < QT_VERSION_CHECK(5, 8, 0)
	#define Q_FALLTHROUGH()
#endif

class MainWindow;

#define QFA(x)		QString::fromLatin1(x)
//#define QL1(x)		QLatin1String(x)
#define MENU_NO_CHECKBOX	"QMenu::indicator {width: 1px;}"

#define MAXROWS	4
#define MAXCOLS	10
#define NFIELDS	10

#define MAX_VOLCHARS	255
#define MAX_CATLEN	64							// maximum length of a category

#define EOFCHAR	0xff	// result of assigning EOF to a character variable
#define EOS '\0'	// standard end of string

// macro to test for error or end of file
#define WSSERROR_OR_EOF(fp)	(ferror(fp) | feof(fp))

// Diskbits flags for struct filerec included in disk file (new with version 6.1)
#define HAS_COMMENT				1
#define HAS_CATEGORY				2
#define IS_DOT_ID					4
#define HAS_LFN					8
#define IN_ARC						16
#define HAS_EXTRA_INFO			32

// Transient bit flags for struct filerec
#define PREVIOUSLY_SELECTED	1
#define MARK_TAGGED				2
#define IN_TAGGED_ARC			4
#define IN_TAGGED_SUB			8
#define ON_TAGGED_DISK			16
#define TAGGED_ANY_REASON		30
#define PREVIOUSLY_CATALOGED	32
#define CATEGORY_EDITED			64
#define COMMENT_EDITED			128
#define VOLUME_EDITED			256
#define EXTRA_EDITED				512
#define ANY_EDITED				(CATEGORY_EDITED | COMMENT_EDITED | VOLUME_EDITED | EXTRA_EDITED)

#define hasComment(x)			(HAS_COMMENT & (x)->Diskbits)
#define hasNoComment(x)			(!(HAS_COMMENT & (x)->Diskbits))
#define hasCategory(x)			(HAS_CATEGORY & (x)->Diskbits)
#define hasNoCategory(x)		(!(HAS_CATEGORY & (x)->Diskbits))
#define hasExtraInfo(x)			(HAS_EXTRA_INFO & (x)->Diskbits)
#define hasNoExtraInfo(x)		(!(HAS_EXTRA_INFO & (x)->Diskbits))
#define isDotID(x)				(IS_DOT_ID & (x)->Diskbits)
#define hasLFN(x)					(HAS_LFN & (x)->Diskbits)
#define inArc(x)					(IN_ARC & (x)->Diskbits)

#define categoryEdited(x)		(CATEGORY_EDITED & (x)->Bitflags)
#define commentEdited(x)		(COMMENT_EDITED & (x)->Bitflags)
#define extraEdited(x)			(EXTRA_EDITED & (x)->Bitflags)
#define volumeEdited(x)			(VOLUME_EDITED & (x)->Bitflags)
#define anyEdited(x)				(ANY_EDITED & (x)->Bitflags)

#define setCategoryEdited(x)		(x)->Bitflags |= CATEGORY_EDITED
#define setCategoryNotEdited(x)	(x)->Bitflags &= ~CATEGORY_EDITED
#define setCommentEdited(x)		(x)->Bitflags |= COMMENT_EDITED
#define setCommentNotEdited(x)	(x)->Bitflags &= ~COMMENT_EDITED
#define setExtraEdited(x)			(x)->Bitflags |= EXTRA_EDITED
#define setExtraNotEdited(x)		(x)->Bitflags &= ~EXTRA_EDITED
#define setVolEdited(x)				(x)->Bitflags |= VOLUME_EDITED
#define setVolNotEdited(x)			(x)->Bitflags &= ~VOLUME_EDITED
#define setExtraEdited(x)			(x)->Bitflags |= EXTRA_EDITED
#define setExtraNotEdited(x)		(x)->Bitflags &= ~EXTRA_EDITED

#define setHasComment(x)			(x)->Diskbits |= HAS_COMMENT
#define setHasNoComment(x)			(x)->Diskbits &= ~HAS_COMMENT
#define setHasCategory(x)			(x)->Diskbits |= HAS_CATEGORY
#define setHasNoCategory(x)		(x)->Diskbits &= ~HAS_CATEGORY
#define setHasExtra(x)				(x)->Diskbits |= HAS_EXTRA_INFO
#define setHasNoExtra(x)			(x)->Diskbits &= ~HAS_EXTRA_INFO
#define setIsDotID(x)				(x)->Diskbits |= IS_DOT_ID
#define setHasLFN(x)					(x)->Diskbits |= HAS_LFN
#define setInArc(x)					(x)->Diskbits |= IN_ARC

#define MAX_SEL	38			// largest user-entered file selection number
// dimension for file & disk selection arrays
#define SEL_DIM (MAX_SEL+1)

// File selection values.
// Commented out values were options in the DOS version that are no longer implemented
#define SEL_FILENAME				1
#define SEL_SUBDIR				2
#define SEL_VOLUME				3
//#define SEL_FROM_FILENAME		4
//#define SEL_FROM_EXTENSION		5
//#define SEL_FROM_SUBDIR			6
//#define SEL_FROM_VOLUME			7
#define SEL_VOL_NUM_LOW_LIMIT	8
#define SEL_VOL_NUM_HI_LIMIT	9
#define SEL_COMMENT				10
#define SEL_CATEGORY				11
#define SEL_EXTRA_INFO			12
#define SEL_DATE_LOW_LIMIT		13
#define SEL_DATE_HI_LIMIT		14
#define SEL_SIZE_LOW_LIMIT		15
#define SEL_SIZE_HI_LIMIT		16
//#define SEL_CONF_EXCEPTIONS	17

#define SEL_ZIPPY					18

#define SEL_ARC_FILES			19
#define SEL_ARC_AND_MEMB		20
#define SEL_ARCMEM_BY_ARCNAME	21

#define SEL_FILENAME_NODIACRIT	22
#define SEL_SUBDIR_NODIACRIT		23
#define SEL_VOLUME_NODIACRIT		24
#define SEL_COMMENT_NODIACRIT		25
#define SEL_CATEGORY_NODIACRIT	26
#define SEL_EXTRA_INFO_NODIACRIT	27
#define SEL_ZIPPY_NODIACRIT		28

#define SEL_SELF_EXTRACTING	29
#define SEL_FONT					30
#define SEL_GRAPHICS				31
#define SEL_MOVIES				32
#define SEL_SOUND					33
//#define SEL_TAGGED				34
//#define SEL_NOT_TAGGED			35

#define SEL_FROM_PREVIOUS		36
#define SEL_ADD_TO_PREVIOUS	37
#define SEL_REM_FROM_PREVIOUS	38

#define REJECT		1
#define ACCEPT		0

#define CASE_PRESERVE	0
#define CASE_FORCE_UP	1
#define CASE_FORCE_LOW	2

#define NO_DIZ		0
#define DIZ_SELF	1
#define DIZ_ZIP	2

#define COMPARE_NOMORE		0
#define COMPARE_FILENAME	1
#define COMPARE_EXTENSION	2
#define COMPARE_BOTH			3

#define RESIZING_DELAY_MS	100

EXTERN MainWindow *mainWindowPtr;

struct filerec
	{
	quint64			Dsize;			// file size (uncompressed for archives)
	QDate				Ddate;			// dos date stamp
	QTime				Dtime;			// dos time stamp
	unsigned long	Volno;			// volume number, or if IN_ARC bit is set,
											//	entry number for containing archive
	unsigned long	Subdirno;		// subdirectory number, 0 for root
	QString			Comment;			// comment or empty qstring
	QString			Category;		// category or empty qstring
	QString			ExtraInfo;		// extra information or empty qstring
	QString			Lfn;				// long file name
	unsigned short	Diskbits;		// flags which are written to disk when
											//	database is saved
	unsigned short	Bitflags;		// transient flags (see defines above)
	unsigned short	ExtOffset;		// Extension starts at Lfn[ExtOffset].  0 means
											//	no extension.  Not saved with database.
	};

struct diskrec
	{
	quint64				Vbytes;			// disk capacity (may be inaccurate for CD/DVD)
	quint64				Vfree;			// free bytes (0 for CD/DVD)
	QString				VolName;		// disk label
	unsigned long		Vfiles;			// number of files (1 for dummy indexing),
												//	not decremented by database editing
	unsigned long		Vsubdir;			// number of subdirectories other than root
												//	(0 for dumming indexing), not decremented
												//	by database editing
	QDate					Vdate;			// date indexed
	unsigned long		firstOnDisk;	// First file record for this disk.  If there is a
												// dotID record, this will be it. Not saved with database.
	};

EXTERN struct filerec *Dr;
EXTERN struct diskrec *Vrec;

struct listOfFiles
	{
	QString Fnptr;
	struct listOfFiles *NextList;
	};

EXTERN QString
	MatchName,
	ZipName;

EXTERN QString *SubDir;			// array of subdirectory names
EXTERN unsigned long *SubDisk;	// array of disk numbers for subdirectory names
EXTERN unsigned long *Sortindx;	// array of subscripts for sorted files.  High
											// bit was set to mark first on page in DOS
											// version to make paging backwards easier.
EXTERN unsigned long *volidnum;

EXTERN unsigned long
	Diskno,			// number of disks in database
	Keydisk,			// disk being checked for backups
	Maxdisks,		// current maximum number of disks in database
	MaxdisksIncMultiplier,	// current multiplier for Maxdisks_increment
	Maxfiles,		// current maximum number of files in database
	MaxfilesIncMultiplier,	// current multiplier for Maxfiles_increment
	Maxsubs,			// current maximum number of subdirs in database
	MaxsubsIncMultiplier,	// current multiplier for Maxsubs_increment
	Nofile,			// number of files in database (includes subdir and volume records)
	NoPlainFiles,	// number of file in database excluding subdir and volume records
	Noselect,		// number of files selected
	StartDiskNum,	// lower limit for selection by number in disk label
	StopDiskNum,		// upper limit for selection by number in disk label
	Subno;			// number of subdirectories in database

#define SortKey_FN		1		// file name
#define SortKey_EXT		2		// file extension
#define SortKey_DISK	3		// disk volume label
//#define SortKey_DISKNO	4		// numeric part of disk label
#define SortKey_SUB		4		// subdirectory name
#define SortKey_SIZE	5		// file size
#define SortKey_TTAG	6		// data and time
#define SortKey_CAT		7		// category
#define SortKey_COM		8		// comment
#define SortKey_EXTRA	9		// extra info
#define SortKey_ARCNAM	10		// archive file name
#define SortKey_ARCEXT	11		// archive file extension

#define MAXSORT	11				// largest user-entered file sort key
#define SORTDIM	(MAXSORT+1)	// sort key used in backup check

EXTERN QDateTime
	Compd1,			// lower limit date for selecting
	Compd2;			// upper limit date for selecting

EXTERN int
	Nkey,						// number of sort keys
	Nselect,					// number of selection options
	Sel[SEL_DIM],
	Sortdir[SORTDIM],		// list of sort directions, 1-->ascending, -1 desc.
	SortKey[SORTDIM],	// list of sort keys
	Vsrtdir,					// 1 for ascending volume sorts, -1 for descending
	Vsrtkey;					// sort key for disk volume listings

EXTERN bool
	Changed,					// true if database has been edited or disks indexed
	InhibitQuit,			// true if indexing in progress and quit should be ignored
	MessageLogWindowOpen, // true if message log window is open
	NoId,					// don't print vol, subdir entries
	Usesel[SEL_DIM];		// array indicating which selection options to use

EXTERN QString
	Comparc,					// archive name to compare against when selecting
	Compcat,					// category to compare against when selecting
	Compcom,					// comment to compare against when selecting
	Compextra,				// extra info to compare against when selecting
	Compdsk,					// disk name to compare against when selecting
	Compsub,					// subdirectory name to compare against when selecting
	ShortDateFormat,		// used for date display format
	StartDisk,				// vol to compare against when selecting files
	StartExt,				// file ext to compare against when selecting files
	StartFile,				// file name to compare against when selecting files
	StartSubdir;			// subdir to compare against when selecting files

EXTERN quint64
	Compsiz1,
	Compsiz2;

EXTERN int (*qstringCompareFunction)(const QString &, const QString &);

EXTERN QCollator
	*collator,				// Case sensitivity and numeric mode are configurable
	*simpleCollator;		// This one is case insensitive and numeric mode is off

EXTERN QStyle
	*menuBarStyle,
	*XPStyle,
	*VistaStyle;

#endif // WSSINDEX_H
